/* P1_ADDRESS.C  910415  */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"

/* ===========================================================================

   function name: address_
   input	       : ---
   output	    : ---
   return value : SUCCESS	- if there is no error detected
		            FAIL		- otherwise, if an error detected

   global vars  : token_type 	- type of current token
 		            token_buf	- current token string
		            symbol_table	- symbol table data structure
		            debug		- debugging statement flag

   description	 : This routine parses the indirect address assignment
		            statement.

			ADDRESS <-  @scalar_exp:type:length[log_var] = LOG_EXP;


		  Scalar_exp is a scalar expression. Type is either int, real
		  or logical. Length is the number of bits being used to assign
		  the right-hand-side value to. Log_var is either a logical
		  expression or '$'.

=========================================================================== */

int address_()
{
   int lhs_type,	 /* left-hand-side type : int, real or logical */
       lhs_length, /* bit width of left-hand-side */
       lhs_index,	 /* address of the left-hand-side */
       lhs_offset, /* index of offset of lhs id */
       subscr_type,/* type of left-hand-side subscript:scalar or parallel*/
                   /* this variable is no longer set by subscript and
			             may not be reliable */
       rhs_index,	  /* right-hand-side value */
       rhs_offset,  /* offset of right hand side */
       return_value;/* value being returned to the calling routine */

   char offset_width[10]; /* character string of the offset width */
/* char lhs_width[10];*/	  /* character string of left-hand-side width */


   if (debug == 2)
      printf ("In address_() .....................................\n");

   return_value = SUCCESS;

if(prsadd(&lhs_type,&lhs_length,&lhs_index,&lhs_offset,&subscr_type) != SUCCESS)
 {if (return_value != FAIL)
   {if (token_type != T_ASSIGN)
      {return_value = FAIL;
       print_msg(ASGNOP_ERR, token_buf);}
    else
      {token_type = get_token();
       if (log_exp(&rhs_index,&rhs_offset) == FAIL)
			 return_value = FAIL;
       else if (symbol_table[rhs_index].mode2 == T_PARALLEL &&
		          subscr_type == T_SCALAR)
         {print_msg (ASGNP2S_ERR, NULL);
          return_value = FAIL;}
       else
         {if (symbol_table[rhs_index].mode1 == T_INT && lhs_type == T_REAL)
             float_(&rhs_index,&rhs_offset);
          if (symbol_table[rhs_index].mode1 == T_REAL && lhs_type == T_INT)
	          fix_(&rhs_index,&rhs_offset);
          if (symbol_table[rhs_index].mode1 != lhs_type)
            {print_msg (ASGNCON_ERR, NULL);
             return_value = FAIL;}
	       else
            {pc_itoa((symbol_table[rhs_index].width - lhs_length),offset_width);
             if (subscr_type == T_PARALLEL)
               {if (symbol_table[rhs_index].mode2 == T_SCALAR)
                  {print_opcode("mvid_",T_MVID, T_NONE, T_SCALAR);
		             print_args(off_exp1(rhs_index,rhs_offset),
				                  off_exp2(lhs_index,lhs_offset),
				                  lhs_length, NULL);}
		          else
                  {print_opcode("mvid_",T_MVID, T_NONE, T_PARALLEL);
		             print_args(off_exp1(rhs_index,rhs_offset),
			                   	off_exp2(lhs_index,lhs_offset),
				                  lhs_length, NULL);}
	            }
	          else
	            {print_opcode("mvid_",T_MVID, T_NONE, T_SCALAR);
		          print_args(symbol_table[lhs_index].tokpts,
			                  off_exp2(rhs_index,rhs_offset),
			                  off_exp3(lhs_index,lhs_offset),
			                  NULL, NULL);}

	          if (token_type != T_EOS)
	            {return_value = FAIL;
		          print_msg(EOS_ERR, token_buf);}
             else
                token_type = get_token();
	         }
	      }
      }
   }
 }
   if (debug == 2)
      printf("address() returning return_value : %d\n", return_value);

   return(return_value);

} /* end of address() */


prsadd(lhs_type,lhs_length,lhs_index,lhs_offset,subscr_type)
int *lhs_type,*lhs_length,*lhs_index,*lhs_offset,*subscr_type;
{
   int return_value,offset_index,offset_offset;
	char lhs_width[50];

   printf("In prsadd, token_buf=%s\n",token_buf);
   return_value = SUCCESS;
   if (element(&lhs_index,&lhs_offset) == FAIL)
     {return_value = FAIL;
      print_msg(ADDR_LHS_ERR, NULL);}
   else if (symbol_table[*lhs_index].mode1 != T_INT &&
            symbol_table[*lhs_index].mode2 != T_SCALAR)
     {return_value = FAIL;
      print_msg(ADDR_SCA_ERR, NULL);}
   else if (token_type != T_COLON)
     {return_value = FAIL;
      print_msg(ADDR_COL_ERR, token_buf);}
   else
	  {token_type = get_token();
      if (token_type != T_INT && token_type != T_REAL &&
          token_type != T_LOGICAL)
        {return_value = FAIL;
         print_msg(ADDR_TYPE_ERR, token_buf);}
      else
        {*lhs_type = token_type;
         if ((token_type = get_token()) != T_COLON)
           {return_value = FAIL;
            print_msg(ADDR_COL_ERR, token_buf);}
         else if ((token_type = get_token()) != T_DEC_CONST)
           {return_value = FAIL;
            print_msg(ADDR_WIDTH_ERR, token_buf);}
         else
           {*lhs_length = atoi(token_buf);
            strcpy(lhs_width, token_buf);
            if ((token_type = get_token()) != T_LSQU)
	     {return_value = FAIL;
		    print_msg(LSQU_ERR, token_buf);}
            else
              {token_type = get_token();
               if (subscript(lhs_index,&offset_index,&offset_offset) == FAIL)
                 {return_value = FAIL;
		            print_msg(SUBSCR_ERR, NULL);}
               else
                 {*lhs_index = offset_index;
		            *lhs_offset = offset_offset;
		            if (symbol_table[*lhs_index].mode1 != T_LOGICAL &&
		                symbol_table[*lhs_index].mode2 != T_PARALLEL)
                    {return_value = FAIL;
		               print_msg (SUBSCR_ERR, NULL);}
                  else
                    {if (strcmp(symbol_table[*lhs_index].tokpts, "$") == 0)
                       *subscr_type = T_PARALLEL;
		               else
			              *subscr_type = T_SCALAR;
                     if (token_type != T_RSQU)
		                 {return_value = FAIL;
			               print_msg (RSQU_ERR, token_buf);}
                     else
  			              token_type = get_token();
                    }
                 }
             }
         }
      }
   }
 return(return_value);
}






