/* P1_ALLOCATE.C 910415 */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"
/* ===========================================================================

   function name: allocate
   input               : ---
   output           : ---
   return value : return_value  - SUCCESS if no error in parsing the allocate
				- FAIL, otherwise.

   global vars  : token_type    - type or current token
			    token_buf   - current token string
			    symbol_table        - symbol table data structure
			    debug               - debugging statement flag

   description   : This routine is parsing the allocate statement. It will check
			    the right syntax of allocate statement :

				 allocate (alloc_num) index_var in log_par_var;

		      1) it will check the index_var is defined in the symbol
			      table or not and check for the right data
			      type : T_INDEX.
		      2) check the keyword "in"
		      3) check the logical parallel variable syntax.
			      - declared as parallel variable
			      - parallel subscript "[$]" should be occured.
	       - @indirect address - see p1_address
	    4) keyword 'endallocate' should be oocured at the end.
   WARNING      5) this routine used subscript() and may accept
			 "illegal" log_par_var subscripts.

		   If there is an error during the parsing, this routine will
		   return a FAIL value to the calling routine.

This routine generates:
	 T_ALLOC   index_var,log_par_var,count

=========================================================================== */
char *off_exp1();
char *off_exp2();

int allocate_()
{
   int invar_index,
       invar_offset,
       end_index,
       end_offset,
       log_index,
       lpar_flag,
       return_value,
       arg1_index,
       arg1_offset,
		 lhs_type,
		 lhs_length,
		 subscr_type;
   int alloc_num_dx;
   char *end_label;

   if (debug == 2)
      {printf ("In allocate().........................................\n");
      printf ("token_type=%d, token_buf=%s\n",token_type,token_buf);}

   return_value = SUCCESS;

   end_label = make_label ("AL$E", ++if_count);  /* use if count */
   if (token_type == T_DEC_CONST)
     {alloc_num_dx = get_const(token_buf,T_DEC_CONST);
      token_type = get_token(); }
   else if (token_type == T_ID )
     {if ((alloc_num_dx = symtab_fetch(token_size, token_buf)) == FAIL)
	  {print_msg(UNDEF_ERR, token_buf);  return(FAIL);}
      else
	switch(symbol_table[alloc_num_dx].mode1)
	  {case T_INT:
	   case T_LONGINT:
	   case T_OCT:
	   case T_HEX:
	   case T_BIN:
		 token_type = get_token();
		break;
	   case T_CHAR:
	   case T_CARD:
		printf("%d ERROR - %s is an illegal type for allocate\n",linenum,token_buf);
		break;
	   case T_REAL:
		 fix_(&alloc_num_dx,SUCCESS);
		 token_type = get_token();
		break;
	   case T_LOGICAL:
	   case T_INDEX:
	   default:
		   alloc_num_dx = get_const("1",T_DEC_CONST);
	  }
      }
   else
     alloc_num_dx = get_const("1",T_DEC_CONST);

   if (get_index(&invar_index,&invar_offset) == FAIL)
      return_value = FAIL;
   else if (token_type != T_IN)
     {print_msg (IN_ERR, token_buf);
      return_value = FAIL;}
   else if ((token_type = get_token()) == T_LPAR)
     {lpar_flag = TRUE;
      token_type = get_token();}
   else
      lpar_flag = FALSE;

   if (token_type == T_ADDRESS)
     { printf("At T_ADDRESS in p1_Alloc\n");
	  if(prsadd(&lhs_type,&lhs_length,&arg1_index,&arg1_offset,&subscr_type) != SUCCESS)
/*log_index = prsadd();*/
      return_value = FAIL;}
   else if (token_type != T_ID)
     {print_msg (ID_ERR, token_buf);
      return_value = FAIL;}
   else if ((log_index = symtab_fetch(token_size, token_buf)) == FAIL)
     {print_msg (UNDEF_ERR, token_buf);
      return_value = FAIL;}
   else if (symbol_table[log_index].mode2 != T_PARALLEL &&
	    symbol_table[log_index].mode1 != T_LOGICAL)
     {print_msg (LOGPAR_ERR, token_buf);
      return_value = FAIL;}
   else
     {token_type = get_token();
      if(subscript(log_index,&arg1_index,&arg1_offset) == FAIL)
	       return_value = FAIL;}

  if (return_value != FAIL)
   {print_opcode ("alloc_", T_ALLOCATE, T_NONE, T_NONE);
    print_args (off_exp1(invar_index,invar_offset),
		off_exp2(arg1_index,arg1_offset),
		off_exp3(alloc_num_dx,SUCCESS), NULL, NULL);
    do_it(invar_index,invar_offset);

    if (lpar_flag == TRUE)
      {if (token_type != T_RPAR)
	 {return_value = FAIL;
	  print_msg (RPAR_ERR, token_buf); }
       else token_type = get_token();}
    if ((return_value = body()) != FAIL)
      {if (token_type == T_ELSENANY)
	 {token_type = get_token();
	  print_opcode("elsenany_",T_ELSENANY, T_NONE, T_PARALLEL);
	  /* two labels required by endalloc_ */
	  print_args(end_label, end_label, NULL, NULL, NULL);
	  if ((return_value = body()) == FAIL)
	     {return_value = FAIL;
	      print_msg(ALLELSE_ERR, token_buf);}
	 }
       if (token_type != T_ENDALLOCATE)
	 {return_value = FAIL;
	  print_msg(ENDALLOCATE_ERR, token_buf);}
       else
	 {print_opcode("endalloc_",T_ENDALLOCATE, T_NONE, T_NONE);
	  print_args(end_label, NULL, NULL, NULL, NULL);
	  token_type = get_token();
	  if (get_index(&end_index,&end_offset) == FAIL) return_value = FAIL;
	  else
	    {if (end_index != invar_index)
		    {print_msg(INXUNM_ERR, symbol_table[end_index].tokpts);
		return_value = FAIL;}
	  else
	    {if (token_type != T_EOS)
	       {return_value = FAIL;
		print_msg(T_EOS, token_buf);}
	     else token_type = get_token();
	    }
	 }
      }
   }
   }

   if (debug == 2)
      printf ("allocate returning return_value : %d\n", return_value);

   return(return_value);

} /* end of allocate() */



