/* P1_ANDFOR.C  910415 */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"

/* ===========================================================================

   function name: andfor_
   input	       : ---
   output	    : ---
   return value : return_value	- SUCCESS, if no errors
			 	                     - FAIL, otherwise

   global vars  : symbol_table	- array containing symbol table
		            for_count	- counter for making 'for' statement labels
		            token_type	- type of current token
		            token_buf	- character string for current token
		            debug		- flag for debugging statements

   description	: This routine is part of a recursive descent parser and
		           handles the following production from the grammar:

           ANDFOR_ <-- index in LOG_EXP MOREANDFOR BODY endandfor index ;

		  The index variable must be an identifier that has been
		  declared as an 'index parallel' identifier. The expression
		  must evaluate to a logical parallel result. The body
		  of the 'for' statement can be empty. The keywords 'in'
		  and 'endfor' are required. The index variable must be
		  repeated at the end of the 'for' loop. The loop is
		  executed once for each responder in the expression. The
		  expression is evaluated only once at the beginning of
		  the loop and the result is stored in the index variable.
		  The index variable is updated on each pass of the loop.
		  The index variable may not be modified during the loop
		  execution, but may be used to index a parallel variable.

=========================================================================== */

int andfor_(old_fail_label)
char *old_fail_label;
{
   int return_value;		/* return value of function */
   int invar_index;		/* symbol table index of index variable */
   int invar_offset;    /* offset of index variable */
   int expr_index;		/* symbol table index of expression result */
   int expr_offset;     /* offset of expression result */
   char *start_label;	/* label to print at start of 'for' */
   char *repeat_label;	/* label to print at loop of 'for' */
   char *new_fail_label;/* label to exit on and fail */
   char *flag;          /* local flag variable for andfor */
   char inter[10],longint[10],expt[10],signif[10];

   if (debug == 2)
      printf ("In andfor_()........................................\n");

   pc_itoa(T_INT*SIZE_MODE2+T_CNST,inter);
   pc_itoa(W_LONGINT,longint);
   pc_itoa(W_EXPT,expt);
   pc_itoa(W_SIGNIF,signif);
   start_label = make_label ("AF$S", ++for_count);
   repeat_label = make_label ("AF$R", for_count);
   new_fail_label = make_label ("AF$X", for_count);
   install_scalar((flag = make_label ("AF$F", for_count)),"1","0",
		            inter,longint,expt,signif,FALSE,shapedx);

   print_opcode ("setscope", T_SETSCOPE, T_NONE, T_NONE);
   print_args (start_label, NULL, NULL, NULL, NULL);

   get_index(&invar_index,&invar_offset);
   if (invar_index == FAIL)
      return_value = FAIL;
   else if (token_type != T_IN)
     {print_msg (IN_ERR, token_buf);
      return_value = FAIL;}
   else
     {token_type = get_token();
      log_exp(&expr_index,&expr_offset);
      if(expr_index == FAIL) return_value = FAIL;
      else if (symbol_table[expr_index].mode2 != T_PARALLEL)
        {print_msg (PAREXP_ERR, NULL);
	      return_value = FAIL;}
      else if (symbol_table[expr_index].mode1 != T_LOGICAL)
        {print_msg (LOGEXP_ERR, NULL);
	      return_value = FAIL;}
      else
        {print_opcode ("andfor_", T_ANDFOR, T_NONE, T_NONE);
 	      print_args (off_exp1(expr_index,expr_offset), old_fail_label,
		               repeat_label, off_exp2(invar_index,invar_offset), flag);
         do_it(invar_index,invar_offset);
         andbody_(new_fail_label);

         if ((return_value = body()) != FAIL)
	        {if (token_type != T_ENDANDFOR)
	           {print_msg (ENDANDFOR_ERR, token_buf);
	            return_value = FAIL;}
	         else
	           {token_type = get_token();
	            if (strcmp(token_buf, symbol_table[invar_index].tokpts) != 0)
	              {print_msg (INXUNM_ERR, token_buf);
		            return_value = FAIL;}
               else
	              {print_opcode ("endandfor", T_ENDANDFOR, T_NONE, T_NONE);
		            print_args (new_fail_label, repeat_label,
		                        off_exp1(invar_index,invar_offset),
										flag, old_fail_label);
		            if ((token_type = get_token()) != T_EOS)
		              {print_msg (EOS_ERR, token_buf);
		               return_value = FAIL;}
		            else
		              {token_type = get_token();
		               return_value = SUCCESS;}
	              }
	           }
           }
        }
     }

   if (debug == 2)
      printf("andfor() returning return_value %d\n", return_value);

   return(return_value);
}
