/* P1_ANDIF.C  910415 */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"

/* ===========================================================================

   function name: andif_
   input	       : ---
   output	    : ---
   return value : return_value	- SUCCESS, if no errors
				                     - FAIL, otherwise

   global vars  : symbol_table	- array containing symbol table
		            if_count	- counter for making 'if' statement labels
		            token_type	- type of current token
		            token_buf	- character string for current token
		            debug		- flag for debugging statements

   description  : This routine is part of a recursive descent parser and
		            handles the following production from the grammar:

	ANDIF_ <-- INDEX in LOG_EXP MORE_ANDIF andthen INDEX BODY endandif ;

=========================================================================== */

int andif_(fail_label)
char *fail_label;
{
   int return_value;		/* return value of function */
   int expr_index;		/* symbol table index of expression result */
   int expr_offset;     /* offset of expression result */
   int log_mode;		   /* logical expression result mode: SCALAR or
				               PARALLEL */
   int invar_index;     /* index variable */
   int invar_offset;
   char *start_label;	/* label to print at start of 'if' */


   if (debug == 2)
      printf ("In andif_()........................................\n");

   start_label = make_label ("AI$S", ++if_count);

   print_opcode ("setscope", T_SETSCOPE, T_NONE, T_NONE);
   print_args (start_label, NULL, NULL, NULL, NULL);

   get_index(&invar_index,&invar_offset);
   if (invar_index  == FAIL) return_value = FAIL;
   else if (token_type != T_IN)
     {print_msg (IN_ERR, token_buf);
      return_value = FAIL;}
   else
     {token_type = get_token();
      log_exp(&expr_index,&expr_offset);
      if(expr_index == FAIL) return_value = FAIL;
      else if (symbol_table[expr_index].mode1 != T_LOGICAL)
        {print_msg (LOGEXP_ERR, NULL);
         return_value = FAIL;}
      else
        {if (symbol_table[expr_index].mode2 == T_PARALLEL)
           log_mode = T_PARALLEL;
         else if (symbol_table[expr_index].mode2 == T_SCALAR)
           log_mode = T_SCALAR;
         print_opcode ("andif_", T_ANDIF, T_NONE, log_mode);
         print_args (off_exp1(expr_index,expr_offset), fail_label,
		               off_exp2(invar_index,invar_offset), NULL, NULL);
         do_it(invar_index,invar_offset);
         if ((return_value = andbody_(fail_label)) != FAIL )
           {if (token_type != T_ATHEN)
              {print_msg(ATHEN_ERR, token_buf);
               return_value = FAIL;}
            else
              {print_opcode("and_then",T_ATHEN,T_NONE,T_NONE);
               print_args(off_exp1(invar_index,invar_offset),
								  NULL,NULL,NULL,NULL);
               do_it(invar_index,invar_offset);
               token_type = get_token();
               if (strcmp(token_buf, symbol_table[invar_index].tokpts) != 0)
                   print_msg(INXMIS_ERR, token_buf);

               token_type = get_token();
               if ((return_value = body()) != FAIL)
	              {if (token_type != T_ENDANDIF)
	                 {print_msg (ENDANDIF_ERR, token_buf);
	                  return_value = FAIL;}
	               else
	                 {token_type = get_token();
                     if (strcmp(token_buf, symbol_table[invar_index].tokpts) != 0)
                        print_msg(INXUNM_ERR, token_buf);
                     else
                       {print_opcode ("endandif_", T_ENDANDIF, T_NONE, log_mode);
		                  print_args (NULL, NULL, NULL, NULL, NULL);
		                  if ((token_type = get_token()) != T_EOS)
		                    {print_msg (EOS_ERR, token_buf);
		                     return_value = FAIL;}
		                  else
		                    {token_type = get_token();
		                     return_value = SUCCESS;}
                       }
	                 }
	              }
              }
           }
        }
     }


   if (debug == 2)
      printf("andif() returning return_value %d\n", return_value);

   return(return_value);
}
