/* P1_ANY.C  910415 */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"

/* ===========================================================================

   function name: any_
   input	       : ---
   output	    : ---
   return value : return_value	- SUCCESS, if no errors
			 	                     - FAIL, otherwise

   global vars  : symbol_table	- array containing symbol table
		            any_count	- counter for making 'any' statement labels
		            token_type	- type of current token
		            token_buf	- character string for current token
		            debug		- flag for debugging statements

   description	 : This routine is part of a recursive descent parser and
		            handles the following production from the grammar:

			ANY_ <-- LOG_EXP BODY1 elsenany BODY2 endany ;

		  The expression must evaluate to a logical parallel result.
		  The body of the 'any' statement can be empty. The keyword
		  'endany' is required. The body of the any statement will
		  only be executed if there is at least one responder in the
		  logical parallel result.

=========================================================================== */

int any_()
{
   int return_value;		/* return value of function */
   int expr_index;	   /* symbol table index of the expression result*/
   int expr_offset;     /* offset of expression result */
   char *start_label;	/* label to print at start of 'any' */
   char *else_label;    /* label to print at start of else */
   char *end_label;		/* label to print at end of 'any' */


   if (debug == 2)
      printf ("In any_()........................................\n");

   start_label = make_label ("AN$S", ++any_count);
   else_label = make_label ("AN$L", any_count);
   end_label = make_label ("AN$E", any_count);

   print_opcode ("setscope", T_SETSCOPE, T_NONE, T_NONE);
   print_args(start_label, NULL, NULL, NULL, NULL);
   log_exp(&expr_index,&expr_offset);
   if(expr_index == FAIL)
		return_value = FAIL;
   else if (symbol_table[expr_index].mode2 != T_PARALLEL ||
	         symbol_table[expr_index].mode1 != T_LOGICAL)
      {print_msg (ANYLOGPAR_ERR, NULL);
       return_value = FAIL;}
   else
     {save_em(expr_index,expr_offset);
		print_opcode ("any  ", T_ANY, T_NONE, T_PARALLEL);
      print_args(off_exp1(expr_index,expr_offset),else_label,NULL, NULL, NULL);
      if ((return_value = body()) != FAIL)
        {if(token_type == T_ELSENANY)
	        {print_opcode("elsenany",T_ELSENANY,T_NONE,T_PARALLEL);
            print_args(else_label,end_label,NULL,NULL,NULL);
            token_type = get_token();
            return_value = body();}
	      else
	        {print_opcode("label",T_LABEL,T_NONE,T_NONE);
		 print_args(else_label,NULL,NULL,NULL,NULL);}

	      if (token_type != T_ENDANY)
	        {print_msg (ENDANY_ERR, token_buf);
	         return_value = FAIL;}
	      else
           {print_opcode ("endany", T_ENDANY, T_NONE, T_PARALLEL);
	         print_args (end_label, NULL, NULL, NULL, NULL);
   	      if ((token_type = get_token()) != T_EOS)
	           {print_msg (EOS_ERR, token_buf);
	            return_value = FAIL;}
	         else
	           {token_type = get_token();
	            return_value = SUCCESS;}
           }
        }
     }

   if (debug == 2)
      printf("any_() returning return_value %d\n", return_value);

   return(return_value);

} /* end of any_() */


