/* P1_ASSEMBLE.C  910415 */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_scanner.h"
#include "p1_parser.h"

/* ===========================================================================

   function name: assemble_
   input	       : ---
   output	    : ---
   return value : ret_value		- SUCCESS, if no errors detected
					                  - FAIL, otherwise

   global vars  : token_type		- type of the current token
		            debug			   - flag for debugging statements
		            next_char		- next character from the source code
		            active_fp		- active file pointer
		            list_fp		   - listing file pointer
	               linenum		   - current line number

   description	 : This routine handles in-line assembly code. It will read
		            the assembly code and echo it back to the listing file
		            with a special intermediate opcode. It will also generate
		            a line of code to the object file consisting of the inter-
		            mediate opcode and the assembly language as an argument.
                  The production from the grammar is:

                         ASSEMBLE  <-- asmcode IN_LINE_CODE endasm;

                  IN_LINE_CODE is the actual in-line assembly language code.

=========================================================================== */

int assemble_()
{
   int ret_value;		/* identifier for return value */
   int end_flag;		/* flag for end of assembly code */
   int first;			/* flag for first line of assembly code */
   int i;			   /* buffer index */
   char line_code[LINE_LENGTH]; /* a buffer that contains a line of
				                       assembly code (maximum 80 characters) */
   char *word;			/* pointer to the first word of assembly code
				            in the buffer */

   if (debug == 2)
      printf("In assemble_()...........................\n");

   end_flag = FALSE;
   ret_value = SUCCESS;
   first = TRUE;

   while(!end_flag)
     {if (first)
        {first = FALSE;
         i = 1;
         line_code[0] = next_char;}
      else
         i = 0;

      while (((line_code[i++] = getc(active_fp)) != EOF) &&
             (line_code[i-1] != '\n'));

      if (line_code[i-1] == EOF)
        {print_msg(EOF_ERR, NULL);
         printf("\nhint: ");
         printf("probably missing or mis-spelling \"endasm;\" keyword\n\n");
         ret_value = FAIL;
         end_flag = TRUE;}
      else
        {line_code[i] = (char) NULL;
         fprintf(list_fp, "%4d ", ++linenum);
         fprintf(list_fp, "%s", line_code);
         word = first_word(line_code);
         if ((strcmp(word, "endasm") == 0) || (strcmp(word, "ENDASM") == 0))
           {ret_value = FAIL;
            print_msg(ASMEOS_ERR, word);}
         else if((strcmp(word,"endasm;") == 0)||(strcmp(word,"ENDASM;") == 0))
           {end_flag = TRUE;
            next_char = getc(active_fp);
            token_type = get_token();}
         else if ((strcmp(word,"debug") == 0) || (strcmp(word,"DEBUG") == 0))
			  { *(word+7) = 0; /* set debug for one digit only */
				 debug = atoi(word+6);}
         else if (word != NULL)
           {print_opcode("asmcode", T_ASSEMBLE, NULL, NULL);
            print_args(line_code, NULL, NULL, NULL, NULL);}
        }
     }

   if (debug == 2)
      printf("Assemble_() returning ret_value = %d\n", ret_value);

   free(word);
   return(ret_value);

} /* end of assemble_() */




/* ===========================================================================

   function name: first_word
   input	       : line_code		- buffer containing assembly code
   output	    : ---
   return value : word			- first word of buffer

   global vars  : ---

   description	 : This routine is finding the first word in the buffer.

=========================================================================== */

char *first_word(line_code)
char *line_code;
{
   char *line;			/* copy of buffer */
   char *word;			/* pointer to first word of 'line' */

   line = (char *) calloc(LINE_LENGTH+1, sizeof(char));
   strcpy(line,line_code);

   while (char_type(*line) == WHITESPACE)
      line++;

   word = line;
   while ((char_type(*line) != WHITESPACE) && (*line != (char) NULL))
      line++;
   *line = (char) NULL;

   return(word);

} /* end of get_word() */
