/* P1_ASSIGN.C  910415 */
#include "stdlib.h"
#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"
#include "ace2.h" /* int rhs_index2, rhs_offset2, lhs2; */

char *off_exp1();

/* ===========================================================================

   function name: assign
   input               : ---
   output           : ---
   return value :               - SUCCESS, if no errors
					       - FAIL, otherwise

   global vars  : token_type    - type of current token
			    debug               - flag for debugging statements
			    symbol_table        - array for symbol table
			    token_buf   - character string for current token

   description   : This routine is part of a recursive descent parser and
			    handles the following production from the grammar:

			ASSIGN <-- identifier MORE_IDENTIFIER = LOG_EXP ;

		  The left side of an assignment statement must contain
		  an identifier found in the symbol table. The right side
		  can be any expression. The following restrictions apply:
		  1) a parallel result cannot be assigned to a scalar
		  identifier, 2) reals and integers can only be assigned
		  to reals and integers (conversions are done as needed),
		  and 3) logicals and indexes can only be assigned to
		  logicals.

=========================================================================== */

/*
  In the connection machine, three types of data move are recognized:
     unsigned
     signed
     floating point.

     The ASP types are mapped onto the CM types as follows:

     T_CHAR, T_CARD, T_LOGICAL - unsigned
     T_FIXED, T_OCTAL, T_BIN, T_HEX - signed
     T_REAL - floating point  - IEEE format
*/

int assign()
{
	int rhs_index;							/* symbol table index of RHS identifier */
	int rhs_offset;							/* offset of rhs */
	int lhs_index;							/* symbol table index of LHS identifier */
	int lhs_index2;
	int lhs_offset;							/* symbol table index for the offset */
	int lhs_offset2;
	int scalar_dx;
	int scalar_dx2;
	int temp_dx;
	int temp_dx2;
	int type,flag,lhsdx_dx,lhsdx_os;		/* items for subsubscript() */
	int type2,flag2,lhsdx_dx2,lhsdx_os2;	/* items for subsubscript() */
	int g_lhs_dx2;


	/* initialize local variables */
	rhs_index = 0;
	rhs_offset = 0;
	lhs_index = 0;
	lhs_index2 = 0;
	lhs_offset = 0;
	lhs_offset2 = 0;
	scalar_dx = 0;
	scalar_dx2 = 0;
	temp_dx = 0;
	temp_dx2 = 0;
	type = 0;
	flag = 0;
	lhsdx_dx = 0;
	lhsdx_os = 0;
	type2 = 0;
	flag2 = 0;
	lhsdx_dx2 = 0;
	lhsdx_os2 = 0;
	g_lhs_dx2 = 0;

	if (debug == 2)
		printf ("In assign().....................................\n");

	/* process left hand side */
	/* g_assign is true inside an assign */
	if ((lhs_index = symtab_fetch(token_size, token_buf)) == FAIL)
	{
		print_msg(UNDEF_ERR, token_buf);  
		return(FAIL);
	}
 
	if ((symbol_table[lhs_index].toktype != T_ID) &&
		(symbol_table[lhs_index].toktype != T_INDEX))
	{
		print_msg(ASGNID_ERR, token_buf);
		printf(" toktype = %d\n",symbol_table[lhs_index].toktype);
	return(FAIL);
	}

	g_lhs_dx = lhs_index;		/* set for p1_subscript.c */
 
	/* if lhs is parallel[log_exp,...], g_lhs_dx is set to a some scalar index, not necessiraly the real lhs_index. */

	/* process subscript if any */
	token_type = get_token();
	cube_flag = FALSE;
	if(subsubscript(lhs_index,&type,&flag,&lhsdx_dx,&lhsdx_os,&lhs_offset,&temp_dx) == FAIL)  
		return(FAIL);
 
	if(cube_flag == TRUE)
	{
		print_msg(CUBE_ERR,token_buf); 
		return(FAIL);
	}
 
	if(type == T_MASKEDMOVE)
	{
		if((g_lhs_dx = get_symtab_temp(symbol_table[lhs_offset].mode1, T_SCALAR,T_NONE)) == FAIL)
		{
			print_msg(TEMP_ERR, "in p1_assig 1"); 
			return(FAIL); 
		}
	}

	if(temp_dx != SUCCESS)
		/* modified 880707 jlp */
		if((scalar_dx =get_symtab_temp(symbol_table[lhs_offset].mode1,T_SCALAR,T_NONE)) == FAIL)
		{
			print_msg(TEMP_ERR, "in p1_assig 2"); 
			return(FAIL); 
		}
		else  /* put indexed offset into a scalar */
		{
			print_opcode ("mvsc_",T_MVSC,symbol_table[lhs_offset].mode1,T_SCALAR);
			print_args (off_exp1(lhs_offset,temp_dx),off_exp2(scalar_dx,SUCCESS),NULL,NULL,NULL);
			lhs_offset = scalar_dx;
		}

	/* process assignment keyword */
	lhs2 = FALSE;
	if(token_type == T_COMMA)
	{
		/* process second left hand side */
		token_type = get_token();
		lhs2 = TRUE;
		if ((lhs_index2 = symtab_fetch(token_size, token_buf)) == FAIL)
		{
			print_msg(UNDEF_ERR, token_buf);  
			return(FAIL);
		}
		if ((symbol_table[lhs_index2].toktype != T_ID) && (symbol_table[lhs_index2].toktype != T_INDEX))
		{
			print_msg(ASGNID_ERR, token_buf);
			printf(" toktype = %d\n",symbol_table[lhs_index].toktype);
			return(FAIL);
		}
 
		/* if lhs is parallel[log_exp,...], g_lhs_dx is set to a some scalar index, not necessiraly the real lhs_index. */

		/* process subscript if any */
		token_type = get_token();
		cube_flag = FALSE;
		if(subsubscript(lhs_index2,&type2,&flag2,&lhsdx_dx2,&lhsdx_os2, &lhs_offset2,&temp_dx2) == FAIL)  
			return(FAIL);
 
		if(cube_flag == TRUE)
		{
			print_msg(CUBE_ERR,token_buf); 
			return(FAIL);
		}
 
		if(type2 == T_MASKEDMOVE)
        { 
			if((g_lhs_dx2 = get_symtab_temp(symbol_table[lhs_offset2].mode1, T_SCALAR,T_NONE)) == FAIL)
			{
				print_msg(TEMP_ERR, "in p1_assig 3"); 
				return(FAIL); 
			}
		}
 
		if(temp_dx2 != SUCCESS)
			if((scalar_dx2 =get_symtab_temp(symbol_table[lhs_offset2].mode1,T_SCALAR,T_NONE)) == FAIL)
			{
				print_msg(TEMP_ERR, "in p1_assig 4"); 
				return(FAIL); 
			}
			else  /* put indexed offset into a scalar */
			{
				print_opcode ("mvsc_",T_MVSC,symbol_table[lhs_offset2].mode1,T_SCALAR);
				print_args (off_exp1(lhs_offset2,temp_dx2),
				off_exp2(scalar_dx2,SUCCESS),NULL,NULL,NULL);
				lhs_offset2 = scalar_dx2;
			}
	} /* end of if comma */
	else if (token_type != T_ASSIGN)
	{
		print_msg(ASGNOP_ERR, token_buf); 
		return(FAIL); 
	}
	
	token_type = get_token();

	/* process right hand side */
	g_assign++;
	log_exp(&rhs_index,&rhs_offset);
	if(rhs_index != FAIL) 
		save_em(rhs_index,rhs_offset);

	if(rhs_index == FAIL) 
	{ 
		g_assign--; 
		return(FAIL);
	}

	if(genconv(&rhs_index, &rhs_offset, &lhs_index, &lhs_offset) == FAIL)
		return FAIL; /* g_assign-- done in genconv */

	if(genassign(type, lhs2, flag, lhs_index, lhs_offset, lhs_index2, lhs_offset2, rhs_index, rhs_offset, rhs_index2, rhs_offset2, lhsdx_dx, lhsdx_os) == FAIL) 
		return FAIL; /* g_assign-- done in genmov */


	if (debug == 2)
		printf ("Assign() returning\n");

	g_assign--;  /* g_assign is true inside an assign*/

	if (token_type == T_EOS)
	{
		token_type = get_token(); 
		return( SUCCESS); 
	}
	else 
	{
		print_msg (EOS_ERR, token_buf); 
		return(FAIL); 
	}

}

/* generate conversion quadruples */
genconv(int *rhs_index, int *rhs_offset, int *lhs_index, int *lhs_offset)
{
  char msgbuf[16],num[8];           /* for debug print  buffers */

/* generate scalar conversion quadruple if necessary */
if(symbol_table[*rhs_index].mode2 == T_REG_SCALAR)
   symbol_table[*rhs_index].mode2 = T_SCALAR;

/* generate mode conversion quadruples if necessary */
switch(symbol_table[*rhs_index].mode1)
 {case T_OCT:
  case T_HEX:
  case T_BIN:
  case T_CARD:
  case T_LOGICAL:
  case T_LONGINT:
  case T_INT: switch(symbol_table[*lhs_index].mode1)
	      { case T_REAL: float_(rhs_index,rhs_offset); break;
		case T_CARD:
		     case T_OCT:
			  case T_HEX:
			  case T_BIN:
		     case T_LONGINT:
		     case T_LOGICAL:
			  case T_INT: break;
			  case T_CHAR: intchr_(rhs_index,rhs_offset); break;
		     default: /* T_LOGICAL */
			 type_name(symbol_table[*lhs_index].mode1,num);
			 strcpy(msgbuf,"int to ");
			 strcat(msgbuf,num);
			 print_msg (ASGNCON_ERR,msgbuf);
			 g_assign--;
			 return(FAIL);
	      };
	      break;
  case T_REAL:switch(symbol_table[*lhs_index].mode1)
	      { case T_OCT:
		case T_BIN:
		case T_LONGINT:
		case T_HEX:
		case T_CARD:
		     case T_INT: fix_(rhs_index,rhs_offset); break;
		     case T_REAL: break;
		     default: /* T_CHAR, T_LOGICAL */
			 type_name(symbol_table[*lhs_index].mode1,num);
			 strcpy(msgbuf,"real to ");
			 strcat(msgbuf,num);
				   print_msg (ASGNCON_ERR,msgbuf);
			 g_assign--;
			 return(FAIL);
	      };
	      break;
   case T_CHAR: switch(symbol_table[*lhs_index].mode1)
	      { case T_CARD:
		     case T_OCT:
			  case T_HEX:
			  case T_BIN:
		     case T_LONGINT:
			  case T_INT:  chrint_(rhs_index,rhs_offset);
			  break;
			  case T_CHAR: break;
		     case T_REAL:
                   /* debugging code */
		 print_opcode("%mvsc_",T_MVSC,99,T_MASKEDMOVE);
		 print_args(off_exp1(lhs_index,SUCCESS),
			    off_exp2(rhs_index,SUCCESS),
			    NULL,NULL,NULL);

		     case T_LOGICAL:
		default: /* T_LOGICAL */
			 type_name(symbol_table[*lhs_index].mode1,num);
			 strcpy(msgbuf,"char to ");
			 strcat(msgbuf,num);
				   print_msg (ASGNCON_ERR,msgbuf);
			 g_assign--;
			 return(FAIL);
	      };
	      break;
   default:  if (symbol_table[*rhs_index].mode1 != symbol_table[*lhs_index].mode1)
		{type_name(symbol_table[*rhs_index].mode1,num);
		 strcpy(msgbuf,num);
		 strcat(msgbuf," to ");
		 type_name(symbol_table[*lhs_index].mode1,num);
		 strcat(msgbuf,num);
		      print_msg (ASGNCON_ERR,msgbuf);
printf("rhs token=%s, mode1=%d\n",symbol_table[*rhs_index].tokpts,symbol_table[*rhs_index].mode1);
printf("lhs token=%s, mode1=%d\n",symbol_table[*lhs_index].tokpts,symbol_table[*lhs_index].mode1);
 }
	      else printf("In p1_assign:genconv, illegal subsubscript_type detected\n");
	      g_assign--;
	      return(FAIL);
  };
return SUCCESS;
}

genassign(int type, int lhs2, int flag, int lhs_index, int lhs_offset, int lhs_index2, int lhs_offset2,
				      int rhs_index, int rhs_offset, int rhs_index2, int rhs_offset2,
												  int lhsdx_dx,   int lhsdx_os)
{ int temp_dx, size, lhs_mode2, c_size_dx;
  char msgbuf[16];           /* for debug print  buffers */
  char c_size[10];  /* char buf for size1 */

/* generate move quadprules */
   switch(type) /* lhs type2 */
      {case T_PARALLEL: /* T_MVSC - asns_idxpa - MREG masked move */
		 strcpy(msgbuf," parallel ");
		 if(!flag){print_msg(ASGNCON_ERR,msgbuf);
							 g_assign--;
		      return(FAIL);}
		 if (((lhs_mode2=symbol_table[rhs_index].mode2) == T_SCALAR) ||
				(lhs_mode2 == T_REG_SCALAR) ||
				(lhs_mode2 == T_CNST ))
	       {size = symbol_table[rhs_index].width;
					 if(lhs_mode2 == T_CNST || size == 0)
		    size=symbol_table[lhs_index].width;
					 pc_itoa(size,c_size);
		c_size_dx = get_const(c_size,T_DEC_CONST);
			       print_opcode("mvsc_",T_MVSC,
					      symbol_table[rhs_index].mode1,T_PARALLEL);
		print_args (off_exp1(rhs_index,rhs_offset),
			    off_exp2(lhs_index,lhs_offset),
						off_exp3(c_size_dx,SUCCESS), NULL,NULL); }
	     else /* rhs_index.mode2 == T_PARALLEL */
	       {/* T_MVPA - asnp_idxpa   pa[$,offset]=pa[os]; */
		if(lhs2 != TRUE) /* only one lhs */
						{if(flag != TRUE)
							{print_msg(ASGNCON_ERR,msgbuf);
							 g_assign--; return(FAIL);}
						 else
		     {if((cube_flag == TRUE) && (code_type == EMULATE))
							   genmvcb(&rhs_index,&rhs_offset);
		      print_opcode("mvpa_",T_MVPA,symbol_table[rhs_index].mode1,T_PARALLEL);
		      print_args (off_exp1(rhs_index,rhs_offset),
				  off_exp2(lhs_index,lhs_offset),NULL,NULL,NULL);
							}
						}
					 else /* two left hand sides */
					  {if(flag != TRUE)
							{print_msg(ASGNCON_ERR,msgbuf);
							 g_assign--; return(FAIL);}
						 else
		     {if((cube_flag == TRUE) && (code_type == EMULATE))
							   genmvcb(&rhs_index,&rhs_offset);
		      print_opcode("mvpa_",T_MVPA,symbol_table[rhs_index].mode1,T_PARALLEL);
		      print_args (off_exp1(rhs_index,rhs_offset),
				  off_exp2(lhs_index,lhs_offset),NULL,NULL,NULL);
		      if((cube_flag == TRUE) && (code_type == EMULATE))
							   genmvcb(&rhs_index2,&rhs_offset2);
		      print_opcode("mvpa_",T_MVPA,symbol_table[rhs_index2].mode1,T_PARALLEL);
		      print_args (off_exp1(rhs_index2,rhs_offset2),
				  off_exp2(lhs_index2,lhs_offset2),NULL,NULL,NULL);
					  }}}
	     break;

       case T_MASKEDMOVE: /* mvsc - asns_idxpa - explicitly masked
					      pa[xx,offset] = sc[os];  */
	   strcpy(msgbuf," maskedmove");
		if(lhs2 != TRUE)
			    {if(flag != TRUE){print_msg(ASGNCON_ERR,msgbuf);
			g_assign--;
								return(FAIL);}
			     /* move "xx" to a temp so that it can be resvfst-ed  */
			temp_dx = get_symtab_temp(T_LOGICAL,T_PARALLEL,W_LOGICAL);
	      print_opcode("resf_",T_RESFST, T_NONE, T_PARALLEL);
	      print_args (off_exp1(lhsdx_dx,lhsdx_os),
					      off_exp2(temp_dx,SUCCESS),
			  NULL, NULL, NULL);
	      print_opcode("mvsc_",T_MVSC,
					  symbol_table[rhs_index].mode1,T_MASKEDMOVE);
	      print_args(off_exp1(temp_dx,SUCCESS),
					     off_exp2(rhs_index,rhs_offset),
			 off_exp3(lhs_index,lhs_offset),NULL,NULL);}
			  else /* lhs2 == TRUE */
			    {if(flag != TRUE){print_msg(ASGNCON_ERR,msgbuf);
			g_assign--;
								return(FAIL);}
			     /* move "xx" to a temp so that it can be resvfst-ed  */
			temp_dx = get_symtab_temp(T_LOGICAL,T_PARALLEL,W_LOGICAL);
	      print_opcode("resf_",T_RESFST, T_NONE, T_PARALLEL);
	      print_args (off_exp1(lhsdx_dx,lhsdx_os),
					      off_exp2(temp_dx,SUCCESS),
			  NULL, NULL, NULL);
	      print_opcode("mvsc_",T_MVSC,
					  symbol_table[rhs_index].mode1,T_MASKEDMOVE);
	      print_args(off_exp1(temp_dx,SUCCESS),
					     off_exp2(rhs_index,rhs_offset),
			 off_exp3(lhs_index,lhs_offset),NULL,NULL);
	      print_opcode("mvsc_",T_MVSC,
					  symbol_table[rhs_index].mode1,T_MASKEDMOVE);
	      print_args(off_exp1(temp_dx,SUCCESS),
					     off_exp2(rhs_index2,rhs_offset2),
			 off_exp3(lhs_index2,lhs_offset2),NULL,NULL);}
	      break;
       case T_SCALAR:  /* asns_idxsc */
	      if(symbol_table[rhs_index].mode2 == T_PARALLEL)
		  {size = symbol_table[rhs_index].width;
		     pc_itoa(size,c_size);
				  c_size_dx = get_const(c_size,T_DEC_CONST);
		   print_opcode("mvpa_",T_MVPA,symbol_table[rhs_index].mode1,T_SCALAR);
				  print_args(off_exp1(rhs_index,rhs_offset),
							    off_exp2(lhs_index,lhs_offset),
							    off_exp3(LP_TRUE,SUCCESS),
							    off_exp4(c_size_dx,SUCCESS),NULL);
					   }
	      else
			  {                print_opcode("mvsc_",T_MVSC,
						symbol_table[rhs_index].mode1, T_SCALAR);
		 print_args (off_exp1(rhs_index,rhs_offset),
			  off_exp2(lhs_index,lhs_offset),
			  NULL, NULL, NULL);
			   if(lhs2)
					    {print_opcode("mvsc_",T_MVSC,
						   symbol_table[rhs_index].mode1, T_SCALAR);
		    print_args (off_exp1(rhs_index2,rhs_offset2),
			     off_exp2(lhs_index2,lhs_offset2),
			     NULL, NULL, NULL);}
					 }
	      break;
       case T_CNST:    /* sc[offset] = sc;  */
			print_opcode("mvsc_",T_MVSC,
						symbol_table[rhs_index].mode1, T_SCALAR);
	      print_args (off_exp1(rhs_index,rhs_offset),
			  off_exp2(lhs_index,lhs_offset),
			  NULL, NULL, NULL);
	      break;
       default: printf("In p1_assign:genassign, illegal subsubscript_type detected\n");
	      g_assign--;
		   return (FAIL);
     };
return SUCCESS;
}

type_name(typeno,typebuf)
int typeno;
char *typebuf;
{
switch(typeno)
 {case T_OCT: strcpy(typebuf,"octal"); break;
  case T_HEX: strcpy(typebuf,"hex"); break;
  case T_BIN: strcpy(typebuf,"binary"); break;
  case T_CARD: strcpy(typebuf,"card"); break;
  case T_LOGICAL: strcpy(typebuf,"logical"); break;
  case T_LONGINT: strcpy(typebuf,"longint"); break;
  case T_INT: strcpy(typebuf,"int"); break;
  /* return encoded number if unrecognized */
  default: pc_itoa(typeno,typebuf); break;
  }
return 0;
}
