/* P1_BODY.C 910415 */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"

/* ===========================================================================

   function name: body
   input	       : ---
   output	    : ---
   return value : return_value	- SUCCESS, if no errors detected
				                     - FAIL, otherwise

   global vars  : debug		- flag for debugging statements
                  token_type	- type of current token
		            sca_temp	- symbol table index of current scalar temp
		            ipar_temp	- symbol table index of current int/par temp
		            rpar_temp	- symbol table index of current real/par temp
		            lpar_temp	- symbol table index of current log/par temp
		            lipar_temp  	- symbol table index of current longint/par tmp

   description	 : This routine is part of a recursive descent parser and
		            handles the following production from the grammar:

	 BODY <-- for FOR_ BODY |
				 first FIRST_ BODY |
				 loop LOOP_ BODY |
				 next NEXT_ BODY |
				 get GET_ BODY |
				 if IF_ BODY |
                                 clear C_BODY |
				 any ANY_ BODY |
				 ifnany IFNANY_ BODY |
				 while WHILE_ BODY |
				 asmcode ASSEMBLE_ BODY |
				 allocate ALLOCATE_ BODY |
				 call CALL_ BODY |
             stop STOP_ BODY |
				 return RETURN_ BODY |
				 setscope SETCSOPE_ BODY |
				 identifier ASSIGN BODY |
				 indirect_address ADDRESS_ BODY |
				 print |
				 stackfor STACKFOR_BODY |
				 stackwhile STACKWHILE_BODY |
				 recurse var_list then assignment |
				 @

		  No code is generated. If none of the specified tokens is
		  found, the function returns SUCCESS. If one of the tokens
		  is found and no errors are detected in subsequent function
		  calls, the function is called recursively to process the
		  next statement, until either an error occurs or no more
		  statements are found.

=========================================================================== */

int body()
{
   int return_value;	/* return value of function */
   int d_v;          /* dummy parameter for call to element */
   int done;			/* recursion control variable */
   char num[13];
   int return_flag;  /* set to SUCCESS by return_,
				            checked by endsub  added 880606 jlp */

   return_value = SUCCESS;
   return_flag = FAIL; /* is SUCCESS if return is last statement 880606 jlp */
   done = FALSE;
   sca_temp = sca_temp_start;          /*ANY changes in this area must be */
   real_sca_temp = REAL_SCA_TEMP_START;/* reconciled with code at beginning */
   ipar_temp = INT_PAR_TEMP_START;     /* of p1_stackwhile.c and stackfor */
   rpar_temp = REAL_PAR_TEMP_START;
   lpar_temp = log_par_temp_start;
   lipar_temp = LINT_PAR_TEMP_START;
   g_offset = g_offset_start;   /* added 880707 jlp */

if(debug == 2)
 printf("\n*********\nAt beginning of body(), line %d token=%s, type=%x\n",linenum, token_buf,token_type);

   pc_itoa(linenum,num);
   print_opcode ("beg_of_stmt ", T_BOS, T_NONE, T_NONE);
   print_args (num,"0",NULL,NULL,NULL);

   if (token_type == T_FOR)
     {token_type = get_token();
      return_value = for_();}
   else if (token_type == T_GET)
     {token_type = get_token();
      return_value = get_();}
   else if (token_type == T_ANY)
     {token_type = get_token();
      return_value = any_();}
   else if (token_type == T_NEXT)
     {token_type = get_token();
      return_value = next_();}
   else if (token_type == T_IF)
     {token_type = get_token();
      return_value = if_();}
   else if (token_type == T_MSG)
     {return_value = msgin_();}
   else if (token_type == T_OUTPUT)
     {return_value = dump_();}
   else if (token_type == T_DINPUT)
     {return_value = dimp_();}
   else if (token_type == T_WHILE)
     {token_type = get_token();
      return_value = while_();}
   else if (token_type == T_ASSEMBLE)
      return_value = assemble_();
   else if (token_type == T_FIRST)
     {token_type = get_token();
      return_value = first_();}
   else if (token_type == T_LOOP)
     {token_type = get_token();
      return_value = loop_();}
   else if (token_type == T_ALLOCATE)
     {token_type = get_token();
      return_value = allocate_();}
   else if (token_type == T_RELEASE)
     {token_type = get_token();
      return_value = release_();}
   else if (token_type == T_CALL)
     {token_type = get_token();
      return_value = call_();}
   else if (token_type == T_RETURN)
     {token_type = get_token();
      if((return_value = return_()) == SUCCESS)
		  return_flag = SUCCESS;}
   else if (token_type == T_STOP)
     {token_type = get_token();
      return_value = stop_();}
   else if (token_type == T_SETSCOPE)
     {token_type = get_token();
      return_value = setscope_();}
   else if (token_type == T_CLEAR)
     {token_type = get_token();
      return_value = clear_(); }
   else if (token_type == T_ADDRESS)
     {token_type = get_token();
      return_value = address_();}
   else if (token_type == T_ID)
      return_value = assign();
   else if (token_type == T_ANDCONS)
      return_value = andcons_();
   else if (token_type == T_PRINT)
      return_value = print_();
   else if (token_type == T_READ)
		 return_value = read_(0,T_READ);
   else if (token_type == T_READLN)
		 return_value = read_(1,T_READ);
   else if (token_type == T_REREAD)
		 return_value = read_(0,T_REREAD);
   else if (token_type == T_STACKWHILE)
      return_value = stackwhile_();
   else if (token_type == T_STACKFOR)
      return_value = stackfor_();
   else if (token_type == T_RECURSE)
      return_value = recurse_();
   else if (token_type == T_CLOSE ||
				token_type == T_COPY  ||
	    token_type == T_GETADD||
		        token_type == T_LOAD ||
				token_type == T_STORE ||
				token_type == T_DECLAR||
			token_type == T_SCOT  ||
				token_type == T_SCOTL ||
				token_type == T_SCOT8 ||
				token_type == T_SCOTP ||
				token_type == T_SCIN  ||
				token_type == T_SCINL ||
				token_type == T_SCIN8 ||
				token_type == T_SCINP )
      {element(&return_value,&d_v);
		 if(return_value != FAIL) return_value = SUCCESS;
		 if(token_type != T_EOS)
			{print_msg(EOS_ERR,token_buf);
			 return_value = FAIL;}
		 else token_type = get_token();}
	else
     done = TRUE;


   if (return_value == SUCCESS && !done)
       return_value = body();

 if (debug == 2)
      printf ("Body() returning return_value %d, return_flag=%d, line %d\n",
					return_value, return_flag, linenum);

   if(return_flag == SUCCESS) return(TRUE);
   else return(return_value);

} /* end of body() */



/* ===========================================================================

   function name: make_label
   input	       : name		- string containing beginning of label
		            counter	- integer used at the end of label
   output	    : ---
   return value : label		- pointer to new label

   global vars  : ---

   description	 : This routine constructs a new label from 2 arguments it
		            receives and returns a pointer to the new label.

=========================================================================== */

char *make_label(name, counter)
char *name;
int counter;
{
   char *label;		/* pointer to new label */
   char num[5];		/* string for integer used at the end of label */
   int slen;

   if (debug == 2) printf ("In make_label().........................\n");

   pc_itoa(counter, num);
   slen = strlen (name) + strlen (num) + 2;
   label = (char *)calloc (slen, sizeof(char));
   if(label == NULL)
	  {printf("In make_label, out of memory. line %d.\n",linenum);
		exit(1);}
   strcpy (label, name);
   strcat (label, num);

   if (debug == 2) printf ("make_label() returning : %s\n", label);

   return(label);

} /* end of make_label() */




/* ===========================================================================

   function name: get_index
   input        : ---
   output	    : index,offset
   return value : index		- symbol table location of index variable, if
				                 no errors
				               - FAIL, otherwise

   global vars  : token_size	- size of current token
		            token_buf	- character string for current token
		            token_type	- type of current token
		            debug		- flag for debugging statements

   description	 : This routine checks to see if the current token is an
		            identifier that has been declared as an index parallel
		            variable. If so, the symbol table index of the variable
        	         is returned as index, otherwise, FAIL is returned.
		            The code commented out below will allow the index to be
		            subscripted (i.e. xx[$,i,j])

=========================================================================== */

int get_index(id_index,id_offset)
int *id_index,*id_offset;
{
   int arg1_index,arg1_offset;  /* for subscript call */

   if (debug == 2)
      printf ("In get_index() ...........................\n");

   *id_index = FAIL;
   *id_offset = SUCCESS;

   if (strcmp(token_buf,"EACH") == 0)
     {strcpy(token_buf,"ITS");
		token_size = 3;
		token_type = T_ID;}

   if (token_type != T_ID)
     {print_msg (ID_ERR, token_buf);
      print_msg (MISINX_ERR, NULL);}
   else if((*id_index = symtab_fetch(token_size, token_buf)) == FAIL)
      print_msg (UNDEF_ERR, token_buf);
   else if (symbol_table[*id_index].toktype != T_INDEX)
      print_msg (INXMIS_ERR, token_buf);
   else if (symbol_table[*id_index].mode2 != T_PARALLEL)
      print_msg (INXPAR_ERR, token_buf);
   else if((token_type = get_token()) == T_LSQU)
      {subscript(*id_index,&arg1_index,&arg1_offset);
       *id_index=arg1_index;
       *id_offset=arg1_offset;}

   if(debug == 2)printf("Get_index returning %s\n",
			               symbol_table[*id_index].tokpts);

   return(*id_index);

} /* end of get_index() */

/* save the index variable in ITS */
do_it(invar_index, invar_offset)
int invar_index, invar_offset;
{
  if(invar_index != symtab_fetch(3,"ITS"))
	 {print_opcode("mvpa_",T_MVPA,T_LOGICAL,T_PARALLEL);
	  print_args(off_exp1(invar_index,invar_offset),
					 off_exp2(symtab_fetch(3,"ITS"),SUCCESS),
					 NULL,NULL,NULL);}
return 0;
}

/* save varible in THE */
save_the(scalar_dx,scalar_off,parallel_dx,parallel_off)
int scalar_dx,scalar_off,parallel_dx,parallel_off;
{
  /* put it in THE entry */
	  print_opcode("mvsc_",T_MVSC,T_LOGICAL,T_MASKEDMOVE);
	  print_args(symbol_table[symtab_fetch(3,"THE")].tokpts,
					 off_exp2(scalar_dx,scalar_off),
					 off_exp3(parallel_dx, parallel_off),
					 NULL, NULL);

return 0;
}
