/* P1_CALL.C 910415 */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"
/* ===========================================================================

   function name: call
   input	       : ---
   output	    : ---
   return value : ret_value	- SUCCESS, if no error is detected
                              - FAIL, otherwise if an error occured

   global vars  : token_type 	- type of current token
		            token_buf	- current token string
		            symbol_table	- symbol table data structure
		            debug		- debugging statement flag

   description  : This routine parses the 'call' statement in the main
	               program.

			CALL_  <--  call sub_name

		  sub_name is the name of subroutine to be invoked.
		  This routine first will check if the subroutine name is a
		  valid name. i,e : subroutine name should not be conflict
		  with variable names.

=========================================================================== */

int call_()
{
   int ret_value;		/* variable for return value */
   int sub_index;		/* symbol table index for the identifier */
   char save_buf[32];           /* save sub name for call quadruple */
   int i;
   int garg1,garg2,garg3,garg4,garg5,garg6,garg7,garg8;

   if (debug == 2)
      printf ("In call_()...........................................\n");

   ret_value = SUCCESS;

   for(i=sca_temp_start-1; i>=INT_SCA_TEMP_START; i--)
		{print_opcode ("stack_", T_STACK, T_NONE,
							 symbol_table[i].mode2);
		 print_args(off_exp1(i,SUCCESS),NULL,NULL,NULL,NULL); }
   for(i=log_par_temp_start-1; i>=LOG_PAR_TEMP_START; i--)
		{print_opcode ("stack_", T_STACK, T_NONE,
							 symbol_table[i].mode2);
		 print_args(off_exp1(i,SUCCESS),NULL,NULL,NULL,NULL); }

   if (token_type == T_ID)
     {strcpy(save_buf,token_buf); /* moved 880620 jlp */
      sub_index = symtab_fetch(token_size, token_buf);
      if (sub_index != FAIL &&
			 symbol_table[sub_index].temp != T_SUBROUTINE)
        {print_msg (DUP_ERR, token_buf);
         print_msg (SUBNAME_ERR, token_buf);
         ret_value = FAIL;}
		else if ((token_type = get_token()) == T_LPAR)
		  {if((token_type = get_token()) != T_LPAR)
			  {print_msg (LPAR_ERR, token_buf);
				ret_value = FAIL;}
      else
		  {if((token_type = get_token()) == T_EOS)
			  {print_opcode ("call_", T_CALL, T_NONE, T_NONE);
				print_args ("0", save_buf, NULL, NULL, NULL);
				token_type = get_token();
				ret_value = SUCCESS;}
			else {print_msg(EOS_ERR, token_buf);
					ret_value = FAIL;}
		  }
	   }
		else if(token_type == T_EOS)
		  {print_opcode ("call_", T_CALL, T_NONE, T_NONE);
			print_args ("0", save_buf, NULL, NULL, NULL);
			token_type = get_token();
			ret_value = SUCCESS;}
      else
        {save_cnt = 0; /* parameters() uses list_stack */
         list_pt  = 0;
			print_opcode("stack_scope_",T_STACK_SCOPE,T_NONE,T_NONE);
			print_args(symbol_table[LP_TRUE].tokpts,NULL,NULL,NULL,NULL);
			while(token_type != T_EOS && ret_value != FAIL)
				ret_value = parameters();
			print_opcode("pop_scope_",T_POP_SCOPE,T_NONE,T_NONE);
			print_args(NULL,NULL,NULL,NULL,NULL);
			print_opcode("call_",T_CALL,T_NONE,T_NONE);
			print_args("0", save_buf, NULL,NULL,NULL);
			print_opcode("stack_scope_",T_STACK_SCOPE,T_NONE,T_NONE);
			print_args(symbol_table[LP_TRUE].tokpts,NULL,NULL,NULL,NULL);
	      while (0 < save_cnt) /* generate return quadruples  */
            {garg8=list_stack[--save_cnt];
             garg7=list_stack[--save_cnt];
             garg6=list_stack[--save_cnt];
             garg5=list_stack[--save_cnt];
             garg4=list_stack[--save_cnt];
             garg3=list_stack[--save_cnt];
             garg2=list_stack[--save_cnt];
             garg1=list_stack[--save_cnt];
             genmove(garg1,garg2,garg3,garg4,garg5,garg6,garg7,garg8,T_FROM);}
			print_opcode("pop_scope_",T_POP_SCOPE,T_NONE,T_NONE);
			print_args(NULL,NULL,NULL,NULL,NULL);
         if (token_type != T_EOS)
           {print_msg(EOS_ERR, token_buf);
            ret_value = FAIL;}
         else
            token_type = get_token();
        }
     }
   else
     {print_msg(MISSUB_ERR, NULL);
      ret_value = FAIL;}

   for(i=INT_SCA_TEMP_START; i<sca_temp_start; i++)
		{print_opcode ("pop_", T_POP, T_NONE, symbol_table[i].mode2);
		 print_args(off_exp1(i,SUCCESS),NULL,NULL,NULL,NULL); }
   for(i=LOG_PAR_TEMP_START; i<log_par_temp_start; i++)
		{print_opcode ("pop_", T_POP, T_NONE, symbol_table[i].mode2);
		 print_args(off_exp1(i,SUCCESS),NULL,NULL,NULL,NULL); }

   if (debug == 2)
      printf("call_() returning ret_value %d\n", ret_value);

   return(ret_value);

}   /* end of call_() */



