/* P1_DECL_TABLE 910415 */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"


/* ===========================================================================

   function name: install_scalar
   input	       : arg1		- pointer to scalar identifier name
		            arg2		- pointer to repeat count for this identifier
		            arg3		- pointer to initial value for this identifier
		            arg4     - pointer to mode
		            arg5     - pointer to width of scalar
		            arg6     - pointer to expt (exponent) width for real
                  arg7     - pointer to signif (mantisa) width for real
		            arg8     - TRUE if T_TEMP, FALSE otherwise
						arg9     - io shape
   output	    : ---
   return value : ---

   global vars  : scalar_table	- array to store information for scalar IDs
		            total_scalars	- total number of entries in 'scalar_table'

   description  : This routine stores the information for the scalar
		            variables during the compilation of the source code. It
		            will store this information temporarily in 'scalar_table'
		            until it is needed.

=========================================================================== */

void install_scalar(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9)
char *arg1, *arg2, *arg3, *arg4, *arg5, *arg6, *arg7;
int arg8, arg9;
{
  int mode1,mode2;
  char c_string[32], c_arg3[32];

   if (debug == 2)
     printf("In install_scalar()\n");
   if(total_scalars >= MAX_SCALARS)
		{printf("%d ERROR - scalar table overflow in p1_decl_table at %s\n",
					linenum, arg1);
		 exit(1);}

   scalar_table[total_scalars].arg1 =
 			     	(char *)calloc(strlen(arg1)+1,sizeof(char));
   strcpy(scalar_table[total_scalars].arg1, arg1);

   scalar_table[total_scalars].arg2 =
 				(char *)calloc(strlen(arg2)+1,sizeof(char));
   strcpy(scalar_table[total_scalars].arg2, arg2);

   scalar_table[total_scalars].arg3 =
					(char *)calloc(strlen(arg3)+1,sizeof(char));
   strcpy(scalar_table[total_scalars].arg3, arg3);

   scalar_table[total_scalars].arg4 =
				(char *)calloc(strlen(arg4)+1,sizeof(char));
   strcpy(scalar_table[total_scalars].arg4, arg4);

   scalar_table[total_scalars].arg5 =
				(char *)calloc(strlen(arg5)+1,sizeof(char));
   strcpy(scalar_table[total_scalars].arg5, arg5);

   scalar_table[total_scalars].arg6 =
				(char *)calloc(strlen(arg6)+1,sizeof(char));
   strcpy(scalar_table[total_scalars].arg6, arg6);

   scalar_table[total_scalars].arg7 =
				(char *)calloc(strlen(arg7)+1,sizeof(char));
   strcpy(scalar_table[total_scalars].arg7, arg7);

   scalar_table[total_scalars].arg8 = arg8;

   scalar_table[total_scalars].arg9 = arg9;

   strcpy(c_arg3,scalar_table[total_scalars].arg3);
   if(arg9 == FAIL)
		strcpy(c_string,"CS$");
	else if (arg9 == SUCCESS)
		{strcpy(c_string,"CS$");
		 strcpy(c_arg3,"CS$");}
	else strcpy(c_string,symbol_table[scalar_table[total_scalars].arg9].tokpts);

   if((code_type==EMULATE  &&
	   (module_flag != T_SUBROUTINE || in_subody == TRUE)) ||
		 code_type==CM)
     {mode1 = atoi(scalar_table[total_scalars].arg4) / SIZE_MODE2;
      mode2 = atoi(scalar_table[total_scalars].arg4) % SIZE_MODE2;
      if(arg8 == TRUE)
        print_opcode("temp_",T_TEMP,mode1,mode2);
      else
        print_opcode("decl_",T_DECLARE,mode1,mode2);
      if(mode2 == T_CNST)
        print_args(scalar_table[total_scalars].arg1, scalar_table[total_scalars].arg2,
           c_arg3,
           scalar_table[total_scalars].arg6, scalar_table[total_scalars].arg7);
      else
        print_args6(scalar_table[total_scalars].arg1, scalar_table[total_scalars].arg2,
           scalar_table[total_scalars].arg3,
           scalar_table[total_scalars].arg6, scalar_table[total_scalars].arg7,
           c_string);
	  }

   total_scalars++;

   if (debug == 2)
      printf ("Returning from install_scalar()\n");

} /* end of install_scalar() */



/* ===========================================================================

   function name : dump_scalar
   input	        : ---
   output        : ---
   return value  : ---

   global vars   : scalar_table	- array for storing scalar variables
		             total_scalars	- total number of entries in 'scalar_table'

   description	  : This routine dumps all the declarations for scalar variables
		             at the end of the listing and object file after the source
		             program has been compiled.

=========================================================================== */

void dump_scalar()
{
   int i;
   int mode1,mode2;

   if (debug == 2)
      printf("In dump_scalar()..........................\n");

 if(code_type!=EMULATE  && code_type!=CM)
   {for (i = 0; i < total_scalars; i++)
      {mode1 = atoi(scalar_table[i].arg4) / SIZE_MODE2;
       mode2 = atoi(scalar_table[i].arg4) % SIZE_MODE2;
       if(scalar_table[i].arg8 == TRUE)
         print_opcode("temp_",T_TEMP,mode1,mode2);
       else
         print_opcode("decl_",T_DECLARE,mode1,mode2);
       print_args6(scalar_table[i].arg1, scalar_table[i].arg2,
						scalar_table[i].arg3, scalar_table[i].arg6,
						scalar_table[i].arg7,
						symbol_table[scalar_table[i].arg9].tokpts);
      }
   }
   if (debug == 2)
      printf("Returning from dump_scalar()\n");

} /* end of dump_scalar() */
