/* P1_DEFINE.C 910415 */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"

/* ===========================================================================

   function name: define
   input	       : ---
   output	    : ---
   return value : return_value	- SUCCESS, if no errors detected
		  		                     - FAIL, otherwise

   global vars  : token_type	- type of current token
		            debug		- flag for debugging statements
		            token_size	- size of current token
		            token_buf	- character string for current token
		            symbol_table	- array containing symbol table

   description	 : This routine is part of a recursive descent parser and
		            handles the following production from the grammar:

			DEFINE <-- define ( identifier , constant ) ; DEFINE |
				   @

		  '@' represents the empty production. The identifier is
		  inserted in the symbol table as a defined constant. FAIL
		  is returned if a syntax error occurs or if the identifier
		  cannot be inserted in the symbol table. SUCCESS is returned
		  otherwise.

=========================================================================== */

int define(ex_vl)
int ex_vl;
{
   int return_value;		/* value returned by function */
   int arg1_index;		/* symbol table index of defined identifier */
   int id_size;			/* copy of token size of identifier */
   int mode1,mode2;
   char id_buf[TOKEN_SIZE];	/* copy of token buffer of identifier */
   char c_mode[10];
   char c_longint[10];
	int dummy;

   if (debug == 2)
      printf ("In define()...........................\n");

   return_value = SUCCESS;
   if(ex_vl == 2) return(return_value); /* multi program file, exit */

   if (token_type == T_DEFINE)
     {if ((token_type = get_token()) != T_LPAR)
        {print_msg (LPAR_ERR, token_buf);
	      return_value = FAIL;}
      else if ((token_type = get_token()) != T_ID)
        {print_msg (ID_ERR, token_buf);
	      return_value = FAIL;}
      else
        {id_size = token_size;
	      strcpy(id_buf, token_buf);
         if ((token_type = get_token()) == T_COMMA) token_type = get_token();
	      if (token_type == T_BIN_CONST || token_type == T_OCT_CONST ||
		       token_type == T_DEC_CONST || token_type == T_HEX_CONST)
	        {mode1 = T_INT; mode2 = T_CNST;
                 dummy = 1;
	         arg1_index = symtab_insert(T_DEF_CONST, mode1, mode2,
	                       id_size, id_buf, &dummy, W_INT, 0, W_INT, 0);
	         if (arg1_index != FAIL)
	           {pc_itoa(mode1*SIZE_MODE2+mode2,c_mode);
		         pc_itoa(W_INT,c_longint);
		         install_scalar(id_buf, "1", token_buf,c_mode,c_longint,
				                  "0",c_longint,FALSE,shapedx);
					if(code_type != EMULATE || module_flag != T_SUBROUTINE)
						{if (module_flag == T_MAIN) print_opcode("entry_", T_ENTRY, NULL, NULL);
						 else print_opcode("extern_", T_EXTERN, T_NONE, T_NONE);
						 print_args(id_buf, NULL, NULL, NULL, NULL);}
		         if ((token_type = get_token()) != T_RPAR)
	              {print_msg (RPAR_ERR, token_buf);
		            return_value = FAIL;}
		         else if ((token_type = get_token()) != T_EOS)
		           {print_msg (EOS_ERR, token_buf);
		            return_value = FAIL;}
 		         else
		           {token_type = get_token();
		            return_value = SUCCESS;}
              }
	         else
	           return_value = FAIL;
           }
         else
	        {print_msg (CONST_ERR, token_buf);
	         return_value = FAIL;}
       }
   }

   if (debug == 2)
      printf("Define() returning return_value %d\n", return_value);

   return(return_value);
}
