/* P1_DEFVAR.C 910415 */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"

/* ===========================================================================

   function name: defvar
   input	       : ---
   output	    : ---
   return value : return_value	- SUCCESS, if no errors detected
		  		                     - FAIL, otherwise

   global vars  : token_type	- type of current token
		            debug		- flag for debugging statements
		            token_size	- size of current token
		            token_buf	- character string for current token
		            symbol_table	- array containing symbol table
                  defvar_table  - an array of equivalenced variables

   description	 : This routine is part of a recursive descent parser and
		            handles the following production from the grammar:

		DEFVAR <-- defvar ( identifiera , identifierb ) ; DEFVAR |
				   @

		  '@' represents the empty production. The identifiers are
		  inserted in the defvar table. FAIL is returned if a syntax
		  error occurs or if the identifiers cannot be inserted in
		  the defvar table. SUCCESS is returned otherwise. The
		  identifiers must be declared in other statements.

=========================================================================== */

int defvar()
{
   int return_value;		/* value returned by function */
   int ret_value;               /* added 880603 jlp */
   char c_opb[10];
   char c_cnb[10];
   int i;

   if (debug == 2)
      printf ("In defvar()...........................\n");

   return_value = SUCCESS;

   if (token_type == T_DEFVAR)
     {if ((token_type = get_token()) != T_LPAR)
        {print_msg (LPAR_ERR, token_buf);
	      return_value = FAIL;}
      else if ((token_type = get_token()) != T_ID)
        {print_msg (ID_ERR, token_buf);
	      return_value = FAIL;}
      else if ( defvar_cnt < MAX_DEFVAR)
        /* check for duplicate entries in defvar_table */
        {for(i=0; i<defvar_cnt && return_value != FAIL; i++)
 	        if(strcmp(defvar_table[i].ida,token_buf) == 0 )
	          {printf("ERROR - %s multiply defvared\n",token_buf);
              return_value = FAIL;}
           if(return_value != FAIL)
             {defvar_table[defvar_cnt].opa = 0; /* init ops to zero */
	           defvar_table[defvar_cnt].opb = 0;
              defvar_table[defvar_cnt].ida = (char *) calloc(token_size+1,sizeof(char));
	           strcpy(defvar_table[defvar_cnt].ida, token_buf);
              if ((token_type = get_token()) == T_COMMA) token_type = get_token();
	           else if (token_type == T_SUB || token_type == T_ADD)
	             {defvar_table[defvar_cnt].opa  = token_type;
	              token_type = get_token();
	              if (token_type == T_DEC_CONST || token_type == T_OCT_CONST ||
	                  token_type == T_HEX_CONST || token_type == T_BIN_CONST )
	                 defvar_table[defvar_cnt].cna  = atoi(token_buf);
                 else if ((ret_value = define_fetch(token_buf)) != FAIL)
                    defvar_table[defvar_cnt].cna=atoi(scalar_table[ret_value].arg3);
	              else {print_msg(UNDEF_ERR, token_buf); return_value=FAIL;}
                 if((token_type=get_token())==T_COMMA) token_type = get_token();}
              if(token_type == T_DEC_CONST || token_type == T_OCT_CONST ||
	              token_type == T_HEX_CONST || token_type == T_BIN_CONST )
                 {if(code_type != EMULATE || module_flag != T_SUBROUTINE)
						  {print_opcode("defvar_",T_DEFVAR, T_NONE, T_NONE);
							print_args(defvar_table[defvar_cnt].ida,token_buf,"0","0",NULL);}
						token_type = get_token();}
	           else if (token_type != T_ID)
	             {print_msg (ID_ERR, token_buf); return_value = FAIL; }
	           else
                {defvar_table[defvar_cnt].idb = (char *) calloc(token_size+1,sizeof(char));
	              strcpy(defvar_table[defvar_cnt].idb, token_buf);
	              if ((token_type=get_token()) == T_SUB || token_type == T_ADD)
 	                {defvar_table[defvar_cnt].opb  = token_type;
	                 token_type = get_token();
	                 if(token_type == T_DEC_CONST || token_type == T_OCT_CONST ||
	                    token_type == T_HEX_CONST || token_type == T_BIN_CONST )
	                   {defvar_table[defvar_cnt].cnb  = atoi(token_buf);
	                    token_type = get_token();}
                 else if ((ret_value = define_fetch(token_buf)) != FAIL)
                   {defvar_table[defvar_cnt].cna=atoi(scalar_table[ret_value].arg3);
	                 token_type = get_token();}
	              else {print_msg(UNDEF_ERR, token_buf); return_value=FAIL;}
				    }
	           /* normalize defvar entries to a,nill,nill = b + (signed num) */
              if(defvar_table[defvar_cnt].opb == T_ADD)
                 if(defvar_table[defvar_cnt].opa == T_ADD)
		             defvar_table[defvar_cnt].cnb =
                        defvar_table[defvar_cnt].cnb-defvar_table[defvar_cnt].cna;
                 else
		             defvar_table[defvar_cnt].cnb =
                       defvar_table[defvar_cnt].cnb+defvar_table[defvar_cnt].cna;
	           else if(defvar_table[defvar_cnt].opa == T_ADD)
		           defvar_table[defvar_cnt].cnb =
                      -defvar_table[defvar_cnt].cnb-defvar_table[defvar_cnt].cna;
              else
		           defvar_table[defvar_cnt].cnb =
                      -defvar_table[defvar_cnt].cnb+defvar_table[defvar_cnt].cna;
	           defvar_table[defvar_cnt].opb = T_ADD;
              if(code_type != EMULATE || module_flag != T_SUBROUTINE)
					 {print_opcode("defvar_", T_DEFVAR, T_NONE, T_NONE);
	              if (defvar_table[defvar_cnt].opa == 0 &&
	                  defvar_table[defvar_cnt].opb == 0)
                    print_args(defvar_table[defvar_cnt].ida,
		                         defvar_table[defvar_cnt].idb,"0","0",NULL);
	              else
	                {pc_itoa(defvar_table[defvar_cnt].opb,c_opb);
	                 pc_itoa(defvar_table[defvar_cnt].cnb,c_cnb);
                    print_args(defvar_table[defvar_cnt].ida,
		                         defvar_table[defvar_cnt].idb,c_opb,c_cnb, NULL);}
					 }
              defvar_cnt++;
             }
           if (token_type != T_RPAR)
	          {print_msg (RPAR_ERR, token_buf);
		        return_value = FAIL;}
	        else if ((token_type = get_token()) != T_EOS)
		       {print_msg (EOS_ERR, token_buf);
		        return_value = FAIL;}
 	        else
		       {token_type = get_token();
		        return_value = SUCCESS;}
        }
     }
   else printf("ERROR - too many defvars\n");
  }

  if (debug == 2)
     printf("Defvar() returning return_value %d\n", return_value);

  return(return_value);
}
