/* P1_ELEMENT.C 910520 */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"

#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"
#include "ace2.h"

#define C_UNDER 95
#define SZSC  4
#define BITS  (SZSC*6)
#define WDS  BITS/SIZE_HOST+1
/* from table_io.h for ACE */
#define T_ROW   20
#define T_COL   21
#define T_ROWCOL 22
#define T_TAB    23
#define T_TABCOL 24
#define T_FUN 25
#define T_CONST 26
#define REAL_SIZE 32  /* same as em_commo.h */

extern char arg_str_ff[100];
char *fun_fun();
int fun_type1;  /* set by fun_fun for gen_fun_type */
int fun_type2;  /* set by fun_fun for gen_fun_type */

/* ===========================================================================

   function name: element
   input               : ---
   output           : result_index, result_offset
   return value : result_index  - symbol table index of result of the parse
						     - FAIL if errors detected

   global vars  : token_type    - type of current token
			    symbol_table        - array containing symbol table
			    token_buf   - character string for current token
			    token_size  - size of current token
			    debug               - flag for debugging statements

   description   : This routine is part of a recursive descent parser and
			    handles the following production from the grammar:

			ELEMENT <-- + ELEMENT
				      |  - ELEMENT
				      |  ( LOG_EXP )
				      |  identifier MORE_IDENTIFIER
				      |  index
				      |  defined constant
				      |  constant
				      |  unary functions
				      |  unary address

		  If a unary plus or minus is found, intermediate code is
		  generated after a call to 'ELEMENT' returns the index
		  of the item to be operated on. The index of the final
		  result is returned. If a left parenthesis is found, no code
		  is generated for the item returned by the call to
		  'log_exp'. The symbol table index of the item is returned.
		  If an identifier, defined constant, constant, or index
		  is found, no code is generated and the symbol table index
		  is returned.

=========================================================================== */
#define T_BAR 124 /* ascii | - used by ACE */
int acetype;   /* used in element and fun_conv */

int element(result_index,result_offset)
int *result_index,*result_offset;
{
   int arg1_index;              /* symbol table index of argument 1, if any */
   int arg1_offset;     /* offset of arg1 */
   int op_type;                 /* token type of operator (plus or minus) */
   int minmax_index;            /* symbol table index for the unary function */
   int minmax_offset;   /* offset of minmax */
   int nth_index;               /* nthvalue index  */
   int nth_offset;      /* offset of nthvalue */
   unsigned int minmax_type;            /* type of unary function */
   int minmax_result;   /* symbol table index for the result of unary
					       function : MINDEX and MAXDEX */
   int minmax_scalar;   /* symbol table index for the result of the
					       unary function : MINVAL and MAXVAL */
	char arg_str[100];
	char arg_str2[100];
   int table;
   int indx1,ofst1;
   char *chrptr;

   if (debug == 2)
      {printf ("In element()..................................\n");
       printf("Token_type=%d, token_buf=%s\n",token_type,token_buf);};

   *result_index = FAIL;
   *result_offset = SUCCESS;

aceloop:  /* reentry for ace multi word functions */
   switch(token_type)
   {case T_PLUS:
    case T_MINUS:
      op_type = token_type;
      token_type = get_token();
      if (element(&arg1_index,&arg1_offset) != FAIL)
	{if (symbol_table[arg1_index].mode1 != T_REAL &&
	     symbol_table[arg1_index].mode1 != T_OCT  &&
	     symbol_table[arg1_index].mode1 != T_HEX  &&
	     symbol_table[arg1_index].mode1 != T_LONGINT  &&
	     symbol_table[arg1_index].mode1 != T_BIN  &&
	     symbol_table[arg1_index].mode1 != T_CHAR &&
	     symbol_table[arg1_index].mode1 != T_CARD &&
	     symbol_table[arg1_index].mode1 != T_INT)
			  {print_msg (EXPCON_ERR, token_buf);
			   printf("In p1_element. arg1.mode1=%d\n",
			   symbol_table[arg1_index].mode1 );}
	 else
	   {*result_index = get_symtab_temp (symbol_table[arg1_index].mode1,
								      symbol_table[arg1_index].mode2,
					     symbol_table[arg1_index].width);
		 if (*result_index != FAIL)
		   {if (op_type == T_PLUS)
		      print_opcode ("pls_", T_PLUS, symbol_table[arg1_index].mode1,
								   symbol_table[arg1_index].mode2);
		    else
		      print_opcode ("min_", T_MINUS,symbol_table[arg1_index].mode1,
						   symbol_table[arg1_index].mode2);
		    print_args (off_exp1(arg1_index,arg1_offset),
			  off_exp2(*result_index,*result_offset),NULL,NULL,NULL);
		   }
		}
	}
      break;
    case T_LPAR:
      token_type = get_token();
      if(log_exp(result_index,result_offset) != FAIL)
	{if (token_type == T_RPAR)
		token_type = get_token();
	      else
		{print_msg (RPAR_ERR, token_buf);
		 *result_index = FAIL;
		 *result_offset = SUCCESS;}
	}
      break;
/* Since ACE allows multi word functions, the initial word will alwasy
   be T_ID.  If an ACE function is recognized by aceident, a special
   flag is set so that it will loop to the top of element and switch
   to the correct ACE case.  Thus, SQUARE will be T_ID, aceident will
   process ROOT to produce a token_type of T_SQRT and ace_type T_FUN */
    case T_ID:
      table = FAIL;
      if ((arg1_index = symtab_fetch(token_size, token_buf)) == FAIL)
	   {if((*result_index = aceid()) == FAIL)
	       print_msg (UNDEF_ERR, token_buf);
	    *result_offset = SUCCESS;
	    break;}
      else if (symbol_table[arg1_index].toktype == T_ACE)
	   {*result_index = arg1_index; *result_offset = SUCCESS;
	    /* acetype is set here only.  It is not cleared, because
	       it is used in fun_conv to adjust for ace name parsing
	       which stops at (, while asc1 functions stop on the
	       keyword */
	    if((acetype=aceident(&arg1_index,&table)) == FAIL)
		{/* look for combined row-col and tab-col names */
		 rowcolident(arg1_index,&indx1,&ofst1,T_ID,&table);
		 *result_index = indx1;
		 *result_offset = ofst1;
if(indx1 == FAIL) printf("%d ERROR - unrecognized ACE identifier preceding %s\n",
linenum,token_buf);
		 break;}
	    else if(acetype == T_COL)
	       {if(token_type == T_PAR_SUBSCRIPT) token_type = get_token();
		*result_index = arg1_index; *result_offset = SUCCESS;
		break;}
	    else if(acetype == T_ROW)
		{if((acetype = more_rowident(arg1_index,result_index,result_offset,&table))
		     == T_COL)
		     printf("In p1_eleme, at %d acetype = T_COL\n",linenum);}
	    else if(acetype == T_TAB)
		more_tabident(arg1_index,result_index,result_offset,&table);
	    else if(acetype == T_ID) /* UNKNOWN */
		more_identifier (arg1_index,result_index,result_offset);
	    else if(acetype ==  T_FUN)
		{*result_index = arg1_index; *result_offset = SUCCESS;
		/* since functions have no table, argument is used for
		   token_type */
		token_type = table;
		 }
	    break;}
      else
	{token_type = get_token();
	 more_identifier (arg1_index,result_index,result_offset);}
if(table != FAIL)
   printf(" %d WARNING - table set to %d in p1_element\n",linenum,table);
      break;
    case T_SID:
/*      if (check_dol() == FAIL)
	   {printf("%d ERROR - check_dol, unrecognized system temp %s\n",
					     linenum,token_buf);
	    print_msg (UNDEF_ERR, token_buf);
				break;}    */
/*      *result_index = INT_PAR_TEMP_START; */
      *result_index = get_const(token_buf,/*token_type*/T_STRING);
//printf("T_SID, token=%s, result=%s\n",token_buf,symbol_table[*result_index].tokpts);
		token_type = get_token();
      break;
    case T_STRING:
    case T_DEF_CONST:
    case T_BIN_CONST:
    case T_OCT_CONST:
    case T_DEC_CONST:
    case T_HEX_CONST:
	 case T_FLT_CONST:
      *result_index = get_const(token_buf,token_type);
		token_type = get_token();
	 /* set to prevent looping at bottom */
	 if(acetype == T_FUN) acetype = T_CONST;
      break;
	 case T_MAXDEX:
    case T_MINDEX:
    case T_MAXVAL:
    case T_MINVAL:
      minmax_type = token_type;
      token_type = get_token();
      if (element(&minmax_index,&minmax_offset) != FAIL)
	{if ((symbol_table[minmax_index].mode1 != T_INT  &&
	      symbol_table[minmax_index].mode1 != T_REAL &&
	      symbol_table[minmax_index].mode1 != T_HEX  &&
	      symbol_table[minmax_index].mode1 != T_BIN  &&
	      symbol_table[arg1_index].mode1 != T_LONGINT  &&
	      symbol_table[minmax_index].mode1 != T_OCT  &&
	      symbol_table[minmax_index].mode1 != T_CARD) ||
	     (symbol_table[minmax_index].mode2 != T_PARALLEL))
	    print_msg(MINMAX_ERR, NULL);
	 else
	   {if ((minmax_result =
			      get_symtab_temp(T_LOGICAL, T_PARALLEL, W_LOGICAL)) !=  FAIL)
	      {switch (minmax_type)
		 {case T_MAXVAL:
						case T_MAXDEX: print_opcode("maxdex", T_MAXDEX,
					      symbol_table[minmax_index].mode1,
															 symbol_table[minmax_index].mode2);
				 print_args(off_exp1(minmax_index,minmax_offset),
								off_exp2(minmax_result,SUCCESS),
								     NULL, NULL, NULL);
				 break;
		   case T_MINVAL:
		   case T_MINDEX: print_opcode("mindex", T_MINDEX,
					       symbol_table[minmax_index].mode1,
															  symbol_table[minmax_index].mode2);
						 print_args(off_exp1(minmax_index,minmax_offset),
								 off_exp2(minmax_result,SUCCESS),
								      NULL, NULL, NULL);
				  break;
		  }
	       *result_index = minmax_result;
	       if(minmax_type == T_MINVAL ||
						minmax_type == T_MAXVAL)
						{if((minmax_scalar =
						   get_symtab_temp(symbol_table[minmax_index].mode1,T_SCALAR,T_NONE)) != FAIL)
		     {if(minmax_type == T_MINVAL)
								{print_opcode("minval",minmax_type,
												  symbol_table[minmax_index].mode1,
												  symbol_table[minmax_index].mode2);
						       print_args(off_exp1(minmax_index,minmax_offset),
							off_exp2(minmax_scalar,SUCCESS),
							NULL, NULL, NULL);}
							 else
								{print_opcode("maxval",minmax_type,
												  symbol_table[minmax_index].mode1,
												  symbol_table[minmax_index].mode2);
						       print_args(off_exp1(minmax_index,minmax_offset),
							off_exp2(minmax_scalar,SUCCESS),
							NULL, NULL, NULL);}
						  *result_index = minmax_scalar;
		   }
					 }
	     }
	  }
       }
 else printf("p1_element failed\n");
      break;
    case T_NTHVAL:
    case T_NTHDEX:
      minmax_type = token_type;
      token_type = get_token();
      if(token_type == T_LPAR) token_type = get_token();
		else printf("%d - ERROR left paranthesis expected in max/min function\n",linenum);
      if (element(&minmax_index,&minmax_offset) != FAIL)
	{if ((symbol_table[minmax_index].mode1 != T_INT  &&
	      symbol_table[minmax_index].mode1 != T_REAL &&
	      symbol_table[minmax_index].mode1 != T_HEX  &&
	      symbol_table[minmax_index].mode1 != T_BIN  &&
	      symbol_table[arg1_index].mode1 != T_LONGINT  &&
	      symbol_table[minmax_index].mode1 != T_OCT  &&
	      symbol_table[minmax_index].mode1 != T_CARD) ||
	     (symbol_table[minmax_index].mode2 != T_PARALLEL))
	    print_msg(MINMAX_ERR, NULL);
	 else
	   {if ((minmax_result =
			      get_symtab_temp(T_LOGICAL, T_PARALLEL, W_LOGICAL)) !=  FAIL)
	       {if (token_type == T_COMMA) token_type = get_token();
			       if (element(&nth_index,&nth_offset) != FAIL)
		    {print_opcode("nthdex", T_NTHDEX, symbol_table[minmax_index].mode1,
							     symbol_table[minmax_index].mode2);
				    print_args(off_exp1(minmax_index,minmax_offset),
					      off_exp2(minmax_result,SUCCESS),
					      off_exp3(nth_index,nth_offset), NULL, NULL);}
				    else
					    printf("%d ERROR -  nth value missing from nthdex\n",linenum);
	       if(token_type == T_RPAR) token_type = get_token();
	       else printf("%d - ERROR right paranthesis expected in max/min function. %s\n",
										  linenum,token_buf);
				  }
				 else
					  printf("%d ERROR in nthdex 1\n",linenum);
		     *result_index = minmax_result;
			  }
	}
		  else {printf("%d ERROR in nthdex 2\n",linenum); token_type = get_token();}
	if(minmax_type == T_NTHVAL)
		  {if((minmax_scalar =
			    get_symtab_temp(symbol_table[minmax_index].mode1,T_SCALAR,T_NONE)) != FAIL)
	     {print_opcode("nthval",minmax_type,
							  symbol_table[minmax_index].mode1,
							  symbol_table[minmax_index].mode2);
			   print_args(off_exp1(minmax_index,minmax_offset),
				    off_exp2(minmax_scalar,SUCCESS),
				    off_exp3(nth_index,nth_offset), NULL, NULL);}
		     *result_index = minmax_scalar;
	}
		  break;
    case T_OPEN:
	   *result_index = gen_fun("open_",T_OPEN,T_INT,T_SCALAR,2);
      break;
    case T_COPY:
      strcpy(arg_str,fun_fun());
      print_opcode("copy_",T_COPY,NULL,T_PARALLEL);
      print_args(arg_str,NULL,NULL,NULL,NULL);
      *result_index = SUCCESS;
      break;
    case T_ADDRESS: printf ("In address @ statement\n");
      token_type = get_token();
      *result_index = address_();
      break;
    case T_CLOSE:
      strcpy(arg_str,fun_fun());
      print_opcode("close_",T_CLOSE,NULL,NULL);
      print_args(arg_str,NULL,NULL,NULL,NULL);
      *result_index = SUCCESS;
      break;
    case T_LOAD:
      strcpy(arg_str,fun_fun());
	  chrptr = strrchr(arg_str,' '); /* find terminating space */
	  *chrptr = 0;
	  chrptr = strrchr(arg_str,' '); /* find PENULTAMATE space */
	  *chrptr = 0;
	  strcpy(arg_str2,chrptr+1);
	  print_opcode("decl_",T_DECLARE,T_INT,T_CNST);
	  print_args("INPT$", "1", "4", "0", "16");
      print_opcode("open_",T_OPEN,NULL,NULL);
      print_args(arg_str,"INPT$","HANDLE$,C0D$","HANDLE$,C1D$",NULL);
      print_opcode("load_",T_LOAD,NULL,NULL);
      print_args("HANDLE$,C0D$",arg_str2,NULL,NULL,NULL);
      print_opcode("close_",T_CLOSE,NULL,NULL);
      print_args("HANDLE$,C0D$",NULL,NULL,NULL,NULL);
      *result_index = SUCCESS;
      break;
    case T_STORE:
      strcpy(arg_str,fun_fun());
	  chrptr = strrchr(arg_str,' '); /* find terminating space */
	  *chrptr = 0;
	  chrptr = strrchr(arg_str,' '); /* find PENULTAMATE space */
	  *chrptr = 0;
	  strcpy(arg_str2,chrptr+1);
	  print_opcode("decl_",T_DECLARE,T_INT,T_CNST);
	  print_args("OTPT$", "1", "2", "0", "16");
      print_opcode("open_",T_OPEN,NULL,NULL);
      print_args(arg_str,"OTPT$","HANDLE$,C0D$","HANDLE$,C1D$",NULL);
      print_opcode("store_",T_STORE,NULL,NULL);
      print_args("HANDLE$,C0D$",arg_str2,NULL,NULL,NULL);
      print_opcode("close_",T_CLOSE,NULL,NULL);
      print_args("HANDLE$,C0D$",NULL,NULL,NULL,NULL);
      *result_index = SUCCESS;
      break;
    case T_GETADD:
		*result_index = gen_fun("getadd_",T_GETADD,T_INT,T_SCALAR,WDS); break;
    case T_DECLAR:
		*result_index = gen_fun("declar_",T_DECLAR,T_INT,T_SCALAR,WDS); break;
    case T_FSTCD:
		*result_index = gen_fun("fstcd_",T_FSTCD,T_INT,T_SCALAR,WDS); break;
    case T_NXTCD:
		*result_index = gen_fun("nxtcd_",T_NXTCD,T_INT,T_SCALAR,WDS); break;
    case T_PREVCD:
		*result_index = gen_fun("prevcd_",T_PREVCD,T_INT,T_SCALAR,WDS); break;
    case T_TRNCD:
		*result_index = gen_fun("trncd_",T_TRNCD,T_INT,T_SCALAR,WDS); break;
    case T_TRNACD:
		*result_index = gen_fun("trnacd_",T_TRNACD,T_INT,T_SCALAR,WDS); break;
    case T_PREFIX:
		*result_index = gen_fun("prefix_",T_PREFIX,T_INT,T_SCALAR,WDS); break;
    case T_FSTCD8:
		*result_index = gen_fun("fstcd8_",T_FSTCD8,T_HEX,T_SCALAR,64); break;
    case T_NXTCD8:
		*result_index = gen_fun("nxtcd8_",T_NXTCD8,T_HEX,T_SCALAR,64); break;
    case T_PREVCD8:
		*result_index = gen_fun("prevcd8_",T_PREVCD8,T_HEX,T_SCALAR,64); break;
    case T_TRNCD8:
		*result_index = gen_fun("trncd8_",T_TRNCD8,T_HEX,T_SCALAR,64); break;
    case T_TRNACD8:
		*result_index = gen_fun("trnacd8_",T_TRNACD8,T_HEX,T_SCALAR,64); break;
    case T_PREFIX8:
		*result_index = gen_fun("prefix8_",T_PREFIX8,T_HEX,T_SCALAR,64); break;
    case T_HIT:
		*result_index = gen_fun_morph("hit_",T_HIT,T_INT,T_PARALLEL,16); break;
    case T_TKNCMP:
		*result_index = gen_fun_type("tkncmp_",T_TKNCMP,T_INT,T_PARALLEL,16); break;
    case T_SCIN:
		*result_index = gen_fun_scio("scin_",T_SCIN,T_INT,T_SCALAR,WDS); break;
    case T_SCINL:
		*result_index = gen_fun_scio("scinl_",T_SCINL,T_INT,T_SCALAR,WDS); break;
    case T_SCINP:
		*result_index = gen_fun_scio("scinp_",T_SCINP,T_INT,T_SCALAR,WDS); break;
    case T_SCIN8:
		*result_index = gen_fun_scio("scin8_",T_SCIN8,T_INT,T_SCALAR,WDS*8); break;
    case T_SCOT:
		*result_index = gen_fun_scio("scot_",T_SCOT,T_INT,T_SCALAR,WDS);   break;
    case T_SCOTL:
		*result_index = gen_fun_scio("scotl_",T_SCOTL,T_INT,T_SCALAR,WDS); break;
    case T_SCOTP:
		*result_index = gen_fun_scio("scotp_",T_SCOTP,T_INT,T_SCALAR,WDS); break;
    case T_SCOT8:
		*result_index = gen_fun_scio("scot8_",T_SCOT8,T_INT,T_SCALAR,WDS*2); break;
    case T_COUNT:
		*result_index = gen_fun("count_",T_COUNT,T_INT,T_SCALAR,0); break;
    case T_ENCODE:
		*result_index = gen_fun("encode_",T_ENCODE,T_CHAR,T_SCALAR,0); break;
    case T_DECODE:
		*result_index = gen_fun("decode_",T_DECODE,T_INT,T_SCALAR,0); break;
    case T_EXP:
		*result_index = gen_fun_conv("exp_",T_EXP,T_REAL,T_PARALLEL,REAL_SIZE,T_REAL); break;
    case T_SQRT:/* gen_fun_conv will convert arguments to T_REAL */
		*result_index = gen_fun_conv("sqrt_",T_SQRT,T_REAL,T_PARALLEL,REAL_SIZE,T_REAL); break;
    case T_SIN:/* gen_fun_conv will convert arguments to T_REAL */
		*result_index = gen_fun_conv("sin_",T_SIN,T_REAL,T_PARALLEL,REAL_SIZE,T_REAL); break;
    case T_COS:/* gen_fun_conv will convert arguments to T_REAL */
		*result_index = gen_fun_conv("cos_",T_COS,T_REAL,T_PARALLEL,REAL_SIZE,T_REAL); break;
    case T_RAND: /* rand does not take any arguments and assumed to be scalar */
                *result_index = gen_fun_rand("rand_",T_RAND,T_LONGINT,T_SCALAR,2); break;
    /* gen_fun_type sets type1, set to 0 below */
    case T_SUM:
		*result_index = gen_fun_type("sum_",T_SUM,0,T_SCALAR,0); break;
    case T_SIBDEX:
      if(wt == WT)
	      *result_index = gen_fun("sibdex_",T_SIBDEX,T_LOGICAL,T_PARALLEL,1);
		else
	      *result_index = gen_fun("sibdex_",T_SIBDEX,T_LOGICAL,T_PARALLEL,2);
      break;
	 case T_MSG:
		*result_index = msgin_();
		token_type = get_token();
		break;
	 case T_OUTPUT:
		*result_index = dump_();
		token_type = get_token();
		break;
	 case T_DINPUT:
		*result_index = dimp_();
		token_type = get_token();
		break;
	 case T_DUMPMSG:
		*result_index = dump_();
		token_type = get_token();
		break;
	 case C_UNDER:
	 case T_BAR:  /* an expression failure, but ok for ACE */
     *result_index = FAIL;  /* do not print error message */
      break;
    default:
      print_msg (ELEM_ERR, token_buf);
      printf(" token_type=%d\n",token_type);
     *result_index = FAIL;
      break;
    }
if(debug == 2) printf("At bottom of element, acetype=%d, token=%s, type=%d\n",acetype,token_buf,token_type);
   /* ace function found via T_ID, must repeat case to process araguments */
   /* leave acetype intact for fun_conv  */
   if(acetype == T_FUN) goto aceloop;

   if(debug == 2) if(*result_index == FAIL)
		printf("Element returning FAIL, token_buf=%s\n",token_buf);
	else
		printf("Element returning %s, token_buf=%s\n",
				  symbol_table[*result_index].tokpts,token_buf);
   if (debug == 2)
      printf ("Element() returning *result_index %d, *result_offset %d\n",
	      *result_index, *result_offset);

   return(*result_index);

} /* end of element() */

/* check token_buf for IPTMP, etc */
check_dol()
{
   int result;
   char foo_buf[TOKEN_SIZE+1];

   result = SUCCESS;
   strcpy(foo_buf,token_buf);
   strcat(foo_buf,"$+");
   if(strcmp(foo_buf,symbol_table[INT_PAR_TEMP_START].tokpts) != 0)
      result = FAIL;
/*printf("In check_dol, foo_buf=%s, iptmp=%s\n",foo_buf,
	symbol_table[INT_PAR_TEMP_START].tokpts);*/
	return(result);
 }


/* ===========================================================================

   function name: make_constname
   input         : ---
   output           : ---
   return value : const_name    - pointer to character name for constant

   description   : This routine generates a unique name for a constant.
			    The name consists of the letters 'const', followed by
			    a number, followed by '_'. Followed by D(decimal),
		  B(binary), H(hexidecimal) or O(octal).  A pointer to
		  the name is returned.
=========================================================================== */

char const_buffer[100];

char *make_constname(const_str,const_type)
char *const_str;
int const_type;
{
   char *const_name;            /* return value of function */


 if (debug == 2 || debug == 9)
      printf("In make_constname().  const_str=%s\n",const_str);

      const_name = &const_buffer[0];
      strcpy (const_name,"C");
      strcat (const_name,const_str);
      if(wt == WT)
       switch(const_type)
	     {default: print_msg(CONST_WRN,const_str);
	      case T_DEC_CONST: strcat(const_name,"D_"); break;
	      case T_HEX_CONST: strcat(const_name,"H_"); break;
	      case T_OCT_CONST: strcat(const_name,"O_"); break;
	      case T_BIN_CONST: strcat(const_name,"B_"); break;
	      case T_FLT_CONST: strcat(const_name,"F_"); break;
	      case T_STRING:    strcat(const_name,"S_"); break;
	}
      else switch(const_type)
	     {default: print_msg(CONST_WRN,const_str);
	 case T_DEC_CONST: strcat(const_name,"D$"); break;
	      case T_HEX_CONST: strcat(const_name,"H$"); break;
	      case T_OCT_CONST: strcat(const_name,"O$"); break;
	      case T_BIN_CONST: strcat(const_name,"B$"); break;
	      case T_FLT_CONST: strcat(const_name,"F$"); break;
	      case T_STRING:    strcat(const_name,"S$"); break;
	};

   if (debug == 2 || debug == 9)
      printf("make_constname() returning const_name : %s\n", const_name);

   return(const_name);

} /* end of make_constname() */


open_err()
{ /* OPEN_ERR - Syntax error in OPEN at */
  printf("ERROR - Syntac error in OPEN at %d\n",linenum);
  return(FAIL);
}

/* ******************************************
   Function fun_fun is a generic parser for system subroutines.
   It processes parameter strings reducing them to index,offset
   pairs.  It then expands them using off_exp1.  The expanded
   strings are concatinated together and returned as the result.
   If there is an error, the nil string is returned.
   ****************************************** */
/* int fun_type1;  set by fun_fun for gen_fun_type */
/* int fun_type2;  set by fun_fun to type two of first argument
		   for gen_fun_morph.  Thus the type of the call
		   scalar vs parallel can be determined by the first
		   argument. */
char *fun_fun()
{ int par_index,par_offset;
  int result;

  arg_str_ff[0] = 0;   /* nil string is error return */
  /* single word ACE functions, such as sum have already
     progressed to "(" */
  if (strcmp(token_buf,"(") != 0)
     if((token_type = get_token()) != T_LPAR)
	{printf("%d ERROR - in fun_fun, token=%s\n",linenum,token_buf);
	 return arg_str_ff;}

     token_type = get_token();
     strcpy(arg_str_ff,"");
     result = SUCCESS;
     fun_type2 = FAIL;
     while (token_type != T_RPAR && result != FAIL)
       {result = log_exp(&par_index,&par_offset);
	fun_type1 = symbol_table[par_index].mode1;
	if (fun_type2 == FAIL)
	   fun_type2 = symbol_table[par_index].mode2;
	strcat(arg_str_ff,off_exp1(par_index,par_offset));
	if(token_type == T_COMMA)
	  token_type = get_token();
	strcat(arg_str_ff," ");
       }
  token_type = get_token();
  return arg_str_ff;
}
/* same as fun_fun, except it converts all arguments to
   the type1 specified.  It returns type2 or arguments.*/
char *fun_conv(type1,type2)
int type1, *type2;
{ int par_index,par_offset;
  int result;
/*
printf("In fun_conv, acetype=%d, type1=%d, token=%s, token_type=%x\n",
acetype,type1,token_buf,token_type);
*/
  arg_str_ff[0] = 0;   /* nil string is error return */
  if(acetype != T_FUN)  token_type = get_token();
  if(strcmp(token_buf,"(") != 0)
    {if((token_type = get_token()) != T_LPAR)
	{printf("%d ERROR - in fun_conv, acetype=%d, token=%s\n",
	 linenum,acetype,token_buf);
	 return arg_str_ff;}}  /* changed from FAIL to avoid warning 9807 */
   else
    {strcpy(arg_str_ff,"");
     token_type = get_token();
     result = SUCCESS;
     while (token_type != T_RPAR && result != FAIL)
/*       {result = log_exp(&par_index,&par_offset);  */
       {result = sum_dif_exp(&par_index,&par_offset);
	fun_type1 = symbol_table[par_index].mode1;
	if(type1 != fun_type1)
	  if(type1 == T_REAL)
	     float_(&par_index,&par_offset);
	  else if (type1 == T_INT)
	     fix_(&par_index,&par_offset);
	  else printf("%d ERROR - can't convert arguments type1\n",linenum);
	strcat(arg_str_ff,off_exp1(par_index,par_offset));
	if(token_type == T_COMMA)
	  token_type = get_token();
	 else if(token_type != T_RPAR)
	   printf("%d WARNING - function arguments not seperated by a comma, token=%s\n",
		  linenum,token_buf);
	strcat(arg_str_ff," ");
		  }
	 }
  token_type = get_token();
  *type2 = symbol_table[par_index].mode2;

  return(arg_str_ff);
}

/* *result_index = gen_fun("prevcd_",T_PREVCD,T_INT,T_SCALAR,WDS); */
/* type1,type2,size are for returned value, pointed to by result_index */
gen_fun(carg1,arg2,type1,type2,size)
char *carg1;
int  arg2,type1,type2;
int  size; /* the number of scalar temps to reseve */
{
  int fun_dx;
  char *arg_str;

  fun_dx = get_symtab_temp(type1,type2,size);
  arg_str = fun_fun();
  if(lhs2)
    {rhs_index2 = get_symtab_temp(type1,type2,size);
     rhs_offset2 = SUCCESS;}
   print_opcode(carg1,arg2,NULL,type2);
   if(lhs2)
       print_args(arg_str,symbol_table[fun_dx].tokpts,
		  symbol_table[rhs_index2].tokpts,NULL,NULL);
     else
       print_args(arg_str,symbol_table[fun_dx].tokpts,NULL,NULL,NULL);
  return(fun_dx);
}
/* same as gen_fun, except puts type1 in print_opcode call */
gen_fun_rand(carg1,arg2,type1,type2,size)
char *carg1;
int  arg2,type1,type2;
int  size; /* the number of scalar temps to reseve */
{  int fun_dx;
  char *arg_str;
  fun_dx = get_symtab_temp(type1,type2,size);
  arg_str = fun_fun();
  if(lhs2)
    {rhs_index2 = get_symtab_temp(type1,type2,size);
     rhs_offset2 = SUCCESS;}
   print_opcode(carg1,arg2,type1,type2);
   if(lhs2)
       print_args(arg_str,symbol_table[fun_dx].tokpts,
		  symbol_table[rhs_index2].tokpts,NULL,NULL);
     else
       print_args(arg_str,symbol_table[fun_dx].tokpts,NULL,NULL,NULL);
  return(fun_dx);
}

/* same as gen_fun, but arguments are converted to type type */
/* by fun_conv for ACE functions                             */
gen_fun_conv(carg1,arg2,type1,type2,size,type)
char *carg1;
int  arg2,type1,type2,type;
int  size; /* the number of scalar temps to reseve */
{
  int fun_dx, type2a;
  char *arg_str;

if(debug == 2)
{printf("In gen_fun_conv, type1=%d, type2=%d, type=%d\n",type1,type2,type);
 printf("token=%s, token_type=5d\n",token_buf,token_type);}

  arg_str = fun_conv(type1,&type2a);
  fun_dx = get_symtab_temp(type1,type2a,size);
  if(lhs2)
    {rhs_index2 = get_symtab_temp(type1,type2a,size);
     rhs_offset2 = SUCCESS;}
   print_opcode(carg1,arg2,type1,type2a);
   if(lhs2)
       print_args(arg_str,symbol_table[fun_dx].tokpts,
		  symbol_table[rhs_index2].tokpts,NULL,NULL);
     else
       print_args(arg_str,symbol_table[fun_dx].tokpts,NULL,NULL,NULL);
  return(fun_dx);
}

/* gen_fun_morph sets type2 of the call to the type2 of the first arg */
gen_fun_morph(carg1,arg2,type1,type2,size)
char *carg1;
int  arg2,type1,type2,size;
{
  int fun_dx;
  char *arg_str;

  arg_str = fun_fun();
  fun_dx = get_symtab_temp(type1,type2,size);
  if(lhs2)
    {rhs_index2 = get_symtab_temp(type1,type2,size);
     rhs_offset2 = SUCCESS;}
  print_opcode(carg1,arg2,0,fun_type2);
  if(lhs2)
    print_args(arg_str,symbol_table[fun_dx].tokpts,
		symbol_table[rhs_index2].tokpts,NULL,NULL);
  else
    print_args(arg_str,symbol_table[fun_dx].tokpts,NULL,NULL,NULL);
  return(fun_dx);
}


/* type1 result of gen_fun_type is determined by type of argument
*/
gen_fun_type(carg1,arg2,type1,type2,size)
char *carg1;
int  arg2,type1,type2,size;
{
  int fun_dx;
  char *arg_str;

  arg_str = fun_fun();
  if(fun_type1 == T_LOGICAL) fun_type1 = T_INT;
  fun_dx = get_symtab_temp(fun_type1,type2,size);
  if(lhs2)
    {rhs_index2 = get_symtab_temp(fun_type1,type2,size);
     rhs_offset2 = SUCCESS;}
  print_opcode(carg1,arg2,fun_type1,type2);
  if(lhs2)
    print_args(arg_str,symbol_table[fun_dx].tokpts,
		symbol_table[rhs_index2].tokpts,NULL,NULL);
  else
    print_args(arg_str,symbol_table[fun_dx].tokpts,NULL,NULL,NULL);
  return(fun_dx);
}

/* this definition may be messed up, it origniall did not use type1, type2 or size */
gen_fun_scio(carg1,arg2,type1,type2, size)
char *carg1;
int  arg2, type1, type2, size;
{
  int fun_dx;
  char *arg_str;

  arg_str = fun_fun();
//  fun_dx = get_symtab_temp(T_LOGICAL,T_PARALLEL,W_LOGICAL);
  fun_dx = get_symtab_temp(T_LOGICAL,T_PARALLEL,size);
  print_opcode(carg1,arg2,type1,type2);
  print_args(arg_str,symbol_table[SC_IO_BUFFER].tokpts,
	     symbol_table[fun_dx].tokpts,NULL,NULL);
  return(fun_dx);
}


/* ******************************************
   Function sc_fun is a generic parser for structure code system
   subroutines. It processes first two parameters reducing them to
	index,offset pairs.  It then expands them using off_exp1.
   The last two parameters are ouput.
   If there is an error, the nil string is returned.
   ****************************************** */
sc_fun(carg1,arg2)
char *carg1;
int  arg2;
{
   int fpar_index,fpar_offset,spar_index,spar_offset;
   int result;
   int lresult_index,lresult_offset,rresult_index,rresult_offset;

   result = SUCCESS;
   if((token_type = get_token()) != T_LPAR)
	  printf("ERROR in system function call at line %d\n",linenum);
   else
    {token_type = get_token();
     if((result = log_exp(&fpar_index,&fpar_offset)) == FAIL)
       {printf("ERROR - in structure code function at %s\n",token_buf);
	return(result);}
     if(token_type == T_COMMA) token_type = get_token();
     if((result = log_exp(&spar_index,&spar_offset)) == FAIL)
	{printf("ERROR - in structure code function at %s\n",token_buf);
	 return(result);}
     if(token_type == T_COMMA) token_type = get_token();
     if((result = element(&lresult_index,&lresult_offset)) == FAIL)
	{printf("ERROR - in structure code function at %s\n",token_buf);
	 return(result);}
     if(token_type == T_COMMA) token_type = get_token();
     if((result = element(&rresult_index,&rresult_offset)) == FAIL)
	{printf("ERROR - in structure code function at %s\n",token_buf);
	 return(result);}
     print_opcode(carg1,arg2,NULL,NULL);
     print_args(off_exp1(fpar_index,fpar_offset),
		off_exp2(spar_index,spar_offset),
		off_exp3(lresult_index,lresult_offset),
		off_exp4(rresult_index,rresult_offset),
		NULL);
     rhs_index2 = lresult_index; rhs_offset2 = lresult_offset;
     if(token_type != T_RPAR)
		  printf("ERROR - in structure code function at %s\n",token_buf);
     token_type = get_token();
    }
  return(result);
}

clear_()
{
  int ret_value;
  int arg1_index, arg1_offset;


      if (element(&arg1_index,&arg1_offset) != FAIL)
	{print_opcode("clear_",T_CLEAR,symbol_table[arg1_index].mode1,symbol_table[arg1_index].mode2);
	 print_args(symbol_table[arg1_index].tokpts,NULL,NULL,NULL,NULL);
	 if (token_type != T_EOS)
	   {print_msg(T_EOS, token_buf);
	    ret_value = FAIL;}
	 else
	   ret_value = SUCCESS;
	 token_type = get_token();
	 }
       else
	 {printf("%d ERROR - incorrect syntax for clear, token=%s\n",linenum,token_buf);
	  ret_value = FAIL;
	  }
    return ret_value;
}


