/* P1_ERROR,C 910415 */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_scanner.h"
#include "p1_parser.h"

/* ===========================================================================

   function name: print_msg
   input	       : msg_num	- message number in the message table
		            msg_token	- token to be printed with message
   output	    : ---
   return value : ---

   global vars  : linenum	- line number in the listing
		            messages	- table of messages
		            list_fp	- listing file descriptor

   description	 : This routine prints the error or warning message to both
		            the listing file and standard output.

=========================================================================== */

char *messages[] =
{
   "WARNING - unexpected end of file",	           	   /* 0   EOF_ERR */
   "WARNING - unexpected end of comment",           	/* 1   EOC_ERR */
   "WARNING - illegal character in input ignored",    /* 2   CHAR_ERR */
   "WARNING - identifier truncated to 6 characters",	/* 3   IDSIZ_ERR */
   "WARNING - token longer than 50 characters",    	/* 4   TOKSIZ_ERR */
   "ERROR - multiply defined symbol",               	/* 5   DUP_ERR */
   "ERROR - symbol table overflow",                 	/* 6   SYMTAB_ERR */
   "ERROR - out of memory",		              	      /* 7   MEM_ERR */
   "ERROR - mode keyword expected",	                  /* 8   MODE_ERR */
   "ERROR - identifier expected",	              	   /* 9   ID_ERR */
   "ERROR - end of statement expected",             	/* 10  EOS_ERR */
   "ERROR - 'end' keyword expected",	              	/* 11  END_ERR */
   "ERROR - right parenthesis expected",            	/* 12  RPAR_ERR */
   "ERROR - left parenthesis expected",             	/* 13  LPAR_ERR */
   "ERROR - undefined symbol",                      	/* 14  UNDEF_ERR */
   "ERROR - using too many compiler temporary variables",/* 15  TEMP_ERR */
   "ERROR - invalid width",				               /* 16  WIDTH_ERR */
   "ERROR - invalid dimension",				            /* 17  DIM_ERR */
   "ERROR - left square bracket expected",		      /* 18  LSQU_ERR */
   "ERROR - right square bracket expected",  	      /* 19  RSQU_ERR */
   "ERROR - illegal type conversion in assignment",	/* 20  ASGNCON_ERR */
   "ERROR - assignment operator expected",          	/* 21  ASGNOP_ERR */
   "ERROR - can't assign parallel to scalar",	      /* 22  ASGNP2S_ERR */
   "ERROR - invalid symbol on left side of assignment", /* 23  ASGNID_ERR */
   "ERROR - invalid identifier",		      	         /* 24  INVID_ERR */
   "ERROR - constant expected",				            /* 25  CONST_ERR */
   "ERROR - comma expected",				               /* 26  COMMA_ERR */
   "ERROR - unrecognized symbol",			            /* 27  UNRSYM_ERR */
   "ERROR - invalid element detected",		            /* 28  ELEM_ERR */
   "ERROR - illegal type conversion in expression",	/* 29  EXPCON_ERR */
   "ERROR - invalid subscript",				            /* 30  SUBSCR_ERR */
   "ERROR - keyword 'in' expected",		              	/* 31  IN_ERR */
   "ERROR - statement or keyword 'endfor' expected",	/* 32  ENDFOR_ERR */
   "ERROR - parallel expression expected",		      /* 33  PAREXP_ERR */
   "ERROR - logical expression expected",		         /* 34  LOGEXP_ERR */
   "ERROR - index variable expected",			         /* 35  INXMIS_ERR */
   "ERROR - index must be parallel",			         /* 36  INXPAR_ERR */
   "ERROR - unmatched index variable",			         /* 37  INXUNM_ERR */
   "ERROR - parallel variable expected",		         /* 38  PARVAR_ERR */
   "ERROR - logical variable expected",			      /* 39  LOGVAR_ERR */
   "ERROR - statement or keyword 'endnext' expected",	/* 40  ENDNEXT_ERR */
   "ERROR - statement or keyword 'endget' expected",	/* 41  ENDGET_ERR */
   "ERROR - statement or keyword 'endif' expected",	/* 42  ENDIF_ERR */
   "ERROR - statement or keyword 'endifnany' expected",/* 43  ENDIFNANY_ERR */
   "ERROR - statement or keyword 'endany' expected",	/* 44  ENDANY_ERR */
   "ERROR - statement or keyword 'endwhile' expected",/* 45  ENDWHILE_ERR */
   "ERROR - statement or keyword 'endloop' expected",	/* 46  ENDLOOP_ERR */
   "ERROR - keyword 'with' expected",			         /* 47  WITH_ERR */
   "ERROR - keyword 'then' expected",			         /* 48  THEN_ERR */
   "ERROR - can't proceed; good bye ! ", 		         /* 49 INCLUDE_ERR */
   "ERROR - statement or keyword 'loop' expected",	   /* 50  LOOP_ERR */
   "ERROR - statement or keyword 'until' expected",	/* 51  UNTIL_ERR */
   "ERROR - scalar expression expected",		         /* 52  SCAEXP_ERR */
   "ERROR - using too many compiler constant variables",/* 53  SCASYS_ERR */
   "ERROR - invalid associate statement",             /* 54  ASSOC_ERR */
   "ERROR - keyword 'endallocate' expected",          /* 55  ENDALLOCATE_ERR*/
   "ERROR - invalid allocate statement",		         /* 56  ALLOCATE_ERR */
   "WARNING - no assembly codes declaration is created for",/* 57 DEFVAR_ERR */
   "ERROR - invalid define logical statement",		   /* 58  DEFLOG_ERR */
   "ERROR - missing file name after '#include' keyword",/* 59  FILE_ERR */
   "ERROR - can't open file",				               /* 60  OPEN_ERR */
   "ERROR - invalid nested '#include' statement",	   /* 61  NEST_INCL_ERR */
   "ERROR - invalid subroutine name",			         /* 62  SUBNAME_ERR */
   "ERROR - missing subroutine name",			         /* 63  MISSUB_ERR */
   "ERROR - invalid 'return' statement in main program",/* 64  RETURN_ERR */
   "ERROR - invalid type of operand(s)",		         /* 65  TYPE_ERR */
   "ERROR - missing index variable",			         /* 66  MISINX_ERR */
   "ERROR - keyword 'endsetscope' expected",		      /* 67  END_SCOPE_ERR */
   "ERROR - parallel integer or real argument expected",/* 68  MINMAX_ERR */
   "ERROR - logical parallel expression expected in 'any' statement",
							                                 /* 69  ANYLOGPAR_ERR */
   "ERROR - missing end of statement ';' after 'endasm'",/* 70  ASMEOS_ERR */
   "ERROR - scalar variable expected",			         /* 71  ADDR_SCA_ERR */
   "ERROR - missing colon \":\"",			            /* 72  ADDR_COL_ERR */
   "ERROR - decimal constant field length expected",	/* 73  ADDR_FIELD_ERR*/
   "ERROR - data type expected for indirect address",	/* 74  ADDR_TYPE_ERR */
   "ERROR - invalid width for specified data type",	/* 75  ADDR_WIDTH_ERR*/
   "ERROR - invalid left-hand-side address",		      /* 76  ADDR_LHS_ERR */
   "ERROR - logical parallel variable expected",	   /* 77  LOGPAR_ERR */
   "ERROR - keyword 'andthen' expected",              /* 78  ANDTHEN_ERR */
   "ERROR - keyword 'endandif' expected",             /* 79  ENDANDIF_ERR */
   "ERROR - keyword 'endandfor' expected",            /* 80  ENDANDFOR_ERR */
   "ERROR - incorrect AND construction",              /* 81  ANDBODY_ERR */
   "ERROR - undeclared constant type, decimal assumed",/* 82  CONST_WRN */
   "WARNING - offset index of 0 found",               /* 83  ZERO_OFF */
   "ERROR - keyword 'recurse' expected",              /* 84  RECURSE_ERR */
   "ERROR - keyword 'endrecurse' expected",           /* 85  ENDRECURSE_ERR*/
   "ERROR - 'endstackfor' or 'endstackwhile' expected", /* 86  ENDSTACK_ERR*/
   "ERROR - illegal network notation on left hand side",/* 87  CUBE_ERR */
   "ERROR - illegal network expression",                /* 88  NET_ERR */
   "ERROR - nany must be used with parallel expression", /* 89   NANY_ERR*/
   "ERROR - keyword 'do' expected",                   /* 90 DO_ERR */
   "ERROR - ':' expected",                             /* 91 COLON_ERR */
   "ERROR - allocate elsenany error"                   /* 92  ALLELSE_ERR */
};

/* print ERROR at previous line number  */
void print_msg_m1( msg_num, msg_token )
int msg_num;
char *msg_token;
{
  linenum--;
  print_msg(msg_num,msg_token);
}

void print_msg( msg_num, msg_token )
int msg_num;
char *msg_token;
{
   if (include_flag == TRUE)
     {fprintf( list_fp,"\n%d:%s:  %s\n",linenum,include_file,line_buf);
      fprintf( list_fp,"%s ",  messages[msg_num] );
      printf("\n%d:%s:  %s\n",linenum,include_file,line_buf);
      printf("%s ", messages[msg_num] );}
   else
     {fprintf( list_fp,"\n%d:  %s\n",linenum,line_buf);
      fprintf( list_fp," %s ", messages[msg_num] );
      printf("\n%d:  %s\n",linenum,line_buf);
      printf("%s ", messages[msg_num] );}

   if( msg_token != NULL )
     {fprintf( list_fp,": %s", msg_token);
      printf(": %s", msg_token);}
   fprintf(list_fp,"\n");
   printf("\n");

   printf("%d ERROR - STOP generated\n",linenum);
  print_opcode ("stop", T_STOP, T_NONE, T_NONE);
  print_args (NULL, NULL, NULL, NULL, NULL);
      fclose(source_fp);
      fprintf(list_fp, "\n");
      fclose(list_fp);
      fprintf(object_fp, "\n"); /* end with blank line for optimizer */
      fclose(object_fp);
  exit(0);

}
