/* P1_EXPRESSIONS.C 910415 */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"


/* ===========================================================================

   function name: get_op_mode1
   arg2_index  - pointer to symbol table index of 2nd arg
   output           : ---
   return value : op_mode1      - operation mode: int, real, logical

   global vars   : symbol_table - data structure for symbol table

   description  : This routine determines the mode of an operation from
	    the modes of its arguments. The mode of the operation can
	    be INT or REAL. It is determined by the following rules:

	    1) If the arguments have the SAME mode, the operation has
	       that same mode.
	    2) If the arguments have MIXED modes, but are the SAME
		    with regards to being SERIAL or PARALLEL, convert the
		    INT argument to REAL; the mode of the operation is
				REAL.
	    3) If the arguments have MIXED modes and are MIXED with
	       regards to being SERIAL or PARALLEL, convert the
	       mode of the SCALAR argument to the mode of the
	       PARALLEL argument; the mode of the operation is the
			    mode of the PARALLEL argument.

NOTE: float must work for unsigned intergers (CARD) also!
=========================================================================== */

int get_op_mode1(arg1_index,arg1_offset,arg2_index,arg2_offset)
int *arg1_index,*arg1_offset,*arg2_index,*arg2_offset;
{
   int op_mode1;           /* mode of operation: int or real */
   int arg1_mode1;      /* arg1: INT or REAL */
   int arg1_mode2;      /* arg1: SCALAR or PARALLEL */
   int arg2_mode1;   /* arg2: INT or REAL */
   int arg2_mode2;   /* arg2: SCALAR or PARALLEL */
   int arg1_size, arg2_size;
   
   if (debug == 2)
   printf ("In get_op_mode1()...\n");
   
   arg1_size = symbol_table[*arg1_index].width;
   arg2_size = symbol_table[*arg2_index].width;
   arg1_mode1 = symbol_table[*arg1_index].mode1;
   arg1_mode2 = symbol_table[*arg1_index].mode2;
   arg2_mode1 = symbol_table[*arg2_index].mode1;
   arg2_mode2 = symbol_table[*arg2_index].mode2;
   if(arg1_mode2 == T_CNST) arg1_mode2 = T_SCALAR;
   if(arg2_mode2 == T_CNST) arg2_mode2 = T_SCALAR;
/* if both T_INT and of different sizes, convert */
   if((arg1_mode2 == T_SCALAR || arg1_mode2 == T_REG_SCALAR) &&
      (arg2_mode2 == T_SCALAR || arg2_mode2 == T_REG_SCALAR) &&
      (arg1_size != arg2_size)                              )
      if(arg1_size == W_LONGINT)
	 tolong(arg2_index,arg2_offset);
      else
	 tolong(arg1_index,arg1_offset);
/* if both args are the same */
   if (arg1_mode1 == arg2_mode1) op_mode1 = arg1_mode1;
   else switch(arg1_mode1)
     {case T_INT:
      case T_LONGINT:
      case T_OCT:
      case T_HEX:
      case T_BIN:
      case T_CHAR:
      case T_LOGICAL:
      case T_CARD:if(arg2_mode1 == T_REAL)         /* "INT"-REAL */
			 if (arg1_mode2 == arg2_mode2)  /* SCA-SCA or PAR-PAR */
				     {float_(arg1_index,arg1_offset);
				      op_mode1 = T_REAL;}
			      else if ((arg1_mode2 == T_CNST ||    /* SCA-PAR */
						  arg1_mode2 == T_SCALAR  ||
					arg1_mode2 == T_REG_SCALAR) &&
						  arg2_mode2 == T_PARALLEL )
				     {float_ (arg1_index,arg1_offset);
				      op_mode1 = T_REAL;}
			      else if ((arg2_mode2 == T_CNST ||    /* PAR-SCA */
					arg2_mode2 == T_SCALAR ||
					arg2_mode2 == T_REG_SCALAR) &&
						  arg1_mode2 == T_PARALLEL )
/*                                   {fix_ (arg2_index,arg2_offset);
					   op_mode1 = T_INT;}         */
				     {float_ (arg1_index,arg1_offset);
					   op_mode1 = T_REAL;}
			      else if ((arg2_mode2 == T_CNST ||    /* SCA-REG */
						  arg2_mode2 == T_SCALAR ) &&
						  arg1_mode2 == T_REG_SCALAR)
/*                                   {scareg_(arg2_index,arg2_offset);
				      op_mode1 = T_REG_SCALAR;}   */
				     {float_ (arg1_index,arg1_offset);
					   op_mode1 = T_REAL;}
			      else if ((arg1_mode2 == T_CNST ||    /* REG-SCA */
						  arg1_mode2 == T_SCALAR ) &&
						  arg2_mode2 == T_REG_SCALAR)
/*                                   {regsca_ (arg1_index,arg1_offset);
					   op_mode1 = T_SCALAR;}  */
				     {float_ (arg1_index,arg1_offset);
					   op_mode1 = T_REAL;}
			 else printf("%d ERROR - illegal mode2a combination for others, arg1_mode2=%d, arg2_mode2=%d\n",
				    linenum,arg1_mode2,arg2_mode2);
		  else switch(arg2_mode1)
								{case T_INT     :
					case T_LONGINT :
					case T_OCT     :
					case T_HEX     :
					case T_BIN     :
					case T_CARD    : if(arg1_mode1 == T_CHAR)
										     chrint_(arg1_index,arg1_offset);
					  op_mode1 = arg1_mode1; break;
					case T_CHAR    : if(arg1_mode1 == T_INT)
										     chrint_(arg2_index,arg2_offset);
					  op_mode1 = arg1_mode1; break;
			 default: printf("%d ERROR - illegal mode1 combination\n",linenum); break;
								}
		    break;
      case T_REAL:  if(arg2_mode1 == T_INT ||  /* REAL-"INT" */
		       arg2_mode1 == T_LONGINT ||
		       arg2_mode1 == T_OCT     ||
		       arg2_mode1 == T_BIN     ||
		       arg2_mode1 == T_HEX     ||
		       arg2_mode1 == T_CARD )
		       if ((arg1_mode2 == T_CNST || arg1_mode2 == T_SCALAR ) &&
			   (arg2_mode2 == T_CNST || arg2_mode2 == T_SCALAR ) ||
				(arg1_mode2 == arg2_mode2))  /* SCA-SCA or PAR-PAR */
			   {float_ (arg2_index,arg2_offset); op_mode1 = T_REAL;}
		       else if ((arg1_mode2 == T_CNST ||    /* SCA-PAR */
				 arg1_mode2 == T_SCALAR ) &&
				 arg2_mode2 == T_PARALLEL )
			   {fix_ (arg1_index,arg1_offset); op_mode1 = arg2_mode1;}
		       else if ((arg2_mode2 == T_CNST ||    /* PAR-SCA */
				 arg2_mode2 == T_SCALAR ||
				 arg2_mode2 == T_REG_SCALAR) &&
				 arg1_mode2 == T_PARALLEL )
			   {float_ (arg2_index,arg2_offset);  op_mode1 = T_REAL;}
		       else if ((arg2_mode2 == T_CNST ||    /* SCA-REG */
				 arg2_mode2 == T_SCALAR ) &&
				arg1_mode2 == T_REG_SCALAR)
			   {float_ (arg2_index,arg2_offset);  op_mode1 = T_REAL;}
/*  removed 930104scareg_(arg2_index,arg2_offset);  op_mode2 = T_REG_SCALAR; */
		       else if ((arg1_mode2 == T_CNST ||    /* REG-SCA */
				 arg1_mode2 == T_SCALAR ) &&
				 arg2_mode2 == T_REG_SCALAR)
			   {float_ (arg2_index,arg2_offset);  op_mode1 = T_REAL;}
/*removed 930104regsca_ (arg1_index,arg1_offset);  op_mode2 = T_SCALAR;*/
		      else printf("%d ERROR - illegal mode2a combination for T_REAL, arg1_mode2=%d, arg2_mode2=%d\n",
				linenum,arg1_mode2,arg2_mode2);
		    else printf("%d ERROR - illegal mode1 combination\n",linenum);
		    break;
      }

/*printf("op_mode1 = %d\n",op_mode1);*/
   return(op_mode1);

} /* end of get_op_mode1() */



/* ===========================================================================

   function name: get_op_mode2
   input               : arg1_index     - pointer to symbol table index of 1st arg
			    arg2_index  - pointer to symbol table index of 2nd arg
   output           : ---
   return value : op_mode2      - operation mode: scalar, parallel, common
   global vars   : symbol_table - data structure for symbol table

   description   : This routine determines the mode of an operation from
			    the modes of its arguments. The mode of the operation can be
			    SCALAR, PARALLEL, COMMON_LEFT, or COMMON_RIGHT. The
			    operation mode is determined by the following rules:

			    1) If the arguments have the SAME mode, the operation has
			       that same mode.
			    2) If the arguments have MIXED modes and the SCALAR
				argument is on the left of the operator, the mode of
				    the operation is COMMON_LEFT.
			    3) If the arguments have MIXED modes and the SCALAR
				    argument is on the right of the operator, the mode of
				    the operation is COMMON_RIGHT.

=========================================================================== */

int get_op_mode2(arg1_index, arg2_index)
int arg1_index, arg2_index;
{
   int op_mode2;        /* mode of operation: scalar, parallel, common */
   int arg1_mode2,arg2_mode2;
   
   if (debug == 2)
       printf ("In get_op_mode2().....................\n");

   arg1_mode2 = symbol_table[arg1_index].mode2;
   arg2_mode2 = symbol_table[arg2_index].mode2;

   switch(arg1_mode2)
	  {case T_SCALAR:
	   case T_CNST:
			switch(arg2_mode2)
				{case T_REG_SCALAR: op_mode2 = T_SCALAR_LEFT; break;
				 case T_PARALLEL:   op_mode2 = T_COMMON_LEFT; break;
				 case T_SCALAR:
				 case T_CNST:       op_mode2 = arg1_mode2; break;
				 default: printf("%d ERROR - illegal mode2 combo,",linenum);
							 printf("arg1_mode2=%d, arg2_mode2=%d\n",
										arg1_mode2, arg2_mode2);
							 op_mode2 = FAIL; break;}
			break;
	   case T_REG_SCALAR:
			switch(arg2_mode2)
				{case T_REG_SCALAR: op_mode2 = arg1_mode2; break;
				 case T_PARALLEL:   op_mode2 = T_REG_LEFT; break;
				 case T_SCALAR:
				 case T_CNST:       op_mode2 = T_SCALAR_RIGHT; break;
				 default: printf("%d ERROR - illegal mode2 combo,",linenum);
							 printf("arg1_mode2=%d, arg2_mode2=%d\n",
										arg1_mode2, arg2_mode2);
							 op_mode2 = FAIL; break;}
			break;
	   case T_PARALLEL:
			switch(arg2_mode2)
				{case T_REG_SCALAR: op_mode2 = T_REG_RIGHT; break;
				 case T_PARALLEL:   op_mode2 = arg1_mode2; break;
				 case T_SCALAR:
				 case T_CNST:       op_mode2 = T_COMMON_RIGHT; break;
				 default: printf("%d ERROR - illegal mode2 combo,",linenum);
							 printf("arg1_mode2=%d, arg2_mode2=%d\n",
										arg1_mode2, arg2_mode2);
							 op_mode2 = FAIL; break;}
			break;
	  default:
		  printf("%d ERROR - illegal t_mode2,",linenum);
		  printf("arg1_mode2=%d, arg2_mode2=%d\n",
					 arg1_mode2, arg2_mode2);
		  op_mode2 = FAIL; break;
	  }

   return(op_mode2);

} /* end of get_op_mode2() */



/* ===========================================================================

   function name: get_result_mode2
   input         : op_mode2     - mode of operation: SCALAR, PARALLEL,
						 COMMON_LEFT, or COMMON_RIGHT
   output           : ---
   return value : result_mode2  - mode of result: SCALAR or PARALLEL

   global vars   : debug                - debugging statement flag

   description   : This routine determines the mode of the result argument
			    based on the mode of the operation.

=========================================================================== */

int get_result_mode2(op_mode2)
int op_mode2;
{
   int result_mode2;            /* mode of result: scalar or parallel */

   if (debug == 2)
      printf ("In get_result_mode2().........................\n");

   switch (op_mode2)
   {
      case T_CNST          :
      case T_SCALAR                : result_mode2 = T_SCALAR;
					    break;
      case T_REG_LEFT:
		case T_REG_RIGHT:
      case T_COMMON_LEFT        :
      case T_COMMON_RIGHT       :
      case T_PARALLEL      : result_mode2 = T_PARALLEL;
									  break;
		case T_SCALAR_LEFT   :
		case T_SCALAR_RIGHT  : result_mode2 = T_REG_SCALAR;
									  break;
		case T_REG_SCALAR    : result_mode2 = T_REG_SCALAR;
									  break;
   }

   return(result_mode2);

} /* end of get_result_mode2() */



/* ===========================================================================

   function name: fix_
   input               : arg_index      - symbol table index of value to be converted
   output       : ---
   return value : ---

   global vars  : symbol_table  - data structure for symbol table

   description   : This routine handles the intermediate code generation for
			    converting a real to an integer. The value may be scalar
			    or parallel.

=========================================================================== */

void fix_(arg_index,arg_offset)
int *arg_index,*arg_offset;
{
  int result_index;
  int mode_one,mode_two;
  int size_two;

  if(debug == 2)
    printf("In fix \n");

  mode_one = symbol_table[*arg_index].mode1;
  mode_two = symbol_table[*arg_index].mode2;
  size_two = symbol_table[*arg_index].width;
  result_index =get_symtab_temp(T_INT,mode_two,size_two);

  print_opcode("fix_", T_FIX, mode_one, mode_two);
  if (arg_offset != (int *) SUCCESS)
     print_args (off_exp1(*arg_index,*arg_offset),
		   off_exp2(result_index,SUCCESS), NULL, NULL, NULL);
  else
     print_args (off_exp1(*arg_index,SUCCESS),
		 off_exp2(result_index,SUCCESS), NULL, NULL, NULL);

  *arg_index = result_index;
  *arg_offset= SUCCESS;
}


/* ===========================================================================

   function name: float_
   input               : arg_index      - symbol table index of value to be converted
   output       : ---
   return value : ---

   global vars  : symbol_table  - data structure for symbol table

   description   : This routine handles the intermediate code generation for
			    converting an integer to a real. The value may be scalar
			    or parallel.

=========================================================================== */

void float_(arg_index,arg_offset)
int *arg_index,*arg_offset;
{
  int result_index;
  int mode_one,mode_two;
  int size_two;

  mode_one = symbol_table[*arg_index].mode1;
  mode_two = symbol_table[*arg_index].mode2;
  size_two = 32 /*symbol_table[*arg_index].width*/;
  result_index=get_symtab_temp(T_REAL, mode_two,size_two);
  print_opcode("float_", T_FLOAT, mode_one, mode_two);
  print_args(off_exp1(*arg_index,*arg_offset),
		  off_exp2(result_index,SUCCESS), NULL, NULL, NULL);
  *arg_index = result_index;
  *arg_offset = SUCCESS;
}



/* ===========================================================================

   function name: short_
   input               : arg_index      - symbol table index of value to be converted
   output           : ---
   return value : ---

   global vars  : symbol_table  - data structure for symbol table

   description   : This routine handles the intermediate code generation for
			    converting  to a short integer. The value
			    may be scalar or parallel.

=========================================================================== */

void short_(arg1_index,arg1_offset,result_size)
int *arg1_index,*arg1_offset,result_size;
{
  int arg2_index;
  int mode_one,mode_two;

  if(debug == 2) printf("In short\n");

  mode_one = symbol_table[*arg1_index].mode1;
  mode_two = symbol_table[*arg1_index].mode2;
  if(mode_two == T_CNST) return; /* cnst match all sizes, no conversion */
  arg2_index = get_symtab_temp(mode_one,mode_two,result_size);
  print_opcode("short_", T_SHORT, mode_one, mode_two);
  print_args(off_exp1(*arg1_index,*arg1_offset),
		  off_exp2(arg2_index,SUCCESS), NULL, NULL, NULL);
  *arg1_index = arg2_index;
  *arg1_offset= SUCCESS;
}





/* ===========================================================================

   function name: chrint_
   input               : arg_index      - symbol table index of value to be converted
   output       : ---
   return value : ---

   global vars  : symbol_table  - data structure for symbol table

   description   : This routine handles the intermediate code generation for
			    converting a character to an integer. The value may be scalar
			    or parallel.

=========================================================================== */

void chrint_(arg_index,arg_offset)
int *arg_index,*arg_offset;
{
  int result_index;
  int mode_one,mode_two;
  int size_two;

  if(debug == 2)
    printf("In chrint \n");

  mode_one = symbol_table[*arg_index].mode1;
  mode_two = symbol_table[*arg_index].mode2;
  size_two = symbol_table[*arg_index].width;
  result_index =get_symtab_temp(T_INT,mode_two,size_two);

  print_opcode("chrint_", T_CHRINT, mode_one, mode_two);
  print_args (off_exp1(*arg_index,*arg_offset),
		   off_exp2(result_index,SUCCESS), NULL, NULL, NULL);
  *arg_index = result_index;
  *arg_offset= SUCCESS;
}


/* ===========================================================================

   function name: intchr_
   input               : arg_index      - symbol table index of value to be converted
   output       : ---
   return value : ---

   global vars  : symbol_table  - data structure for symbol table

   description   : This routine handles the intermediate code generation for
			    converting an integer to a character. The value may be scalar
			    or parallel.

=========================================================================== */

void intchr_(arg_index,arg_offset)
int *arg_index,*arg_offset;
{
  int result_index;
  int mode_one,mode_two;
  int size_two;

  mode_one = symbol_table[*arg_index].mode1;
  mode_two = symbol_table[*arg_index].mode2;
  size_two = symbol_table[*arg_index].width;
  result_index=get_symtab_temp(T_CHAR, mode_two,size_two);
  print_opcode("intchr_", T_INTCHR, mode_one, mode_two);
  print_args(off_exp1(*arg_index,*arg_offset),
		  off_exp2(result_index,SUCCESS), NULL, NULL, NULL);
  *arg_index = result_index;
  *arg_offset = SUCCESS;
}


void scareg_(arg_index,arg_offset)
int *arg_index, *arg_offset;
{
  int mode_one, mode_two;

  mode_one = symbol_table[*arg_index].mode1;
  mode_two = symbol_table[*arg_index].mode2;

  mode_two = T_REG_SCALAR;
  symbol_table[*arg_index].mode2=T_REG_SCALAR;
  print_opcode("scareg_",T_SCAREG, mode_one, mode_two);
  print_args (off_exp1(*arg_index,*arg_offset), NULL, NULL, NULL, NULL);

}


void regsca_(arg_index,arg_offset)
int *arg_index, *arg_offset;
{
  int mode_one, mode_two;

  mode_one = symbol_table[*arg_index].mode1;
  mode_two = symbol_table[*arg_index].mode2;

  mode_two = T_SCALAR;
  symbol_table[*arg_index].mode2=T_SCALAR;
  print_opcode("regsca_",T_REGSCA, mode_one, mode_two);
  print_args (off_exp1(*arg_index,*arg_offset), NULL, NULL, NULL, NULL);

}

tolong(arg,offset)
int *arg, *offset;
{
int scalar_temp;

return 0; /*931109*/
   if((scalar_temp=get_symtab_temp(T_LONGINT,T_SCALAR,W_LONGINT)) != FAIL)
     {print_opcode("long_",T_LONG,T_INT,T_SCALAR);
      print_args(off_exp1(*arg,*offset),off_exp2(scalar_temp,SUCCESS),NULL,NULL,NULL);
      *arg = scalar_temp;
      *offset = SUCCESS;}
   else {print_msg(TEMP_ERR,"In expre 1"); return(FAIL);}
   return SUCCESS;
}
