/* 910415 P1_FOR.C */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"

/* ===========================================================================

   function name: for_
   input	       : ---
   output       : ---
   return value : return_value	- SUCCESS, if no errors
			 	                     - FAIL, otherwise

   global vars  : symbol_table	- array containing symbol table
		            for_count	- counter for making 'for' statement labels
		            token_type	- type of current token
		            token_buf	- character string for current token
		            debug		- flag for debugging statements

   description	 : This routine is part of a recursive descent parser and
		            handles the following production from the grammar:

			FOR_ <-- index in LOG_EXP BODY endfor index ;

		  The index variable must be an identifier that has been
		  declared as an 'index parallel' identifier. The expression
		  must evaluate to a logical parallel result. The body
		  of the 'for' statement can be empty. The keywords 'in'
		  and 'endfor' are required. The index variable must be
		  repeated at the end of the 'for' loop. The loop is
		  executed once for each responder in the expression. The
		  expression is evaluated only once at the beginning of
		  the loop and the result is stored in the index variable.
		  The index variable is updated on each pass of the loop.
		  The index variable may not be modified during the loop
		  execution, but may be used to index a parallel variable.

=========================================================================== */

int for_()
{
   int return_value;		/* return value of function */
   int invar_index;		/* symbol table index of index variable */
   int invar_offset;
   int expr_index;		/* symbol table index of expression result */
   int expr_offset;
   char *start_label;	/* label to print at start of 'for' */
   char *end_label;		/* label to print at end of 'for' */
   char *repeat_label;	/* label to print at loop of 'for' */

   if (debug == 2)
      printf ("In for_()........................................\n");

   start_label = make_label ("FO$S", ++for_count);
   end_label = make_label ("FO$E", for_count);
   repeat_label = make_label ("FO$R", for_count);

   print_opcode ("setscope", T_SETSCOPE, T_NONE, T_NONE);
   print_args (start_label, NULL, NULL, NULL, NULL);

   get_index(&invar_index,&invar_offset); /* "each" replaced by ITS */
   if (invar_index == FAIL)
		return_value = FAIL;
   else
     {if(strcmp(symbol_table[invar_index].tokpts,"ITS") == 0);
		else if (token_type == T_IN) token_type = get_token();
		else
         {print_msg (IN_ERR, token_buf);
          return_value = FAIL;}
      log_exp(&expr_index,&expr_offset);
      if(expr_index == FAIL)
         return_value = FAIL;
      else if (symbol_table[expr_index].mode2 != T_PARALLEL)
        {print_msg (PAREXP_ERR, NULL);
	      return_value = FAIL;}
      else if (symbol_table[expr_index].mode1 != T_LOGICAL)
        {print_msg (LOGEXP_ERR, token_buf);
	      return_value = FAIL;}
      else
        {save_em(expr_index,expr_offset);
			print_opcode ("for  ", T_FOR, T_NONE, T_NONE);
 	      print_args (off_exp1(expr_index,expr_offset),
                     off_exp2(invar_index,invar_offset), end_label,
		               repeat_label, NULL);
			do_it(invar_index,invar_offset);
         if ((return_value = body()) != FAIL)
	        {if (token_type != T_ENDFOR)
	           {print_msg (ENDFOR_ERR, token_buf);
	            return_value = FAIL;}
	         else
	           {token_type = get_token();
					if(strcmp(token_buf, "EACH") == 0)
					  {token_type = T_ID;
					   token_size =3;
						strcpy(token_buf,"ITS");}
	            if (strcmp(token_buf, symbol_table[invar_index].tokpts) != 0)
	              {print_msg (INXUNM_ERR, token_buf);
		            return_value = FAIL;}
	            else
	              {print_opcode ("endfor", T_ENDFOR, T_NONE, T_NONE);
		            print_args (off_exp1(invar_index,invar_offset), end_label,
			                     repeat_label, NULL, NULL);
                  if ((token_type = get_token()) == T_LSQU)
		              {while ((token_type = get_token()) != T_RSQU);
		               token_type = get_token();}
		            if (token_type != T_EOS)
		              {print_msg (EOS_ERR, token_buf);
		               return_value = FAIL;}
		            else
		              {token_type = get_token();
		               return_value = SUCCESS;}
	               }
	           }
           }
        }
     }

   if (debug == 2)
      printf("For() returning return_value %d\n", return_value);

   return(return_value);
}
