/* 910415  P1_IDENTIFIER.C */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"


/* ===========================================================================

   function name: more_identifier
   input         : id_index     - symbol table index of 1st identifier
   output           : result_index, result_offset
   return value : result_index  - symbol table index of 1st id, if no subscript
						     - symbol table index of new id, if subscript
						     - FAIL, otherwise

   global vars  : token_type    - current token type
			    token_buf   - current token string
			    symbol_table  - symbol table data structure
			    debug       - debugging statement flag

   description   : This routine is part of a recursive descent parser and
			    handles the following production from the grammar:

			MORE_IDENTIFIER <-- [ SUBSCRIPT ] |
					    @

		  '@' represents the empty production. This routine checks
		  to see if the first identifier (id_index) is a simple
		  variable or if it is subscripted. If no subscript is
		  found, the symbol table index of the first identifier
		  is returned (id_index). FAIL is returned if:

			1) a syntax error occurs in the subscript, or
			2) the first identifier is scalar and the subscript
			   is parallel, or
			3) the first identifier is parallel and the subscript
			   is scalar, or
			4) the subscript is parallel, but not type logical, or
			5) the subscript is scalar, but not type integer.

		  If the subscript found is the special subscript '$', the
		  symbol table index of the first identifier is returned
		  (id_index). Otherwise, if any other correct subscript is
		  found, a temporary identifier is allocated to denote the
		  complete subscripted variable (result_index) and it is
		  returned. The identifier being subscripted may be any type.
		  ?? Used only on the right hand side. ??

       input:     id_index[$,...]
		  id_index[log_exp,...]
      output: result_index[result_offset]
=========================================================================== */

int more_identifier(id_index,result_index,result_offset)
int id_index,*result_index,*result_offset;
{
   int sav_index, size_index, temp_index, table;
	int mode_one, scalar_dx, outbits_dx, outbits;
   int bi_index, logical_dx, the_index;
	char offset[20];
	char awidth[20];

   if (debug == 2)
      printf ("In more_identifier().....................................\n");
   /* parse its a - a[its] - id_index & result_offset must be switched */
   /* parse jane's rate in same manner                                 */
   table = 0;
   if((id_index == (temp_index = symtab_fetch(3,"ITS")))  ||
      (id_index == (temp_index = the_index = symtab_fetch(3,"THE")))  ||
      (id_index == (temp_index = if_row_name(id_index,&table)))     )
	  {if(token_type == T_RSQU)
	     {*result_index=id_index;
	      *result_offset=SUCCESS;
	       return(SUCCESS);}
	   else if(token_type != T_ID)
	     {printf("%d ERROR - parsing ITS or THE, token_type=%d\n",
					 linenum, token_type); return(FAIL);}
	   else
	    {sav_index = symtab_fetch(token_size,token_buf);
	     if(symbol_table[sav_index].toktype == T_ACE)
		 if(aceident(&sav_index,&table) == FAIL)
		    printf("%d ERROR - in p1_ident at T_ACE\n");
	     if(sav_index == FAIL)
	       {if(id_index == the_index)
		 {*result_index = FAIL;
		  *result_offset = SUCCESS;
		   *result_index = np(*result_index);
		   return SUCCESS;}
	       else
		 printf("%d ERROR - illegal ITS argument %s\n",linenum,token_buf);}
	     if(symbol_table[sav_index].mode2 != T_PARALLEL)
	       {printf("%d ERROR - ITS argument must be parallel\n", linenum);
		printf("sav_index=%s, mode2=%d\n", symbol_table[sav_index].tokpts,
				 symbol_table[sav_index].mode2);
		return(FAIL);}
	     else
	       {/*set mode2 = T_SCALAR, num_dim--; before calling subscript */
		symbol_table[sav_index].mode2 = T_SCALAR;
		symbol_table[sav_index].num_dim--;
		token_type = get_token();
		if(symbol_table[sav_index].num_dim > 0)
		  {if(subscript(sav_index,result_index,result_offset) == FAIL)
		     if(token_type == T_PAR_SUBSCRIPT)
			token_type = get_token();
		     else
			return FAIL;
		   else
		     {if((temp_index = get_symtab_temp(T_INT, T_SCALAR, T_NONE)) == FAIL)
			{print_msg(TEMP_ERR,"in p1_ident"); return(FAIL);}
		      pc_itoa(symbol_table[sav_index].width,awidth);
		      size_index = get_const(awidth,T_DEC_CONST);
		      print_opcode("mul_",T_MULT,T_INT,T_SCALAR);
		      print_args(off_exp1(size_index,SUCCESS),
		      off_exp2(*result_offset,SUCCESS),
		      off_exp3(temp_index,SUCCESS),NULL,NULL);
		      *result_index=sav_index;
		      *result_offset=temp_index;}
		  }
		else
		  {*result_index=sav_index;
		   *result_offset=SUCCESS;}
		symbol_table[sav_index].mode2 = T_PARALLEL;
		symbol_table[sav_index].num_dim++;
	       }
	     mode_one = symbol_table[sav_index].mode1;
	     outbits = symbol_table[sav_index].width;
	     pc_itoa(outbits,offset);
	     outbits_dx = get_const(offset,T_DEC_CONST);
	     if((scalar_dx = get_symtab_temp(mode_one,T_SCALAR,T_NONE)) == FAIL)
	       {print_msg(TEMP_ERR,"in p1_ident 2"); return(FAIL);}
	     else /* put indexed offset into a scalar */
	       {print_opcode("mvpa_",T_MVPA,mode_one,T_REG_SCALAR);
		print_args(off_exp1(*result_index,*result_offset),
			   off_exp2(scalar_dx,SUCCESS),
			   off_exp3(temp_index,SUCCESS),
			   off_exp4(outbits_dx,SUCCESS),NULL);
		save_the(scalar_dx,SUCCESS,*result_index,*result_offset);}
	 }
      *result_index = scalar_dx;
      *result_offset = SUCCESS;
       return(SUCCESS);
     }
   else if (id_index == symtab_fetch(5,"THEIR"))
     {if(token_type != T_ID)
	{printf("ERROR - parsing THEIR, linenum=%d, token_type=%d\n",
	 linenum, token_type); return(FAIL);}
      id_index = symtab_fetch(token_size,token_buf);
      token_type = get_token();} /* skip THEIR */
   else if (id_index == symtab_fetch(1,"A"))
     {if(token_type == T_ID)
       {id_index = symtab_fetch(token_size,token_buf);
	bi_index = symbol_table[id_index].bi;
	if((logical_dx = get_symtab_temp(T_LOGICAL,T_PARALLEL,T_NONE)) == FAIL)
	 {print_msg(TEMP_ERR,"in p1_ident 3"); return(FAIL);}
	  mode_one = symbol_table[id_index].mode1;
	  if((*result_index = get_symtab_temp(mode_one,T_SCALAR,T_NONE)) == FAIL)
	  {print_msg(TEMP_ERR,"in p1_ident 4"); return(FAIL);}
	   print_opcode("resf_",T_RESFST, T_NONE, T_PARALLEL);
	   print_args (off_exp1(bi_index,SUCCESS),
		       off_exp2(logical_dx,SUCCESS),NULL, NULL, NULL);
	   outbits = symbol_table[id_index].width;
	   pc_itoa(outbits,offset);
	   outbits_dx = get_const(offset,T_DEC_CONST);
	   print_opcode("mvpa_",T_MVPA,mode_one,T_SCALAR);
	   print_args(off_exp1(id_index,SUCCESS),
		      off_exp2(*result_index,SUCCESS),
		      off_exp3(logical_dx,SUCCESS),
		      off_exp4(outbits_dx,SUCCESS),NULL);
	   do_it(logical_dx,SUCCESS);
	   save_the(*result_index,SUCCESS,id_index,SUCCESS);
	   token_type = get_token();
	   return(SUCCESS);}
/* printf("%d ERROR - At A in p1_ident, token_buf=%s\n",linenum,token_buf);*/
     }
   else
     if(symbol_table[id_index].toktype == T_INDEX && token_type != T_LSQU)
	{*result_index = id_index;
	 *result_offset=SUCCESS;
	 return(SUCCESS); }

   if(subscript(id_index,result_index,result_offset) == FAIL)  return(FAIL);

   if (debug == 2)
      printf ("More_identifier() returning result_index %d\n", result_index);
   return(*result_index);

} /* end of more_identifier() */
