/* 910415 P1_INCLUDE.C */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"
#include "ace2.h"

/* ==========================================================================

   function name : include
   input                : ---
   output            : ---
   return value  : return_value - always SUCCESS

   global vars    : token_type  - type of current token
			     debug              - flag for debugging statements

   description    : this routine is handling the include statement. when an
			     include statement is detected in the source file, this
			     will :

			1 ) Test the included file
			    check if the file can be opened. If the file can
			    be opened or does not exist, this routine will
			    issuing an error message and quit the compilation.

			2 ) switch the active file pointer
			    If the included file has successfully opened, this
			    routine will switch the active file pointer to
			    point to the new file pointer until all data in
			    in the included file have been consumed.

============================================================================ */

void include()
{
   char *new_file;                      /* pointer to the included file name */


   if (debug == 2)
      printf ("In include().........................\n");

   new_file = get_file_name();
   if (strcmp(new_file, "<none>") != 0)
     {if ((include_fp = fopen(new_file, "r")) != NULL)
	{include_file = new_file;
	 active_fp = include_fp;
	 include_flag = TRUE;
	 use_once_flag = TRUE;}
      else
	{print_msg(OPEN_ERR, new_file);
	 print_msg(INCLUDE_ERR, NULL);
	 printf("\n");
	 exit(1);}
     }
   else
     {print_msg(FILE_ERR, NULL);
      print_msg(INCLUDE_ERR, NULL);
      printf("\n");
      exit(1);
   }

   if (debug == 2)
      printf("returning from include()...........................\n");

} /* end of include() */




/* ===========================================================================

   function name : get_file_name
   input                : ---
   output        : ---
   return value  : returns file name

   global vars   : token_type   - type of current token
			     token_buf  - current token string
			     symbol_table       - symbol table data structure
			     debug              - debugging statement flag

   description    : This routine is getting file name from the source program.
		   If no file name is found in the include statement, this
			     routine file return '<fail>' string.

=========================================================================== */
char *get_file_name()
{
   char *file_name;     /* array for file name */
   char *buffer;                /* pointer that points to line_code */
   char ch;                             /* character identifier read from the
						 input buffer */
   char *dxpnt;

   if (debug == 2)
      printf("In get_file_name().....................................\n");


   while( char_type((ch = *ncpnt)) == WHITESPACE)
       ncpnt++;

   if (ch == '\n')
     {file_name = (char *) calloc(strlen("<none>")+3, sizeof(char)); /* jlpbug */
      strcpy(file_name, "<none>");}
   else
     {buffer = (char *) calloc(LINE_LENGTH+2, sizeof(char)); /* jlpbug */
      file_name = buffer;
      while (char_type((*buffer = *ncpnt++)) != WHITESPACE)
	  buffer++;
      *buffer = (char) NULL;}

   /* skip optional ; */
   dxpnt = indexp(file_name, ';');
   if(*dxpnt == ';') *dxpnt = 0;

   if (debug == 2)
      printf("get_file_name returning file_name : %s\n", file_name);

   return(file_name);

} /* end of get_file_name() */

