/* 910415 P1_INIT.C */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"
char *index();
#define T_LEXSIZ  96
/* ===========================================================================

   function name : init_symbol_table
   inputs            : ---
   output            : ---
   return value  : ---

   global vars    : symbol_table - array of structures used as symbol table

   description    : This routine creates temporary identifiers for use by the
			     parser. They are entered into a reserved area of the symbol
			     table. The special parallel subscript '$' is inserted in
			     the last symbol table location.

NOTE: sca_temp_start and log_par_temp_start have been changed from constant
      to variables so that these temps may be reserved (allocated) for more
      that one statment.
=========================================================================== */

void init_symbol_table(ex_vl)
int ex_vl;
{
   int i, j, *cpnt;
   char *string, num[13], field_width[10], expt_width[10], signif_width[10];
   char c_mode[10],c_width[10],c_expt[10],c_signif[10];
   char *pnt, *stringnum, c_siz[10];
   int dim_size[MAX_NUM_DIM];

   if(debug==2) printf("In init_symbol_table. ex_vl=%d\n",ex_vl);
   if(ex_vl == 2) return;
   string = (char *)calloc(50, sizeof(char));
   stringnum = (char *)calloc(50, sizeof(char));
   if(wt == WT) strcpy(string,"STMP_+");
   else strcpy(string,"STMP$+");

   if(debug==2) printf("At int_sca\n");
   /* Create scalar temporary identifiers. */
   for (i=INT_SCA_TEMP_START, j=0; i<=INT_SCA_TEMP_END; i++, j++)
     {pc_itoa(j,num);
      strcpy(stringnum,string);
      strcat(stringnum,num);
      if((cpnt = calloc(strlen(stringnum)+1, sizeof(char))) == NULL)
	{printf("%d ERROR - out of memory in P1_init\n",linenum);
	 exit(1);}
      symbol_table[i].tokpts = (char *) cpnt;
      strcpy(symbol_table[i].tokpts,stringnum);
      symbol_table[i].status = BUSY;
      symbol_table[i].mode1 = T_INT;
      symbol_table[i].mode2 = T_SCALAR;
      symbol_table[i].toktype = T_ID;
      symbol_table[i].num_dim = 0;
      symbol_table[i].width = W_INT;
      symbol_table[i].expt = 0;
      symbol_table[i].signif = W_INT;
      symbol_table[i].temp = TRUE;
      symbol_table[i].offset = 0;
      symbol_table[i].iospec = shapedx;
}
   if(debug==2) printf("After int_sca\n");

   i=INT_SCA_TEMP_START; /* reset i for below */
   pc_itoa((INT_SCA_TEMP_END - INT_SCA_TEMP_START + 1),num);
   pc_itoa(symbol_table[i].mode1*SIZE_MODE2+symbol_table[i].mode2,c_mode);
   pc_itoa(symbol_table[i].width,c_width);
   pc_itoa(symbol_table[i].expt,c_expt);
   pc_itoa(symbol_table[i].signif,c_signif);
   if(wt == WT) install_scalar ("STMP_", num, "0", c_mode, c_width, c_expt, c_signif,TRUE,shapedx);
   else install_scalar ("STMP$", num, "0", c_mode, c_width, c_expt, c_signif,TRUE,shapedx);
   if (module_flag == T_MAIN  || wt_sub == WT)
      {print_opcode("entry_", T_ENTRY, NULL, NULL);
       if(wt == WT) print_args("STMP_", NULL, NULL, NULL, NULL);
       else print_args("STMP$", NULL, NULL, NULL, NULL);}
   else if (module_flag == T_SUBROUTINE && code_type != EMULATE)
      {print_opcode("extern_", T_EXTERN, T_NONE, T_NONE);
       if(wt == WT) print_args("STMP_", NULL, NULL, NULL, NULL);
		 else print_args("STMP$", NULL, NULL, NULL, NULL);}

   /* Initialize variable sca_temp_start */
   if(wt == WT) strcpy(string,"SFTMP_+");
   else strcpy(string,"SFTMP$+");
   sca_temp_start = INT_SCA_TEMP_START;
   if(debug==2) printf("At real_sca\n");
   for (i=REAL_SCA_TEMP_START, j=0; i<=REAL_SCA_TEMP_END; i++, j++)
     {pc_itoa(j,num);
      strcpy(stringnum,string);
      strcat(stringnum,num);
      symbol_table[i].tokpts = (char *)calloc(strlen(stringnum)+1, sizeof(char));
      strcpy(symbol_table[i].tokpts,stringnum);
      symbol_table[i].status = BUSY;
      symbol_table[i].mode1 = T_REAL;
      symbol_table[i].mode2 = T_SCALAR;
      symbol_table[i].toktype = T_ID;
      symbol_table[i].num_dim = 0;
      symbol_table[i].width = W_REAL;
      symbol_table[i].expt = W_EXPT;
      symbol_table[i].signif = W_SIGNIF;
      symbol_table[i].temp = TRUE;
      symbol_table[i].offset = 0;
		symbol_table[i].iospec = shapedx;}
   i=REAL_SCA_TEMP_START; /* reset i for below */
   pc_itoa((REAL_SCA_TEMP_END - REAL_SCA_TEMP_START + 1),num);
   pc_itoa(symbol_table[i].mode1*SIZE_MODE2+symbol_table[i].mode2,c_mode);
   pc_itoa(symbol_table[i].width,c_width);
   pc_itoa(symbol_table[i].expt,c_expt);
   pc_itoa(symbol_table[i].signif,c_signif);
	pnt = index(string,'+');
	*pnt = 0;
   install_scalar(string, num, "0", c_mode, c_width, c_expt, c_signif,TRUE,shapedx);
   if (module_flag == T_MAIN || wt_sub==WT)
      {print_opcode("entry_", T_ENTRY, NULL, NULL);
       print_args(string, NULL, NULL, NULL, NULL);}
   else if (module_flag == T_SUBROUTINE  && code_type != EMULATE)
      {print_opcode("extern_", T_EXTERN, T_NONE, T_NONE);
       print_args(string, NULL, NULL, NULL, NULL);}

   /* create scalar output buffer */
   if(wt == WT) strcpy(string,"OUTBF_");
   else strcpy(string,"OUTBF$");
   symbol_table[SC_IO_BUFFER].tokpts = (char *)calloc(strlen(string)+1, sizeof(char));/* jlpbug */
   strcpy(symbol_table[SC_IO_BUFFER].tokpts,string);
   symbol_table[SC_IO_BUFFER].status = BUSY;
   symbol_table[SC_IO_BUFFER].mode1 = T_INT;
   symbol_table[SC_IO_BUFFER].mode2 = T_SCALAR;
   symbol_table[SC_IO_BUFFER].toktype = T_ID;
   symbol_table[SC_IO_BUFFER].num_dim = 1;
   symbol_table[SC_IO_BUFFER].width = W_INT;
   symbol_table[SC_IO_BUFFER].expt = 0;
   symbol_table[SC_IO_BUFFER].signif = W_INT;
   symbol_table[SC_IO_BUFFER].temp = FALSE;
   symbol_table[SC_IO_BUFFER].offset = 0;
   symbol_table[SC_IO_BUFFER].iospec = shapedx;

   /* generate entry/extern quadruples */
   pc_itoa(OUTBF_SIZE,num);
   pc_itoa(symbol_table[SC_IO_BUFFER].mode1*SIZE_MODE2+
	   symbol_table[SC_IO_BUFFER].mode2,c_mode);
   pc_itoa(symbol_table[SC_IO_BUFFER].width,c_width);
   pc_itoa(symbol_table[SC_IO_BUFFER].expt,c_expt);
   pc_itoa(symbol_table[SC_IO_BUFFER].signif,c_signif);
   install_scalar (string, num, "0", c_mode, c_width, c_expt, c_signif,FALSE,shapedx);
   if (module_flag == T_MAIN || wt == WT)
     {print_opcode("entry_", T_ENTRY, T_NONE, T_NONE);
      print_args(string, NULL, NULL, NULL, NULL);}
   else if (module_flag == T_SUBROUTINE && code_type != EMULATE)
     {print_opcode("extern_", T_EXTERN, T_NONE, T_NONE);
      print_args(string, NULL, NULL, NULL, NULL);}

   /* create scalar stack */
   if(debug==2) printf("At scalar stack\n");
   if(wt == WT) strcpy(string,"STACK_");
   else strcpy(string,"STACK$");
   symbol_table[PARALLEL_STACK].tokpts = (char *)calloc(strlen(string)+1, sizeof(char)); /* jlpbug */
   strcpy(symbol_table[PARALLEL_STACK].tokpts,string);
   symbol_table[PARALLEL_STACK].status = BUSY;
   symbol_table[PARALLEL_STACK].mode1 = T_INT;
   symbol_table[PARALLEL_STACK].mode2 = T_PARALLEL;
   symbol_table[PARALLEL_STACK].toktype = T_ID;
   symbol_table[PARALLEL_STACK].num_dim = 32;
   symbol_table[PARALLEL_STACK].width = 32; /*set size for dumping */
   symbol_table[PARALLEL_STACK].expt = 0;
   symbol_table[PARALLEL_STACK].signif = 32;
   symbol_table[PARALLEL_STACK].temp = FALSE;
   symbol_table[PARALLEL_STACK].offset = 0;
   symbol_table[PARALLEL_STACK].iospec = shapedx;

   /* generate entry/extern quadruples */
   pc_itoa(symbol_table[PARALLEL_STACK].num_dim,num);
   pc_itoa(symbol_table[PARALLEL_STACK].mode1*SIZE_MODE2+
	   symbol_table[PARALLEL_STACK].mode2,c_mode);
   pc_itoa(symbol_table[PARALLEL_STACK].width,c_width);
   pc_itoa(symbol_table[PARALLEL_STACK].expt,c_expt);
   pc_itoa(symbol_table[PARALLEL_STACK].signif,c_signif);
   install_scalar (string,num, "0", c_mode, c_width, c_expt, c_signif,FALSE,shapedx);
   if (module_flag == T_MAIN || wt == WT)
     {print_opcode("entry_", T_ENTRY, T_NONE, T_NONE);
      print_args(string, NULL, NULL, NULL, NULL);}
   else if (module_flag == T_SUBROUTINE && code_type != EMULATE)
     {print_opcode("extern_", T_EXTERN, T_NONE, T_NONE);
      print_args(string, NULL, NULL, NULL, NULL);}

   /* create scalar stack pointer*/
   if(wt == WT) {strcpy(string,"host_stack_"); pc_itoa(STACK_SIZE,num);}
   else {strcpy(string,"host_stack$"); pc_itoa(0,num);}
   symbol_table[SCALAR_STACK].tokpts = (char *)calloc(strlen(string)+1, sizeof(char));
   strcpy(symbol_table[SCALAR_STACK].tokpts,string);
   symbol_table[SCALAR_STACK].status = BUSY;
   symbol_table[SCALAR_STACK].mode1 = T_INT;
   symbol_table[SCALAR_STACK].mode2 = T_SCALAR;
   symbol_table[SCALAR_STACK].toktype = T_ID;
   symbol_table[SCALAR_STACK].num_dim = 0;
   symbol_table[SCALAR_STACK].width = W_INT;
   symbol_table[SCALAR_STACK].expt = 0;
   symbol_table[SCALAR_STACK].signif =  W_INT;
   symbol_table[SCALAR_STACK].temp = FALSE;
   symbol_table[SCALAR_STACK].offset = 0;
   symbol_table[SCALAR_STACK].iospec = shapedx;

   /* generate entry/extern quadruples */
   if(debug==2) printf("At entry/extern\n");
   pc_itoa(symbol_table[SCALAR_STACK].mode1*SIZE_MODE2+
	   symbol_table[SCALAR_STACK].mode2,c_mode);
   pc_itoa(symbol_table[SCALAR_STACK].width,c_width);
   pc_itoa(symbol_table[SCALAR_STACK].expt,c_expt);
   pc_itoa(symbol_table[SCALAR_STACK].signif,c_signif);
   install_scalar (string,"1",num,c_mode,c_width,c_expt,c_signif,FALSE,shapedx);
   if (module_flag == T_MAIN || wt_sub == WT)
     {print_opcode("entry_", T_ENTRY, T_NONE, T_NONE);
      print_args(string, NULL, NULL, NULL, NULL);}
   else if (module_flag == T_SUBROUTINE && code_type != EMULATE)
     {print_opcode("extern_", T_EXTERN, T_NONE, T_NONE);
      print_args(string, NULL, NULL, NULL, NULL);}

/* Create integer parallel temporary identifiers. */
   if(wt == WT) strcpy(string,"IPTMP_+");
   else strcpy(string,"IPTMP$+");
   if(debug==2) printf("At int temps\n");
   for (i = INT_PAR_TEMP_START, j = 0; i <= INT_PAR_TEMP_END; i++, j += W_INT)  /* modified 880705 jlp */
     {pc_itoa(j,num);
      strcpy(stringnum,string);
      strcat(stringnum,num);
      symbol_table[i].tokpts = (char *)calloc(strlen(stringnum)+1, sizeof(char)); /* jlpbug */
      strcpy(symbol_table[i].tokpts,stringnum);
      symbol_table[i].status = BUSY;
      symbol_table[i].mode1 = T_INT;
      symbol_table[i].mode2 = T_PARALLEL;
      symbol_table[i].toktype = T_ID;
      symbol_table[i].num_dim = 1;
      symbol_table[i].width = W_INT;
      symbol_table[i].expt = 0;
      symbol_table[i].signif = W_INT;
      symbol_table[i].temp = TRUE;
      symbol_table[i].offset = 0;
      symbol_table[i].iospec = shapedx;}
   pnt = index(string,'+');
	*pnt = 0;
	if(module_flag != T_SUBROUTINE || code_type != EMULATE || wt_sub == WT)
     {print_opcode ("temp_", T_TEMP, T_INT, T_PARALLEL);
      pc_itoa((INT_PAR_TEMP_END - INT_PAR_TEMP_START + 1), num);
      pc_itoa(W_INT, field_width);
      print_args6("IPTMP$", num, field_width, "0", field_width,
						 symbol_table[shapedx].tokpts);}

   /* Create real parallel temporary identifiers. */

   if(wt == WT) strcpy(string,"RPTMP_+");
   else strcpy(string,"RPTMP$+");
   if(debug==2) printf("At real temps\n");
   for (i = REAL_PAR_TEMP_START, j = 0; i <= REAL_PAR_TEMP_END; i++, j+=W_REAL)/* modified 880705 jlp */
     {pc_itoa(j,num);
      strcpy(stringnum,string);
      strcat(stringnum,num);
      symbol_table[i].tokpts = (char *)calloc(strlen(stringnum)+1, sizeof(char)); /* jlpbug */
      strcpy(symbol_table[i].tokpts,stringnum);
      symbol_table[i].status = BUSY;
      symbol_table[i].mode1 = T_REAL;
      symbol_table[i].mode2 = T_PARALLEL;
      symbol_table[i].toktype = T_ID;
      symbol_table[i].num_dim = 1;
      symbol_table[i].width = W_REAL;
      symbol_table[i].expt = W_EXPT;
      symbol_table[i].signif = W_SIGNIF;
      symbol_table[i].temp = TRUE;
      symbol_table[i].offset = j;   /* used in p1_symta - get_symtab_temp */
      symbol_table[i].iospec = shapedx;}
   pnt = index(string,'+');
	*pnt = 0;
	if(module_flag != T_SUBROUTINE || code_type != EMULATE || wt == WT)
     {print_opcode ("temp_", T_TEMP, T_REAL, T_PARALLEL);
      pc_itoa((REAL_PAR_TEMP_END - REAL_PAR_TEMP_START + 1), num);
      pc_itoa(W_REAL, field_width);
      pc_itoa(W_EXPT, expt_width);
      pc_itoa(W_SIGNIF, signif_width);
      print_args6(string, num, field_width, expt_width, signif_width,
						 symbol_table[shapedx].tokpts);}

   /* Create logical parallel temporary identifiers. */
   if(wt == WT) strcpy(string,"LPTMP_+");
   else strcpy(string,"LPTMP$+");
   if(debug==2) printf("At log temps\n");
   for (i=LOG_PAR_TEMP_START, j=0; i<=LOG_PAR_TEMP_END; i++, j+=W_LOGICAL)
     {pc_itoa(j,num);
      strcpy(stringnum,string);
      strcat(stringnum,num);
      symbol_table[i].tokpts = (char *)calloc(strlen(stringnum)+1, sizeof(char));
      strcpy(symbol_table[i].tokpts,stringnum);
      symbol_table[i].status = BUSY;
      symbol_table[i].mode1 = T_LOGICAL;
      symbol_table[i].mode2 = T_PARALLEL;
      symbol_table[i].toktype = T_ID;
      symbol_table[i].num_dim = 1;
      symbol_table[i].width = W_LOGICAL;
      symbol_table[i].expt = 0;
      symbol_table[i].signif = W_LOGICAL;
      symbol_table[i].temp = TRUE;
      symbol_table[i].offset = 0;
      symbol_table[i].iospec = shapedx;}
   pnt = index(string,'+');
	*pnt = 0;
	if(module_flag != T_SUBROUTINE || code_type != EMULATE || wt_sub == WT)
     {print_opcode ("temp_", T_TEMP, T_LOGICAL, T_PARALLEL);
      pc_itoa((LOG_PAR_TEMP_END - LOG_PAR_TEMP_START + 1), num);
      pc_itoa(W_LOGICAL, field_width);
      print_args6(string, num, field_width,"0", field_width,
						 symbol_table[shapedx].tokpts);}

   /* init log_par_temp_start */
   log_par_temp_start = LOG_PAR_TEMP_START;
   g_offset_start = G_OFFSET_START;

   /* Create long integer parallel temporary identifiers. */

   if(wt == WT) strcpy(string,"LITMP_+");
   else strcpy(string,"LITMP$+");
   if(debug==2) printf("At lint temps\n");
   for (i=LINT_PAR_TEMP_START, j=0; i<=LINT_PAR_TEMP_END; i++, j+=W_LONGINT) /* modified 880705 jlp */
     {pc_itoa(j,num);
      strcpy(stringnum,string);
      strcat(stringnum,num);
      symbol_table[i].tokpts = (char *)calloc(strlen(stringnum)+1,sizeof(char)); /* jlpbug */
      strcpy(symbol_table[i].tokpts,stringnum);
      symbol_table[i].status = BUSY;
      symbol_table[i].mode1 = T_LONGINT;
      symbol_table[i].mode2 = T_PARALLEL;
      symbol_table[i].toktype = T_ID;
      symbol_table[i].num_dim = 1;
      symbol_table[i].width = W_LONGINT;
      symbol_table[i].expt = 0;
      symbol_table[i].signif = W_LONGINT;
      symbol_table[i].temp = TRUE;
      symbol_table[i].offset = 0;
      symbol_table[i].iospec = shapedx;
       }
   pnt = index(string,'+');
	*pnt = 0;
	if(module_flag != T_SUBROUTINE || code_type != EMULATE || wt_sub == WT)
     {print_opcode ("temp_",T_TEMP, T_LONGINT, T_PARALLEL);
      pc_itoa((LINT_PAR_TEMP_END - LINT_PAR_TEMP_START + 1), num);
      pc_itoa((W_LONGINT), field_width);
      pc_itoa(0,expt_width);
      print_args6(string, num, field_width, expt_width,field_width,
						 symbol_table[shapedx].tokpts);}

   /* Insert unique parallel logical true value location. */
   strcpy(string,"TRUE$");
   symbol_table[LP_TRUE].tokpts = (char *)calloc(strlen(string)+1, sizeof(char));
   strcpy(symbol_table[LP_TRUE].tokpts, string);
   symbol_table[LP_TRUE].status = BUSY;
   symbol_table[LP_TRUE].mode1 = T_LOGICAL;
   symbol_table[LP_TRUE].mode2 = T_PARALLEL;
   symbol_table[LP_TRUE].toktype = T_TRUE;
   symbol_table[LP_TRUE].width = W_LOGICAL;
   symbol_table[LP_TRUE].expt = 0;
   symbol_table[LP_TRUE].signif = W_LOGICAL;
   symbol_table[LP_TRUE].temp = FALSE;
   symbol_table[LP_TRUE].offset = 0;
   symbol_table[LP_TRUE].iospec = shapedx;

	if(module_flag != T_SUBROUTINE || code_type != EMULATE || wt_sub == WT)
     {print_opcode ("decl_", T_DECLARE, T_LOGICAL, T_PARALLEL);
      pc_itoa(W_LOGICAL, field_width);
      pc_itoa(0,expt_width);
      print_args6(string, "1", field_width, expt_width, field_width,
						 symbol_table[shapedx].tokpts);}

   /*  Insert unique parallel logical false value location. */
   if(debug==2) printf("At lp_false\n");
   if(wt == WT) strcpy(string,"FALSE_");
   else strcpy(string,"FALSE$");
   symbol_table[LP_FALSE].tokpts = (char *)calloc(strlen(string)+1, sizeof(char));
   strcpy(symbol_table[LP_FALSE].tokpts, string);
   symbol_table[LP_FALSE].status = BUSY;
   symbol_table[LP_FALSE].mode1 = T_LOGICAL;
   symbol_table[LP_FALSE].mode2 = T_PARALLEL;
   symbol_table[LP_FALSE].toktype = T_FALSE;
   symbol_table[LP_FALSE].width = W_LOGICAL;
   symbol_table[LP_FALSE].expt = 0;
   symbol_table[LP_FALSE].signif = W_LOGICAL;
   symbol_table[LP_FALSE].temp = FALSE;
   symbol_table[LP_FALSE].offset = 0;
   symbol_table[LP_FALSE].iospec = shapedx;

	if(module_flag != T_SUBROUTINE || code_type != EMULATE || wt == WT)
     {print_opcode ("decl_", T_DECLARE, T_LOGICAL, T_PARALLEL);
      pc_itoa(W_LOGICAL, field_width);
      pc_itoa(0,expt_width);
      print_args6(string, "1", field_width, expt_width, field_width,
						 symbol_table[shapedx].tokpts);}

   /* Insert unique parallel subscript '$' in last location. */
   symbol_table[SYMTAB1_SIZE-1].tokpts = (char *)calloc(5, sizeof(char));/* jlpbug */
   strcpy(symbol_table[SYMTAB1_SIZE-1].tokpts, "$");
   symbol_table[SYMTAB1_SIZE-1].status = BUSY;
   symbol_table[SYMTAB1_SIZE-1].mode1 = T_LOGICAL;
   symbol_table[SYMTAB1_SIZE-1].mode2 = T_PARALLEL;
   symbol_table[SYMTAB1_SIZE-1].toktype = T_PAR_SUBSCRIPT;
   symbol_table[SYMTAB1_SIZE-1].width = W_LOGICAL;
   symbol_table[SYMTAB1_SIZE-1].expt = 0;
   symbol_table[SYMTAB1_SIZE-1].signif = W_LOGICAL;
   symbol_table[SYMTAB1_SIZE-1].temp = FALSE;
   symbol_table[SYMTAB1_SIZE-1].offset = 0;
   symbol_table[SYMTAB1_SIZE-1].iospec = shapedx;

   if(debug==2) printf("At max_num_dim\n");
   for (i=0; i<MAX_NUM_DIM; i++)
		dim_size[i] = 0;

   symtab_insert(T_ID,T_LOGICAL,T_PARALLEL,6,"IOMASK",dim_size,1,0,1,1);
	print_opcode("decl_",T_DECLARE, T_LOGICAL, T_PARALLEL);
	print_args6("IOMASK","1","1","0","1",symbol_table[shapedx].tokpts);

   get_const("10",T_DEC_CONST); /* assure 2, 8, 10 and 16 are in */
   get_const("8",T_DEC_CONST);  /* scalar table for em_print routines */
   get_const("2",T_DEC_CONST);
   get_const("1",T_DEC_CONST);
   get_const("0",T_DEC_CONST);
   get_const("16",T_DEC_CONST);
   get_const("48",T_DEC_CONST); /* 48 is ASCII zero C48D$ */
   get_const("45",T_DEC_CONST); /* 47 is ASCII minus sign  C45D$ */
   get_const("32",T_DEC_CONST); /* 32 is ASCII blank C32D$ */

/* insert peid - the pe identification field in symbol table */
   if(debug==2) printf("At peid\n");
   if(code_type == EMULATE && wt == WT)
	  {print_opcode("defvar_",T_DEFVAR, T_NONE, T_NONE);
	   print_args("YID","PEID","256","0",NULL);
	   pc_itoa(yid_siz,c_siz);
      print_opcode("defvar_",T_DEFVAR, T_NONE, T_NONE);
	   print_args("XID","PEID","256",c_siz,NULL);
	   pc_itoa(yid_siz+xid_siz,c_siz);
      print_opcode("defvar_",T_DEFVAR, T_NONE, T_NONE);
	   print_args("ZID","PEID","0",c_siz,NULL);
      print_opcode ("decl_", T_DECLARE, T_INT, T_PARALLEL);
      print_args6("SYS1_", "1", "32", "0", "32",symbol_table[shapedx].tokpts);
      symtab_insert(T_ID,T_INT,T_PARALLEL,5,"SYS1$",dim_size,32,0,32,1);
      print_opcode ("decl_", T_DECLARE, T_INT, T_PARALLEL);
      print_args6("SYS2_", "1", "32", "0", "32",symbol_table[shapedx].tokpts );
      symtab_insert(T_ID,T_INT,T_PARALLEL,5,"SYS2$",dim_size,32,0,32,1);
      print_opcode ("decl_", T_DECLARE, T_INT, T_PARALLEL);
      print_args6("SYS3_", "1", "64", "0", "64",symbol_table[shapedx].tokpts );
      symtab_insert(T_ID,T_INT,T_PARALLEL,5,"SYS3$",dim_size,64,0,64,1);
      symtab_insert(T_ID,T_CARD,T_PARALLEL,4,"PEID",dim_size,PEID_SZ,0,PEID_SZ,1);
      pc_itoa(PEID_SZ,c_siz);
		print_opcode("decl_",T_DECLARE, T_LOGICAL, T_PARALLEL);
		print_args6("PEID","1",c_siz,"0",c_siz,symbol_table[shapedx].tokpts);
		symtab_insert(T_ID,T_CARD,T_PARALLEL,3,"XID",dim_size,xid_siz,0,xid_siz,1);
      pc_itoa(xid_siz,c_siz);
		print_opcode("decl_",T_DECLARE, T_LOGICAL, T_PARALLEL);
		print_args6("XID","1",c_siz,"0",c_siz,symbol_table[shapedx].tokpts);
		symtab_insert(T_ID,T_CARD,T_PARALLEL,3,"YID",dim_size,yid_siz,0,yid_siz,1);
      pc_itoa(yid_siz,c_siz);
		print_opcode("decl_",T_DECLARE, T_LOGICAL, T_PARALLEL);
		print_args6("YID","1",c_siz,"0",c_siz,symbol_table[shapedx].tokpts);
		symtab_insert(T_ID,T_CARD,T_PARALLEL,3,"ZID",dim_size,zid_siz,0,zid_siz,1);
      pc_itoa(zid_siz,c_siz);
		print_opcode("decl_",T_DECLARE, T_LOGICAL, T_PARALLEL);
		print_args6("ZID","1",c_siz,"0",c_siz,symbol_table[shapedx].tokpts);
		symtab_insert(T_ID,T_LOGICAL,T_PARALLEL,6,"ASCTMP",dim_size,1,0,1,1);
		print_opcode("decl_",T_DECLARE, T_LOGICAL, T_PARALLEL);
		print_args6("ASCTMP","1","1","0","1",symbol_table[shapedx].tokpts);
		symtab_insert(T_ID,T_LOGICAL,T_PARALLEL,7,"ASCTMP1",dim_size,1,0,1,1);
		print_opcode("decl_",T_DECLARE, T_LOGICAL, T_PARALLEL);
		print_args6("ASCTMP1","1","1","0","1",symbol_table[shapedx].tokpts);
		symtab_insert(T_ID,T_CARD,T_PARALLEL,7,"IOSTACK",dim_size,256,0,256,1);
		print_opcode("decl_",T_DECLARE, T_CARD, T_PARALLEL);
		print_args6("IOSTACK","1","256","0","256",symbol_table[shapedx].tokpts);
     }

   if(debug==2) printf("At THEM\n");
   symtab_insert(T_ID,T_LOGICAL,T_PARALLEL,10,"GLOBAL_BI$",dim_size,1,0,1,1);
   symtab_insert(T_ID,T_LOGICAL,T_PARALLEL,4,"THEM",dim_size,1,0,1,1);
   symtab_insert(T_ID,T_CHAR,T_PARALLEL,4,"LEX$",dim_size,T_LEXSIZ,0,T_LEXSIZ,1);
   symtab_insert(T_ID,T_CHAR,T_PARALLEL,8,"LEX_VAL$",dim_size,T_LEXSIZ,0,T_LEXSIZ,1);
   symtab_insert(T_ID,T_LOGICAL,T_PARALLEL,7,"LEX_BI$",dim_size,1,0,1,1);
   symtab_insert(T_ID,T_LOGICAL,T_PARALLEL,5,"THEIR",dim_size,1,0,1,1);
   symtab_insert(T_INDEX,T_LOGICAL,T_PARALLEL,3,"ITS",dim_size,1,0,1,1);
   symtab_insert(T_ID,T_LOGICAL,T_PARALLEL,3,"THE",dim_size,1,0,1,1);
   symtab_insert(T_ID,T_LOGICAL,T_PARALLEL,2,"AN",dim_size,1,0,1,1);
   symtab_insert(T_ID,T_LOGICAL,T_PARALLEL,1,"A",dim_size,1,0,1,1);
   symtab_insert(T_ID,T_LOGICAL,T_PARALLEL,1,"@",dim_size,1,0,1,1);
	if(module_flag != T_SUBROUTINE || code_type != EMULATE || wt_sub == WT)
     {print_opcode ("decl_", T_DECLARE, T_LOGICAL, T_PARALLEL);
      print_args6("THEM", "1", "1", "0", "1" ,symbol_table[shapedx].tokpts);
      pc_itoa(T_LEXSIZ, field_width);
      print_opcode ("decl_", T_DECLARE, T_INT, T_PARALLEL);
      print_args6("LEX$", "1", field_width, "0", field_width,symbol_table[shapedx].tokpts );
      print_opcode ("decl_", T_DECLARE, T_INT, T_PARALLEL);
      print_args6("LEX_VAL$", "1", field_width, "0", field_width,symbol_table[shapedx].tokpts );
      print_opcode("decl_",T_DECLARE, T_LOGICAL, T_PARALLEL);
      print_args6("LEX_BI$","1","1","0","1",symbol_table[shapedx].tokpts);
      print_opcode("decl_",T_DECLARE, T_LOGICAL, T_PARALLEL);
      print_args6("GLOBAL_BI$","1","1","0","1",symbol_table[shapedx].tokpts);
      print_opcode ("decl_", T_DECLARE, T_LOGICAL, T_PARALLEL);
      print_args6("THEIR", "1", "1", "0", "1",symbol_table[shapedx].tokpts);
      print_opcode ("decl_", T_DECLARE, T_LOGICAL, T_PARALLEL);
      print_args6("ITS", "1", "1", "0", "1",symbol_table[shapedx].tokpts);
      print_opcode ("decl_", T_DECLARE, T_LOGICAL, T_PARALLEL);
      print_args6("THE", "1", "1", "0", "1",symbol_table[shapedx].tokpts);
      print_opcode ("decl_", T_DECLARE, T_LOGICAL, T_PARALLEL);
      print_args6("AN", "1", "1", "0", "1",symbol_table[shapedx].tokpts);
      print_opcode ("decl_", T_DECLARE, T_LOGICAL, T_PARALLEL);
      print_args6("A", "1", "1", "0", "1",symbol_table[shapedx].tokpts);
      print_opcode ("decl_", T_DECLARE, T_LOGICAL, T_PARALLEL);
      print_args6("@", "1", "1", "0", "1",symbol_table[shapedx].tokpts);}

   symtab_insert(T_ID,T_LOGICAL,T_SCALAR,7,"PERFORM",dim_size,1,0,1,0);
   symtab_insert(T_ID,T_LOGICAL,T_SCALAR,10,"PA_PERFORM",dim_size,1,0,1,0);
   symtab_insert(T_ID,T_LOGICAL,T_SCALAR,10,"SC_PERFORM",dim_size,1,0,1,0);
	if(module_flag != T_SUBROUTINE || code_type != EMULATE || wt_sub == WT)
     {print_opcode ("decl_", T_DECLARE, T_INT, T_SCALAR);
      print_args6("PERFORM", "1", "1", "0", "1",symbol_table[shapedx].tokpts);
      print_opcode ("decl_", T_DECLARE, T_INT, T_SCALAR);
      print_args6("PA_PERFORM", "1", "1", "0", "1",symbol_table[shapedx].tokpts);
      print_opcode ("decl_", T_DECLARE, T_INT, T_SCALAR);
      print_args6("SC_PERFORM", "1", "1", "0", "1",symbol_table[shapedx].tokpts);}

/* HANDLE$ is hard coded in T_LOAD and therefore need not be in the symbol table */
	print_opcode ("decl_", T_DECLARE, T_INT, T_SCALAR);
    print_args6("HANDLE$", "2", "0", "0", "16",symbol_table[shapedx].tokpts);
} /* end of init_symbol_table() */

