/* P1_KEYWORD.C  910415 */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"

/* =========================================================================*/

struct asprol_key
	 {/* format of ASPROL keyword table */
		    char *name; /* pointer to keyword */
		    int type;           /* token type of keyword */
		   };

/*==========================================================================*/

struct asprol_key asprol_key_table[] =  /* ASPROL compiler keyword table */
{
		      "ANY", T_ANY,
		      "ALLOCATE", T_ALLOCATE,
	    "ASMCODE", T_ASSEMBLE,
		      "ASSOCIATE", T_ASSOCIATE,
		      "BIN", T_BIN,
		      "CALL", T_CALL,
		      "CARD", T_CARD,
		      "CHAR", T_CHAR,
	    "CLOSE", T_CLOSE,
		       "CLEAR", T_CLEAR,
				"CONTIGUOUSLY",T_CONTIG,
	    "COPY",T_COPY,
		      "COS", T_COS,
				"COUNT", T_COUNT,
				"DECLAR", T_DECLAR,
		      "DEFINE", T_DEFINE,
		      "DEFVAR", T_DEFVAR,
		      "DEFLOG", T_DEFLOG,
		      "OUTPUT", T_OUTPUT,
		      "DECODE", T_DECODE,
		      "DO", T_DO,
		      "do", T_DO,
		      "INPUT", T_DINPUT,
		      "DURING", T_DURING,
		      "ENCODE", T_ENCODE,
		      "end", T_END,
		      "END", T_END,
		      "ENDALLOCATE", T_ENDALLOCATE,
		      "ENDANY", T_ENDANY,
		      "ENDFOR", T_ENDFOR,
		      "ENDGET", T_ENDGET,
		      "ENDIF", T_ENDIF,
		      "ENDIFNANY", T_ENDIFNANY,
		      "ENDLOOP", T_ENDLOOP,
		      "ENDNEXT", T_ENDNEXT,
		      "ENDSETSCOPE", T_ENDSETSCOPE,
		      "ENDWHILE", T_ENDWHILE,
		      "ELSE", T_ELSE,
		      "ELSENANY", T_ELSENANY,
		      "ELSEWHERE", T_ELSEWHERE,
		      "elsewhere", T_ELSEWHERE,
		      "EXP", T_EXP,
/*                    "FALSE", T_FALSE, */
		      "FOR", T_FOR,
		      "for", T_FOR,
		      "FIRST", T_FIRST,
	    "FROM", T_FROM,
				"FSTCD", T_FSTCD,
				"FSTCD8", T_FSTCD8,
		      "GET", T_GET,
	    "GETADD", T_GETADD,
	    "HEX", T_HEX,
            "HIT", T_HIT,
		      "IN", T_IN,
		      "#INCLUDE", T_INCLUDE,
		      "#include", T_INCLUDE,
		      "INDEX", T_INDEX,
		      "INT", T_INT,
		      "#IOSPEC", T_IOSPEC,
		      "#iospec", T_IOSPEC,
		      "IF", T_IF,
/*                    "IFNANY", T_IFNANY,   */
		      "LOGICAL", T_LOGICAL,
	    "LONGINT", T_LONGINT,
		      "LOOP", T_LOOP,
		      "MAIN", T_MAIN,
		      "MAXDEX", T_MAXDEX,
		      "MAXVAL", T_MAXVAL,
		      "MINDEX", T_MINDEX,
		      "MINVAL", T_MINVAL,
	    "MSG", T_MSG,
	    "NANY", T_NANY,
		      "NEXT", T_NEXT,
	    "NTHDEX", T_NTHDEX,
				"NTHVAL", T_NTHVAL,
	    "NXTCD", T_NXTCD,
	    "NXTCD8", T_NXTCD8,
		      "OCT", T_OCT,
				"OPEN", T_OPEN,
				"LOAD", T_LOAD,
		      "PARALLEL", T_PARALLEL,
	    "PREFIX", T_PREFIX,
				"PREFIX8", T_PREFIX8,
				"PRVCD",T_PREVCD,
				"PRVCD8",T_PREVCD8,
                      "RAND", T_RAND,
		      "REAL", T_REAL,
		      "RETURN", T_RETURN,
	    "RELEASE", T_RELEASE,
		      "SETSCOPE", T_SETSCOPE,
		      "SCALAR", T_SCALAR,
				"SCIN",T_SCIN,
				"SCINL",T_SCINL,
				"SCINP",T_SCINP,
				"SCIN8",T_SCIN8,
				"SCOT",T_SCOT,
				"SCOTL",T_SCOTL,
				"SCOTP",T_SCOTP,
				"SCOT8",T_SCOT8,
				"STORE",T_STORE,
				"SIBDEX",T_SIBDEX,
	    "SILENCE", T_SILENCE,
		      "SIN",T_SIN,
		      "SQRT", T_SQRT,
		      "STOP", T_STOP,
		      "SUBROUTINE", T_SUBROUTINE,
	    "SUM",T_SUM,
	    "table", T_INCLUDE,
		      "THEN", T_THEN,
		      "THENANY", T_THENANY,
            "TKNCMP",T_TKNCMP,
	    "TRNCD",T_TRNCD,
	    "TRNCD8",T_TRNCD8,
				"TRNACD",T_TRNACD,
				"TRNACD8",T_TRNACD8,
		      "TRUE$", T_TRUE,
		      "UNTIL", T_UNTIL,
		      "USING", T_USING,
		      "WHILE", T_WHILE,
		      "WITH", T_WITH,
		      "WHEREVER", T_WHEREVER,
		      "wherever", T_WHEREVER,
		      "ANDFOR",T_ANDFOR,
		      "ENDANDFOR",T_ENDANDFOR,
		      "ANDIF",T_ANDIF,
		      "ANDTHEN",T_ATHEN,
		      "ENDANDIF",T_ENDANDIF,
		      "ANDCONS",T_ANDCONS,
		      "ENDANDCONS",T_ENDANDCONS,
		      "PRINT",T_PRINT,
		      "READ",T_READ,
				"REREAD", T_REREAD,
	    "READLN",T_READLN,
		      "RECURSE",T_RECURSE,
		      "ENDRECURSE",T_ENDRECURSE,
	    "SHAPE",T_SHAPE,
	    "STACK",T_RECURSE,
	    "ENDSTACK",T_ENDRECURSE,
	    "STACKFOR",T_STACKFOR,
	    "ENDSTACKFOR",T_ENDSTACKFOR,
		      "STACKWHILE",T_STACKWHILE,
	    "ENDSTACKWHILE",T_ENDSTACKWHILE,
		      "RECURSEWHILE",T_STACKWHILE,
	    "ENDRECURSEWHILE",T_ENDSTACKWHILE
		   };

/*==========================================================================*/

char *aspro_key_table[] =               /* ASPRO assembler keyword table */
		   {
		      "R0","R1","R2","R3","R4","R5","R6","R7","R8","R9",
		      "R10","R11","R12","R13","R14","R15","X","Y","M",
		      "CH","CL","C","FP1","FP2","FP3","FP4",
		      "FL","LC","ST","PU","PUH","PUL"
		   };

/*==========================================================================*/

char *staran_key_table[] =              /* STARAN assembler keyword table */
		   {
		      "R0","R1","R2","R3","R4","R5","R6","R7","R8","R9",
		      "RA","RB","RC","RD","RE","RF","X","Y","M",
		      "AS","ASH","ASL","BL","DP","C","CL","CH",
		      "F","FL1","FP3","FP1","FP2","FP12","FL2","FPE",
		      "PC","IMASK","PAGE0","PAGE1","PAGE2","DMA",
		      "BULKC","HSDB"
		   };

/*==========================================================================*/

#define NUM_ASPROL_KEYS  (sizeof(asprol_key_table))/(sizeof(struct asprol_key))
#define NUM_ASPRO_KEYS   (sizeof(aspro_key_table))/(sizeof(char *))
#define NUM_STARAN_KEYS  (sizeof(staran_key_table))/(sizeof(char *))

/* ===========================================================================

   function name: compiler_keyword
   input        : token_string - pointer to current token string
			    type         - pointer to current token type
   output           : ---
   return value : found          - TRUE, if token is a compiler keyword
						   - FALSE, otherwise

   global vars  : asprol_key_table - array of ASPROL keywords and token types

   description   : This routine checks to see if the specified token is a
			    reserved keyword in the ASPROL compiler. If so, its
			    type is changed from T_ID to the appropriate keyword
			    type and TRUE is returned. If the token is not a keyword,
			    the type is not changed and FALSE is returned.

=========================================================================== */

int compiler_keyword(token_string, type)
char *token_string;
int *type;
{
   int i;               /* loop index */
   int found;   /* return value of function: TRUE or FALSE */

   found = FALSE;
   for (i = 0; i < NUM_ASPROL_KEYS && !found; i++)
     {if (strcmp(asprol_key_table[i].name,token_string) == 0)
	{*type = asprol_key_table[i].type;
	      found = TRUE;}
     }
   return(found);
}


/* ===========================================================================

   function name: assembler_keyword
   input               : token_string
   output           : ---
   return value : ---

   global vars  : aspro_key_table       - array of ASPRO keywords
			    staran_key_table    - array of STARAN keywords

   description   : This routine checks to see if the specified token is a
			    reserved keyword in either the ASPRO or STARAN assembler.
			    If so, the token will have a '$' inserted as the first
			    character. If not, the token is not changed. No value is
			    returned.

=========================================================================== */

void assembler_keyword(token_string)
char *token_string;
{
   int i;                       /* loop index */
   int found;           /* loop termination variable */
   char tok_buffer[TOKEN_SIZE]; /* temporary array for token */

   found = FALSE;

   if(code_type == ASPRO)
     for (i = 0; i < NUM_ASPRO_KEYS && !found; i++)
       {if (strcmp(aspro_key_table[i],token_string) == 0)
	  {found = TRUE;
	   strcpy(tok_buffer, token_string);
		strcpy(token_string, "$");
		strncat(token_string, tok_buffer, (TOKEN_SIZE-2));}
       }

   if(code_type == STARAN)
     for (i = 0; i < NUM_STARAN_KEYS && !found; i++)
       {if (strcmp(staran_key_table[i],token_string) == 0)
	  {found = TRUE;
		strcpy(tok_buffer, token_string);
		strcpy(token_string, "$");
		strncat(token_string, tok_buffer, (TOKEN_SIZE-2));}
       }

}


