/* 910415  P1_LOGEXP.C */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"

/* ===========================================================================

   function name : log_exp
   input                : ---
   output            : *result_index, *result_offset
		 : *result_index        - symbol table index of result, if no errors
						     - FAIL, otherwise

   global vars   : debug                - flag for debugging statements

   description    : This routine is part of a recursive descent parser and
			     handles the following production from the grammar:

			LOG_EXP <-- LOG_NOT_EXP MORE_LOG_EXP

		  If an argument (resulting from a possible logical not
		  expression) is obtained, it is passed to 'more_log_exp'
		  to perform a possible logical 'and', 'or', or 'xor'. If
		  the first argument cannot be obtained, FAIL is returned.

=========================================================================== */

int log_exp(result_index,result_offset)
int *result_index,*result_offset;
{
   int arg1_index;              /* symbol table index of argument 1 */
   int arg1_offset;

   if (debug == 2)
     { printf("In log_exp().........................................\n");
      printf("In log_exp, token=%s\n",token_buf);  }

   log_not_exp(&arg1_index,&arg1_offset);
   if (arg1_index!= FAIL)
      more_log_exp(arg1_index,arg1_offset,result_index,result_offset);
   else
     {*result_index = FAIL;
      *result_offset = SUCCESS;}

   if (debug == 2)
      printf("Log_exp() returning *result_index %d\n", *result_index);

   return(*result_index);
}

/* move responders to them */
void save_em(result_index, result_offset)
int result_index, result_offset;
{
   int them_dx;

   if(symbol_table[result_index].mode1 == T_LOGICAL &&
		symbol_table[result_index].mode2 == T_PARALLEL)
	  if((them_dx = symtab_fetch(4,"THEM")) != FAIL)
		  {print_opcode("mvpa_",T_MVPA,T_LOGICAL,T_PARALLEL);
			print_args(off_exp1(result_index,result_offset),
						  symbol_table[them_dx].tokpts,NULL,NULL,NULL);}
	  else printf("ERROR - THEM not found in symbol table\n");
	return;
}



/* ===========================================================================

   function name : more_log_exp
   input                : arg1_index    - symbol table index of first argument
   output            : *result_index, *result_offset
		 : *result_index        - symbol table index of result, if no errors
						     - FAIL, otherwise

   global vars   : symbol_table - array containing symbol table
			     token_type - type of current token
			     debug              - flag for debugging statements

   description    : This routine is part of a recursive descent parser and
			     handles the following production from the grammar:

			MORE_LOG_EXP <-- || LOG_NOT_EXP MORE_LOG_EXP |
					 && LOG_NOT_EXP MORE_LOG_EXP |
					 ^^ LOG_NOT_EXP MORE_LOG_EXP |
					 @

		  '@' represents the empty production. If an 'and', 'or' or
		  'xor' sign is found, the symbol table index of the second
		  argument is obtained. Intermediate code is generated and
		  the symbol table index of the result is passed in a
		  recursive call to this function to handle any other 'ands',
	'ors' or 'xors'. The symbol table index of the final result
		  is returned. If no 'and', 'or' or 'xor' sign is found,
		  the function returns the first argument as it was passed.
		  NOTE: Type-checking is done to ensure that the operands
		  of these operations are type logical-- it is an error
	     if they are not of the correct type.

=========================================================================== */

int more_log_exp(arg1_index,arg1_offset,result_index,result_offset)
int arg1_index,arg1_offset,*result_index,*result_offset;
{
   int op_type;         /* operation type: and, or, xor */
   int op_mode2;                /* operation mode: scalar, parallel, common */
   int result_mode2;    /* result mode: scalar or parallel */
   int arg2_index;      /* symbol table index of second argument */
   int arg2_offset;

   if (debug == 2)
      printf ("Entering more_log_exp().\n ");

   if (token_type == T_AND || token_type == T_OR || token_type == T_XOR)
     {if (symbol_table[arg1_index].mode1 != T_LOGICAL &&
	  symbol_table[arg1_index].mode1 != T_INT     &&
	  symbol_table[arg1_index].mode1 != T_HEX     &&
	  symbol_table[arg1_index].mode1 != T_OCT     &&
	  symbol_table[arg1_index].mode1 != T_BIN       )
	{print_msg(EXPCON_ERR, token_buf);
	 printf("In more_log_exp 1\n");
	 *result_index = FAIL;
	 *result_offset = SUCCESS;}
      else
	{op_type = token_type;
	 token_type = get_token();
	 log_not_exp(&arg2_index,&arg2_offset);
	 if (arg2_index != FAIL)
	   {if (symbol_table[arg2_index].mode1 != T_LOGICAL &&
		symbol_table[arg2_index].mode1 != T_INT     &&
		symbol_table[arg2_index].mode1 != T_HEX     &&
		symbol_table[arg2_index].mode1 != T_OCT     &&
		symbol_table[arg2_index].mode1 != T_BIN        )
		   {print_msg(EXPCON_ERR, token_buf);
		    printf("In more_log_exp 2\n");
	       *result_index = FAIL;
	       *result_offset = SUCCESS;}
	    else
	      {op_mode2 = get_op_mode2 (arg1_index, arg2_index);
		    result_mode2 = get_result_mode2 (op_mode2);
		    *result_index = get_symtab_temp (T_LOGICAL, result_mode2,
				    biggest_size(arg1_index,arg2_index));
	       *result_offset = SUCCESS;
		    if (*result_index != FAIL)
		      {if (op_type == T_OR)
			 print_opcode ("or_", T_OR, T_LOGICAL, op_mode2);
		       else if (op_type == T_AND)
			 print_opcode ("and_", T_AND, T_LOGICAL, op_mode2);
		  else
			 print_opcode ("xor_", T_XOR, T_LOGICAL, op_mode2);
		       print_args (off_exp1(arg1_index,arg1_offset),
					     off_exp2(arg2_index,arg2_offset),
					     off_exp3(*result_index,*result_offset),NULL, NULL);
		       more_log_exp (*result_index,*result_offset,result_index,result_offset);}
	      }
	   }
	 else
	   *result_index = FAIL;
	}
     }
   else
     {*result_index = arg1_index;
      *result_offset = arg1_offset;}

   if (debug == 2)
      printf ("More_log_exp() returning *result_index %d\n", *result_index);

   return(*result_index);
}

/* ===========================================================================

   function name : log_not_exp
   input                : ---
   output            : *result_index, *result_offset
		 : *result_index        - symbol table idex of result, if no errors
						     - FAIL, otherwise

   global vars   : debug                - flag for debugging statements
			     token_type - type of current token
			     symbol_table       - array containing symbol table
			     token_buf  - character string for current token

   description    : This routine is part of a recursive descent parser and
			     handles the following production from the grammar:

			LOG_NOT_EXP <-- ! LOG_NOT_EXP |
					SUM_DIF_EXP MORE_LOG_NOT_EXP

		  If a logical not (!) is found, intermediate code is gen-
		  erated after a call to 'log_exp' returns the index of the
		  item to be operated on. The index of the final result is
		  returned.  If the first token is not '!', the function
		  'sum_dif_exp' is called to see if the first token is part
		  of an arithmetic expression. If not, FAIL is returned.
		  If an arithmetic expression is found, the symbol table index
		  of the result is passed to 'more_log_not_exp' to see if it
		  is part of a relational expression. The return value is
		  the symbol table index of the result of the relational
		  expression, if found, or the symbol table index of the
		  arithmetic expression. NOTE: Type-checking is done to ensure
		  that the operand of the 'not' operation is type logical--
		  it is an error if it is not of the correct type.

=========================================================================== */

int log_not_exp(result_index,result_offset)
int *result_index, *result_offset;
{
   int arg1_index;              /* symbol table index of argument 1 */
   int arg1_offset;

   if (debug == 2)
      {printf("In log_not_exp()..................................\n");   
       printf("In log_not_exp, token=%s, type=%d\n",token_buf,token_type);}

   *result_index = FAIL;
   *result_offset = SUCCESS;

   if (token_type == T_NOT)
     {token_type = get_token();
      log_not_exp(&arg1_index,&arg1_offset);
      if (arg1_index != FAIL)
	{if (symbol_table[arg1_index].mode1 != T_LOGICAL &&
	     symbol_table[arg1_index].mode1 != T_INT     &&
	     symbol_table[arg1_index].mode1 != T_HEX     &&
	     symbol_table[arg1_index].mode1 != T_OCT     &&
	     symbol_table[arg1_index].mode1 != T_BIN        )
	   {print_msg(EXPCON_ERR, token_buf);
	    printf("In log_not_exp 1\n");
	    *result_index = FAIL;
	    *result_offset = SUCCESS;}
	 else
	   {*result_index = get_symtab_temp (T_LOGICAL,
						       symbol_table[arg1_index].mode2,
					     symbol_table[arg1_index].width);
	    *result_offset = SUCCESS;
	    if (*result_index != FAIL)
		   {print_opcode("not_", T_NOT, T_LOGICAL,
			      symbol_table[arg1_index].mode2);
		    print_args (off_exp1(arg1_index,arg1_offset),
			   off_exp3(*result_index,*result_offset),
					  NULL, NULL, NULL);}
	   }
	}
      else
	{*result_index = FAIL;
	 *result_offset = SUCCESS;}
   }
   else if (sum_dif_exp(&arg1_index,&arg1_offset) != FAIL)
     more_log_not_exp(arg1_index,arg1_offset,result_index,result_offset);
   else
     {*result_index = FAIL;
      *result_offset = SUCCESS;}

   if (debug == 2)
      printf("Log_not_exp() returning *result_index %d\n", *result_index);

   return(*result_index);
}

/* ===========================================================================

   function name : more_log_not_exp
   input                : arg1_index    - symbol table index of first argument
   output            : *result_index, *result_offset
		 : *result_index        - symbol table index of result, if no errors
						     - FAIL, otherwise

   global vars   : token_type   - type of current token
			     symbol_table       - array containing symbol table
			     debug              - flag for debugging statements

   description    : This routine is part of a recursive descent parser and
			     handles the following production from the grammar:

			MORE_LOG_NOT_EXP <-- < SUM_DIF_EXP  |
						       > SUM_DIF_EXP  |
						       <= SUM_DIF_EXP |
						       >= SUM_DIF_EXP |
						       == SUM_DIF_EXP |
						       != SUM_DIF_EXP |
						       @

		  '@' represents the empty production. If one of the 6
		  relational operators is not found, the function returns
		  the argument it received as is. Otherwise, the function
		  'sum_dif_exp' is called to find the second argument of the
		  relational expression. Intermediate code is generated
		  if no errors are detected. NOTE: Type-checking is done to
		  ensure that the operands of these operations are type
		  integer or real--type conversions are done between these
		  types as needed.

=========================================================================== */

int more_log_not_exp(arg1_index,arg1_offset,result_index,result_offset)
int arg1_index,arg1_offset,*result_index,*result_offset;
{
   int op_type;         /* type of operation: lt, gt, le, ge, eq, ne */
   int arg2_index;      /* symbol table index of second argument */
   int arg2_offset;
   int op_mode1;           /* operation mode: int or real */
   int op_mode2;           /* operation mode: scalar, parallel, common */
   int result_mode2;    /* result mode: scalar or parallel */

   if (debug == 2)
      printf("In more_log_not_exp()................................\n");

   if ((token_type == T_LT || token_type == T_GT || token_type == T_LE  ||
       token_type == T_GE || token_type == T_EQ || token_type == T_NE) &&
       symbol_table[arg1_index].mode1 != T_LOGICAL)
     {if (symbol_table[arg1_index].mode1 != T_REAL &&
	       symbol_table[arg1_index].mode1 != T_OCT  &&
	       symbol_table[arg1_index].mode1 != T_LONGINT  &&
	       symbol_table[arg1_index].mode1 != T_BIN  &&
	       symbol_table[arg1_index].mode1 != T_HEX  &&
	       symbol_table[arg1_index].mode1 != T_CHAR &&
	       symbol_table[arg1_index].mode1 != T_CARD &&
	       symbol_table[arg1_index].mode1 != T_INT)
	{print_msg(EXPCON_ERR, token_buf);
	    printf("In more_log_not_exp 1\n");
	      *result_index = FAIL;
	      *result_offset = SUCCESS;}
      else
	{op_type = token_type;
	      token_type = get_token();
	      if (sum_dif_exp(&arg2_index,&arg2_offset) != FAIL)
		{if (symbol_table[arg2_index].mode1 != T_REAL &&
			  symbol_table[arg2_index].mode1 != T_OCT  &&
			  symbol_table[arg2_index].mode1 != T_HEX  &&
			  symbol_table[arg2_index].mode1 != T_BIN  &&
			  symbol_table[arg2_index].mode1 != T_LOGICAL  &&
			  symbol_table[arg2_index].mode1 != T_CHAR &&
			  symbol_table[arg2_index].mode1 != T_CARD &&
			  symbol_table[arg2_index].mode1 != T_INT)
		   {print_msg(EXPCON_ERR, token_buf);
	    printf("In more_log_not_exp 2\n");
		    *result_index = FAIL;
	       *result_offset = SUCCESS;}
	    else
		   {op_mode1 = get_op_mode1 (&arg1_index, &arg1_offset,
													 &arg2_index,&arg2_offset);
		    op_mode2 = get_op_mode2 (arg1_index, arg2_index);
		    result_mode2 = get_result_mode2 (op_mode2);
		    *result_index = get_symtab_temp(T_LOGICAL, result_mode2, 1);
	       *result_offset = SUCCESS;
		    if (*result_index != FAIL)
		      {if (op_type == T_LT)
			      print_opcode("lt_", T_LT, op_mode1, op_mode2);
			 else if (op_type == T_GT)
			   print_opcode("gt_", T_GT, op_mode1, op_mode2);
			    else if (op_type == T_LE)
			      print_opcode("le_", T_LE, op_mode1, op_mode2);
			    else if (op_type == T_GE)
			      print_opcode("ge_", T_GE, op_mode1, op_mode2);
			 else if (op_type == T_EQ)
			   print_opcode("eq_", T_EQ, op_mode1, op_mode2);
			    else
			      print_opcode("ne_", T_NE, op_mode1, op_mode2);
			    print_args(off_exp1(arg1_index,arg1_offset),
					    off_exp2(arg2_index,arg2_offset),
					    off_exp3(*result_index,*result_offset),
						 NULL, NULL);
		    }
		   }
	   }
	 else
		{*result_index = FAIL;
		 *result_offset = SUCCESS;}
	}
     }
   else
     {*result_index = arg1_index;
      *result_offset = arg1_offset;}

   if (debug == 2)
      printf ("More_log_not_exp() returning *result_index %d\n", *result_index);

   return(*result_index);
}
