/* 910415  P1_MAIN.C */

/* 020803 see p1_dimp and p1_dump */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"
#include "errno.h"

#ifdef _WIN32
#include "dos.h"
#endif


/* ============================================================================

   function name : main
   input                : argc          - number of command line arguments
   argv               - array of character pointers to arguments
   output            : ---
   return value  : exit_value   - 0, if no errors
						  - 1, otherwise

   global vars   : count                - character position in line
			     linenum    - line number in listing
			     inbuf0,inbuf1 - input buffers
			     current_buf        - current input buffer to use
			     fill_buf_flg       - whether buffer needs to be filled
			     buffer     - array of pointers to input buffers
			     ncpnt              - pointer to next character in input buffer
			     current_char  - last character processed
			     next_char  - next character to process
			     use_next_char_save - flag for using the saved next character
			     in_comment - whether scanner is in comment or not
			     previous_type      - previous token type
			     eofflg     - whether EOF has been detected or not
			     source_fp  - source file descriptor
			     list_fp    - listing file descriptor
			     object_fp  - object file descriptor
			     for_count  - number of for statements
			     loop_count         - number of loop statements
			     while_count        - number of while statements
			     if_count   - number of if statements
			     get_count  - number of get statements
			     any_count  - number of any statements
			     next_count - number of next statements
			     total_scalars - number of the scalar table entry
			     module_flag        - type of module: MAIN or SUBROUTINE

   description   : This function is the driver for a scanner, parser, and
			    intermediate code generator program. The program converts
			    a high-level associative programming language (ASPROL) into
			    intermediate code, for later conversion to a particular
			    SIMD assembly language. If no errors are detected, 0
			    is returned; 1 is returned otherwise. (These values were
			    chosen to be consistent with the standard UNIX return
			    values.)

============================================================================ */


main(argc,argv)
int argc;
char *argv[];
/*
_declspec(dllexport) int mainasc1(char *sfile);
int mainasc1(char *sfile)
*/
  {
//   int argc;
//   char *argv[4];

   int exit_value;          /* value returned by main program */
   int return_value;        /* value returned by parser/int. code generator */
   char *make_filename();/* routine to make a new file name */
   char *sourcefile;        /* pointer to source file name */
   char *listfile;          /* pointer to listing file name */
   char *objectfile;        /* pointer to object file name */
   unsigned int off;

   extern int errno;

//   argc = 3;
//   strcpy(argv[1], "-e"); 
//   strcpy(argv[2], sfile);

   exit_value = 0;
   debug = 0;
   upper = TRUE;
	shapedx = get_const("",T_STRING);

/* print version number */

   printf("\nASC Compiler Version 931024\n");

/*
 * Check for correct command usage:
 *      genicode -e filename (genicode -en filename)
 * or
 *      genicode -c filename
 */
  if (argc == 3)
    {/* check for options - this version  supports the -e option
		  only, the other options are include here for reference only*/
     off = 2;  wt = FALSE; wt_sub = FALSE;
     if      (strncmp (argv[1], "-a", 2) == 0){code_type=ASPRO;  exit_value=1;}
     else if (strncmp (argv[1], "-e", 2) == 0) code_type=EMULATE;
     else if (strncmp (argv[1], "-cm",3) == 0){code_type=CM;     exit_value=1;
															  off = 3;}
     else if (strncmp (argv[1], "-c", 2) == 0){code_type=C;      exit_value=1;}
	  else if (strncmp (argv[1], "-wt",3) == 0){code_type=EMULATE;exit_value=1;
															  off = 3; wt=WT;}
     else exit_value = 1;
     sourcefile = argv[2];
     if(strlen(argv[1])>off)
       {debug = atoi (argv[1]+off);
	if (debug <= 0 || debug > 10)
	  {printf("You entered %s, valid debug flag values are:\n",argv[1]+2);
	   printf("\t2 : trace ");}
       }

    if(exit_value == 1)
      printf("Usage:\n  asc1 -e filename\n");
   }
/*
 * Create names for object and listing files; open source file for
 * reading; open object and listing files for writing.
 */
   if (exit_value != 1)
     {if ((source_fp = fopen(sourcefile, "r")) == NULL)
	{printf ("ERROR on open source file %s\n",sourcefile);
	      exit_value = 1;}
	    else if (strcmp(indexp(sourcefile, '.'), ".asc") != 0)
	  {printf ("ERROR wrong file suffix: %s\n",sourcefile);
	      exit_value = 1;}
      else
	{include_flag = FALSE;
	 active_fp = source_fp;
	      listfile = make_filename (sourcefile, ".lst");
	      objectfile = make_filename (sourcefile, ".iob");
	      if ((list_fp = fopen(listfile, "w")) == NULL)
	   {printf ("ERROR can't open list file %s because %d\n",
						  listfile,errno);
		 exit_value = 1;}
	 else if ((object_fp = fopen(objectfile, "w")) == NULL)
	   {printf ("ERROR can't open object file %s because %d\n",
							objectfile,errno);
		 exit_value = 1;}
	}
    }

/*
 * Initialize global variables.
 */
   if (exit_value != 1)
     {count = 0;                /* character position in line */
      linenum  = 0;        /* line number in listing */
      inbuf0[0] = 0;            /* input buffer 0 */
      inbuf1[0] = 0;                    /* input buffer 1 */
      current_buf = 0;          /* current input buffer to use */
      fill_buf_flg = TRUE;      /* buffer needs to be filled */

      buffer[0].start = inbuf0;         /* input buffer pointer array */
      buffer[1].start = inbuf1;
      ncpnt = buffer[current_buf].start;/* pointer to next character */

      use_next_char_save = FALSE; /* initialize flag of using next_char_save */
      current_char = '\n';      /* initialize current character */
      next_char = getc(source_fp);      /* initialize next character */
      in_comment = 0;           /* currently not in a comment */
      previous_type = T_INITVAL;        /* initialize previous token type */
      eofflg = FALSE;           /* have not detected end of file */
      for_count = 0;                    /* number of for statements */
      loop_count = 0;           /* number of loop statements */
      while_count = 0;          /* number of while statements */
      if_count = 0;                     /* number of if statements */
      get_count = 0;                    /* number of get statements */
      any_count = 0;                    /* number of any statements */
      ifnany_count = 0;         /* number of ifnany statements */
      next_count = 0;           /* number of next statements */
      io_count = 0;                /* number of io statements */
      total_scalars = 0;        /* number of scalar table entry */
      module_flag = 0;          /* module type assign to none */
/*      table_parse = FALSE;  */
/*
 * Get the first token from the input and invoke the parser. If a legal
 * program unit was parsed correctly and there are more tokens, signal
 * an error.
 */
      token_type = get_token();
loop:
      if (debug == 2)
	printf ("In main().............................\n");
      if (token_type == T_EOF)
	{if (debug == 2)
		printf ("EOF detected\n");
	      exit_value = 0;}
      else
	{return_value = parse_start(exit_value);
	      if (return_value != FAIL)
		{if (token_type != T_EOF)
		    exit_value = 2;
		 else
		   exit_value = 0;
	   }
	      else
		exit_value = 1;
	}

      if(exit_value == 2) goto loop;

      fclose(source_fp);
      fprintf(list_fp, "\n");
      fclose(list_fp);
      fprintf(object_fp, "\n"); /* end with blank line for optimizer */
      fclose(object_fp);

      if ( debug == 2)
	printf ("Main returning exit_value %d\n", exit_value);

/*    if (debug >= 2) symtab_dump();*/
   }

   return(exit_value);

} /* end of main() */


/* ===========================================================================

   function name : indexp
   input                : str           - string pointer
		   chr          - character to check
   output            : ---
   return value  : str          - a modified string pointer

   global vars   : debug                - debugging statement flag

   description    : This routine move the pointer of the passed string pointer
		   to the character being check.

=========================================================================== */


char *indexp(str,chr)
char *str;
char chr;
{
   while (*str != 0 && *str != chr ) str++;
   return(str);

} /* end of indexp() */


/* ============================================================================

   function name : make_filename
   input                : base          - name of file containing base name of new file
			     suffix     - suffix of new file name
   output            : ---
   return value  : new_file_name - pointer to new file name

   global vars   : ---

   description    : This routine constructs a file name from a given file name
			     by removing the suffix, if any, and replacing it with a
			     different suffix. A pointer is returned to the new file name.

============================================================================ */

char *make_filename (base, suffix)
char *base, *suffix;
{
   char *loc;                      /* location of '.' in base name */
   char *new_file_name; /* pointer to new file name */
   int slen;

   slen = strlen (base) + strlen (suffix) + 1;
   new_file_name = (char *) calloc (slen,sizeof(char));
   strcpy (new_file_name, base);
   loc = (char *)indexp(new_file_name, '.');
   if (loc != NULL)
      strcpy (loc, suffix);
   else
      strcat (new_file_name, suffix);
   return(new_file_name);

} /* end of make_filename() */

