/* 910416 P1_MSGIN.C  */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_scanner.h"
#include "p1_parser.h"

#define LPAR '('

/* ===========================================================================

   function name: msgin_
   input        : ---
   output       : ---
   return value : ret_value             - SUCCESS, if no errors detected
					- FAIL, otherwise

   global vars  : token_type            - type of the current token
		  debug                 - flag for debugging statements
		  next_char             - next character from the source code
		  active_fp             - active file pointer
		  list_fp               - listing file pointer
		  linenum               - current line number

   description  : This routine handles input messages. It will read
		  the messages and echo it back to the listing file
		  with a special intermediate opcode. It will also generate
		  a line of code to the object file consisting of the inter-
		  mediate opcode and the message as an argument.
		  The production from the grammar is:

		     MSG <- msg {(handle)} char MESSAGE char;

		  LINE_CODE is the actual in-line message.

=========================================================================== */
char line_code[LINE_LENGTH]; /* a buffer that contains a line of*/
char del_char;               /*  assembly code (maximum 80 characters) */

int msgin_()
{
   int ret_value;               /* identifier for return value */
   int end_flag;
   int obuf_index,obuf_offset;


   if (debug == 2)
      printf("In msgin_()...........................\n");

   end_flag = FALSE;
   ret_value = SUCCESS;

  /* look for channel */
  while ((del_char = getnextchar()) == ' ');
  obuf_index = sysot_dx; obuf_offset = SUCCESS;
  if(del_char == LPAR)
    {token_type = get_token();
     if(element(&obuf_index,&obuf_offset) != FAIL)
	if(token_type == T_RPAR)
	   while ((del_char = getnextchar()) == ' ');
	else rwbuf_err();
     else rwbuf_err(); }
  putbackchar();
  get_msg(obuf_index,obuf_offset);
  print_opcode("print_nl",T_NL,T_NONE,T_NONE);
  print_args(off_exp1(obuf_index,obuf_offset),
							  NULL,NULL,NULL,NULL);
   if (debug == 2)
      printf("msgin_() returning ret_value = %d\n", ret_value);

   return(ret_value);

} /* end of msgin_() */



get_msg(obuf_index,obuf_offset)
int obuf_index,obuf_offset;
{
	/* local variables */
	int finished;
	int sc_dx;
	int sc_of;    /* dx and of for scalars */


	/* initialize local variables */
	finished = SUCCESS;
	sc_dx = 0;
	sc_of = 0;


	token_type = get_token();
	while( finished != FAIL)
	{
		if( token_type == T_ID		||
			token_type == T_MAXDEX	||
			token_type == T_MINDEX	||
			token_type == T_MAXVAL	||
			token_type == T_MINVAL	||
			token_type == T_NTHVAL	||
			token_type == T_NTHDEX	||
			token_type == T_ADDRESS	||
			token_type == T_FSTCD	||
			token_type == T_NXTCD	||
			token_type == T_PREVCD	||
			token_type == T_TRNCD	||
			token_type == T_TRNACD	||
			token_type == T_PREFIX	||
			token_type == T_FSTCD8	||
			token_type == T_NXTCD8	||
			token_type == T_PREVCD8	||
			token_type == T_TRNCD8	||
			token_type == T_TRNACD8	||
			token_type == T_PREFIX8	||
			token_type == T_COUNT	||
			token_type == T_SUM		||
			token_type == T_SIBDEX )
			if(( finished = sum_dif_exp(&sc_dx,&sc_of)) != FAIL)
			{
				if(symbol_table[sc_dx].mode2 == T_PARALLEL)
				{
					print_opcode("print_nl",T_NL,T_NONE,T_NONE);
					print_args(off_exp1(obuf_index,obuf_offset),NULL,NULL,NULL,NULL);
				}
				print_opcode("dump_", T_DUMP, symbol_table[sc_dx].mode1, symbol_table[sc_dx].mode2);
				print_args(off_exp1(obuf_index,obuf_offset), off_exp2(sc_dx,sc_of), off_exp3(symbol_table[sc_dx].iospec,SUCCESS),NULL,NULL);
				if(symbol_table[sc_dx].mode2 == T_PARALLEL)
				{
					print_opcode("print_nl",T_NL,T_NONE,T_NONE);
					print_args(off_exp1(obuf_index,obuf_offset),NULL,NULL,NULL,NULL);
				}
			}
			else 
				printf("%d ERROR - expression expected in msg\n");
		else if (token_type == T_STRING)
		{
			print_opcode("msg_ ", T_MSG, NULL, NULL);
			print_args(off_exp1(obuf_index,obuf_offset),token_buf, NULL,NULL, NULL);
			token_type = get_token();
		}
		else if (token_type == T_COMMA)
		{
			token_type = get_token();
			print_opcode("print_del",T_PRINT_DEL,T_NONE,T_NONE);
			print_args(off_exp1(obuf_index,obuf_offset),NULL,NULL,NULL,NULL);
		}
		else if (token_type = T_EOS)
		{
			token_type = get_token();
			finished = FAIL;
		}
		else
		{
			printf(" %d ERROR - unexpected token_type %d %s in msg\n", token_type,token_buf);
			token_type = get_token();
		}
	}

	return 0;
}

