/* 910415 P1_NEXT.C */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"

/* ===========================================================================

   function name : next_
   input	        : ---
   output	     : ---
   return value  : return_value	- SUCCESS, if no errors detected
		  		                     - FAIL, otherwise

   global vars   : symbol_table	- symbol table data structure
		             token_type	- type of current token
		             token_buf	- character string for current token
		             debug		- flag for debugging statements
		             next_count	- counter for making 'next' statement labels

   description	  : This routine is part of a recursive descent parser and
		             handles the following production from the grammar:

                	NEXT_ <-- index in MORE_NEXT BODY endnext index ;

		  The index variable must be an identifier that has been
		  declared as an 'index parallel' identifier. The keyword
		  'in' is required. The variable following the keyword 'in'
		  must be an identifier that has been declared as a 'logical
		  parallel' identifier. The body can be empty. The keyword
		  'endnext' is required. The index variable must be repeated
		  at the end of the loop. The logical parallel variable is
		  first stored in the index variable and then modified to
		  point to the next responder. The index variable cannot
		  be modified in the 'next' statement, but can be used to
		  index another parallel variable. The statements in the
		  body are executed only once. The next time the logical
		  parallel variable is used, it will point to the next
		  responder.

		  WARNING - This program has been modified to use subscript()
		  and MAY? accept "illegal" more-next subscripts.
=========================================================================== */

int next_()
{
   int invar_index,
       invar_offset,
       log_index,
       log_offset,
       lpar_flag,
       return_value,
       arg1_index,
       arg1_offset;

   char *start_label,	/* label to print at start of 'next' */
        *else_label,		/* label to print at elsenany of 'next' */
        *end_label;		/* label to print at end of 'next' */

   if (debug == 2)
      printf ("In next().........................................\n");

   return_value = SUCCESS;

   start_label = make_label("NE$S", ++next_count);
   else_label = make_label("NE$L", next_count);
   end_label = make_label("NE$E", next_count);

   print_opcode ("setscope", T_SETSCOPE, T_NONE, T_NONE);
   print_args (start_label, NULL, NULL, NULL, NULL);

   get_index(&invar_index,&invar_offset); /* "each" replaced by ITS */
   if (invar_index == FAIL)
		return_value = FAIL;
   else if(strcmp(symbol_table[invar_index].tokpts,"ITS") == 0);
	else if (token_type == T_IN) token_type = get_token();
	else
     {print_msg (IN_ERR, token_buf);
      return_value = FAIL;}

   if (token_type == T_LPAR)
     {lpar_flag = TRUE;
      token_type = get_token();}
   else
      lpar_flag = FALSE;

   if (token_type != T_ID)
     {print_msg (ID_ERR, token_buf);
      return_value = FAIL;}
   else if ((log_index = symtab_fetch(token_size, token_buf)) == FAIL)
     {print_msg (UNDEF_ERR, token_buf);
      return_value = FAIL;}
   else if (symbol_table[log_index].mode2 != T_PARALLEL ||
            symbol_table[log_index].mode1 != T_LOGICAL)
     {print_msg (LOGPAR_ERR, token_buf);
      return_value = FAIL;}
   else
     {token_type = get_token();
      if(subscript(log_index,&arg1_index,&arg1_offset) == FAIL)
	       return_value = FAIL;
      log_index=arg1_index;
      log_offset=arg1_offset;}

   if (return_value != FAIL)
     {print_opcode ("next", T_NEXT, T_NONE, T_NONE);
      print_args (off_exp1(log_index,log_offset),
		            off_exp2(invar_index,invar_offset),
		            end_label, NULL, NULL);
      do_it(invar_index,invar_offset);
      if (lpar_flag == TRUE)
        {if (token_type != T_RPAR)
           {return_value = FAIL;
            print_msg (RPAR_ERR, token_buf);}
         else
           token_type = get_token();
        }

      if ((return_value = body()) != FAIL)
        {if (token_type == T_ELSENANY)
           {print_opcode ("elsenany", T_ELSENANY, T_NONE, T_PARALLEL);
            print_args(else_label,end_label,NULL, NULL, NULL);
            token_type = get_token();
				return_value = body();}
			else
           {print_opcode("label_", T_LABEL, T_NONE, T_NONE);
			   print_args(else_label, NULL, NULL, NULL, NULL);}
         if (token_type != T_ENDNEXT)
           {return_value = FAIL;
            print_msg(ENDNEXT_ERR, token_buf);}
         else
           {token_type = get_token();
				if(strcmp(token_buf, "EACH") == 0)
				  {token_type = T_ID;
				   token_size =3;
					strcpy(token_buf,"ITS");}
            if (strcmp(token_buf, symbol_table[invar_index].tokpts) != 0)
     		     {print_msg(INXUNM_ERR, token_buf);
               return_value = FAIL;}
            else
              {print_opcode("endnext", T_ENDNEXT, T_NONE, T_NONE);
			      print_args(end_label, NULL, NULL, NULL, NULL);
               if ((token_type = get_token()) == T_LSQU)
		           {while ((token_type = get_token()) != T_RSQU);
		            token_type = get_token();}
		         if (token_type != T_EOS)
                 {return_value = FAIL;
                  print_msg(T_EOS, token_buf);}
               else
                 token_type = get_token();
              }
            }
         }
      }

   if (debug == 2)
      printf ("next() returning return_value : %d\n", return_value);

   return(return_value);

} /* end of next() */

