/* 940321 P1_NP.C */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"

/* ===========================================================================

   function name : np_
   input	        : ---
   output	     : ---
   return value  : return_value	- SUCCESS, if no errors
				                     - FAIL, otherwise

   global vars   : symbol_table	- array containing symbol table
		             token_type	- type of current token
		             token_buf	- character string for current token
		             debug		- flag for debugging statements

   description	  : This routine is part of a recursive descent parser and
		    handles the following noun phrase production from the
                    grammar:

                        THE_ <-- adj , adj ... noun (pp) ;

                    The comma between adjectives is required.  No comma
                    implies the end of the sequence, so the next item
                    is the noun.  The noun may be followed by an optional
                    prepositional phrase.

                    A NP is equivalent to a logical expression and so returns
                    a responder flag vector.
=========================================================================== */

int np(noun_dx)
int noun_dx;
{
   int adj_dx;    		/* symbol table index of expression result */
   int string_dx;

   if (debug == 2)
      printf ("In adj_()........................................\n");

   adj_dx = get_symtab_temp(T_LOGICAL,T_PARALLEL,W_LOGICAL);
   print_opcode("mvpa_",T_MVPA,T_LOGICAL,T_PARALLEL);
	   print_args (symbol_table[LP_TRUE].tokpts,
                       off_exp2(adj_dx,SUCCESS), NULL, NULL, NULL);
loop1:
   string_dx = get_const(token_buf,T_STRING);
   print_opcode("adj_",T_ADJ,T_NONE,T_NONE);
   print_args(off_exp1(string_dx,SUCCESS),
	      off_exp2(adj_dx,SUCCESS),
	      NULL, NULL, NULL);
   token_type = get_token();
   if(token_type == T_COMMA)
     token_type = get_token();
   if(symtab_fetch(token_size,token_buf) == FAIL) goto loop1;
   string_dx = get_const(token_buf,T_STRING);
   print_opcode("noun_",T_NOUN,T_NONE,T_NONE);
   print_args(off_exp1(string_dx,SUCCESS),
	      off_exp2(adj_dx,SUCCESS),
	      NULL, NULL, NULL);
   token_type = get_token();
   adj_dx = pp(adj_dx);
/*   save_em(adj_dx,SUCCESS); */

   return(adj_dx);
}


/* ******************************************
   pp - prepositional phrase

   Check for a preposition, if present, call np
                            otherwise return;
   ****************************************** */

pp(resp_dx)
int resp_dx;
{
/*  printf("In pp - prepositional phrase, line %d, NOT DEBUGGED\n",linenum);*/
  if(token_type == T_PREP)
     {token_type = get_token();
      return np(resp_dx);}
   return resp_dx;
}
