/* 910415 P1_PRINTICODE.C - P1_P_CODE.C */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_scanner.h"

/* ===========================================================================

   function name : print_opcode
   input                : op_string     - character string for op code
			  op_type    - integer representation of op code
			  op_mode1   - mode of operator: int, real or logical
			  op_mode2   - mode of operator: scalar or parallel
   output            : ---
   return value  : ---

   global vars   : list_fp       - listing file pointer
			     object_fp  - object file pointer
			     current_char       - last character printed from source code

   description    : This routine prints the operation code to both the listing
			     file and the object file. Output to the listing file is
			     a character string in character format. Output to the
			     object file is an integer printed in character format.

=========================================================================== */

void print_opcode(op_string, op_type, op_mode1, op_mode2)
char *op_string;        /* character string for op code */
int op_type;            /* integer for op code */
int op_mode1;           /* INT or REAL or LOGICAL*/
int op_mode2;           /* SERIAL or PARALLEL */
{
   int op_code;         /* integer for op code + modes (print to object file)*/

   if (debug == 2)
     {printf ("In print_opcode().......................\n");
      printf("op_type=%x, %s, mode1=%x, mode2=%x\n", op_type, op_string, op_mode1,
	op_mode2);}

   op_code = 0;
   if (current_char != '\n')
      fprintf(list_fp, "\n");
   fprintf(list_fp, "\t%s", op_string);
   op_code += op_type;

   switch (op_mode1)
     {case T_INT                : fprintf(list_fp, "fx");
					   break;
      case T_REAL               : fprintf(list_fp, "fl");
					   break;
      case T_OCT                : fprintf(list_fp, "oc");
					   break;
      case T_BIN                : fprintf(list_fp, "bn");
					   break;
      case T_HEX                : fprintf(list_fp, "hx");
					   break;
      case T_CHAR               : fprintf(list_fp, "ch");
					   break;
      case T_LOGICAL    : fprintf(list_fp, "lg");
					   break;
      case T_CARD               : fprintf(list_fp, "cd");
					   break;
      case T_LONGINT    : fprintf(list_fp, "li");
					   break;
   }
   op_code += op_mode1 * SIZE_MODE2;

   switch (op_mode2)
     {case T_SCALAR              : fprintf(list_fp, "sc");
					       break;
      case T_PARALLEL    : fprintf(list_fp, "pa");
					       break;
      case T_COMMON_LEFT : fprintf(list_fp, "cl");
					       break;
		case T_COMMON_RIGHT: fprintf(list_fp, "cr");
					       break;
      case T_CNST              : fprintf(list_fp, "cn");
					       break;
      case T_MASKEDMOVE  : fprintf(list_fp, "mm");
					       break;
   }
   op_code += op_mode2;

   fprintf(list_fp, "    \t");
   fprintf(object_fp, "%15s %x ", op_string, op_code);
   if (debug == 2)
      printf ("Print_opcode() printed opcode = %x\n", op_code);

}


/* ===========================================================================

   function name : print_args
   input                : arg1_string   - first argument string
			     arg2_string        - second argument string
			     arg3_string        - third argument string
			     arg4_string        - fourth argument string
			     arg5_string        - fifth argument string
   output            : ---
   return value  : ---

   global vars   : list_fp      - listing file pointer
			     object_fp  - object file pointer

   description    : This routine prints the operands of an intermediate
			     code instruction to both the listing file and the object
			     file. The operands are printed to the listing file as
			     character strings separated by a space and a comma.
			     The operands are printed to the object file as character
			     strings separated only by a space. A carriage return is
			     printed only to the listing file to start the next
			     instruction on a new line.

=========================================================================== */

void print_args (arg1_string,arg2_string,arg3_string,arg4_string,arg5_string)
char *arg1_string, *arg2_string, *arg3_string, *arg4_string, *arg5_string;
{
   if (debug == 2)
      printf ("In print_args()...........................\n");

   if (arg1_string != NULL)
     {fprintf(list_fp, "%s", arg1_string);
      fprintf(object_fp, "%s ", arg1_string);}

   if (arg2_string != NULL)
     {fprintf(list_fp, ", %s", arg2_string);
      fprintf(object_fp, "%s ", arg2_string);}

   if (arg3_string != NULL)
     {fprintf(list_fp, ", %s", arg3_string);
      fprintf(object_fp, "%s ", arg3_string);}

   if (arg4_string != NULL)
     {fprintf(list_fp, ", %s", arg4_string);
      fprintf(object_fp, "%s ", arg4_string);}

   if (arg5_string != NULL)
     {fprintf(list_fp, ", %s", arg5_string);
      fprintf(object_fp, "%s ", arg5_string);}

   fprintf(list_fp, "\n");
   fprintf(object_fp, "\n");

   if (debug == 2)
      printf ("Print_args() returning.\n");

}

void print_args6 (arg1_string,arg2_string,arg3_string,arg4_string,
		  arg5_string,arg6_string)
char *arg1_string, *arg2_string, *arg3_string, *arg4_string,
     *arg5_string, *arg6_string;
{
   if (debug == 2)
      printf ("In print_args()...........................\n");

   if (arg1_string != NULL)
     {fprintf(list_fp, "%s", arg1_string);
      fprintf(object_fp, "%s ", arg1_string);}

   if (arg2_string != NULL)
     {fprintf(list_fp, ", %s", arg2_string);
      fprintf(object_fp, "%s ", arg2_string);}

   if (arg3_string != NULL)
     {fprintf(list_fp, ", %s", arg3_string);
      fprintf(object_fp, "%s ", arg3_string);}

   if (arg4_string != NULL)
     {fprintf(list_fp, ", %s", arg4_string);
      fprintf(object_fp, "%s ", arg4_string);}

   if (arg5_string != NULL)
     {fprintf(list_fp, ", %s", arg5_string);
      fprintf(object_fp, "%s ", arg5_string);}

   if (arg6_string != NULL)
     {fprintf(list_fp, ", %s", arg6_string);
      fprintf(object_fp, "%s ", arg6_string);}

   fprintf(list_fp, "\n");
   fprintf(object_fp, "\n");

   if (debug == 2)
      printf ("Print_args() returning.\n");

}
