/* 910415 P1_PARAMETERS.C */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"

/* ===========================================================================

   function name : parameters
   input                : ---
   output            : ---
   return value  :              - SUCCESS, if no errors
					       - FAIL, otherwise

   global vars   : token_type   - type of current token
			     debug              - flag for debugging statements
			     symbol_table       - array for symbol table
			     token_buf  - character string for current token

   description    : This routine is part of a recursive descent parser and
			     handles the following production from the grammar:

			PARAMETERS <-- identifier MORE_IDENTIFIER = LOG_EXP ;

		  The left side of an parametersment statement must contain
		  an identifier found in the symbol table. The right side
		  can be any expression. The following restrictions apply:
		  1) a parallel result cannot be parametersed to a scalar
		  identifier, 2) reals and integers can only be parametersed
		  to reals and integers (conversions are done as needed),
		  and 3) logicals and indexes can only be parametersed to
		  logicals.

=========================================================================== */

/* p1_parameters is modeled after p1_assign */

int parameters()
{
   int rhs_index;               /* symbol table index of RHS identifier */
   int rhs_offset;   /* offset of rhs */
   int lhs_index;               /* symbol table index of LHS identifier */
   int lhs_offset;      /* symbol table index for the offset */
   int scalar_dx;
   int temp_dx;      /* index for a temp field */
   int type,flag,lhsdx_dx,lhsdx_os;  /* items for subsubscript() */
   int paratype;     /* T_TO or T_FROM */

if (debug == 2)
   printf ("In parameters().....................................\n");

/* process left hand side */

   if ((lhs_index = symtab_fetch(token_size, token_buf)) == FAIL)
     {printf("ERROR - undefined variable %s in Call - line %d\n",
				  token_buf,linenum);
      return(FAIL);}
   if (symbol_table[lhs_index].toktype != T_ID &&
       symbol_table[lhs_index].toktype != T_INDEX)
     {print_msg(ASGNID_ERR, token_buf);
      printf(" toktype = %d\n",symbol_table[lhs_index].toktype);
      return(FAIL);};

   /* process subscript if any */
   token_type = get_token();
   cube_flag = FALSE;
   if(subsubscript(lhs_index,&type,&flag,&lhsdx_dx,&lhsdx_os,
						 &lhs_offset,&temp_dx) == FAIL)  return(FAIL);
   if(cube_flag == TRUE){print_msg(CUBE_ERR,token_buf); return(FAIL);}
   if(temp_dx != SUCCESS)
     if((scalar_dx =
			get_symtab_temp(symbol_table[lhs_offset].mode1,T_SCALAR,T_NONE)) == FAIL)
       {print_msg(TEMP_ERR, "in param 1"); return(FAIL); }
     else  /* put indexed offset into a scalar */
       {print_opcode ("mvsc_",T_MVSC,symbol_table[lhs_offset].mode1,T_SCALAR);
	     print_args (off_exp1(lhs_offset,temp_dx),
			 off_exp2(scalar_dx,SUCCESS),NULL,NULL,NULL);
	lhs_offset = scalar_dx;}

/* process parameters keywords */
   paratype = token_type;

	if (paratype != T_TO && paratype != T_FROM)
     printf("%4d: ERROR - illegal parameter expression type = %x\n",linenum,paratype);
   token_type = get_token();

/* process right hand side */
   log_exp(&rhs_index,&rhs_offset);
   if(rhs_index == FAIL) return(FAIL);  /* lhs is unknown, coerec to rhs */
/* symbol_table[lhs_index].mode1 = symbol_table[rhs_index].mode1;
   symbol_table[lhs_index].mode2 = symbol_table[rhs_index].mode2;
   symbol_table[lhs_index].width = symbol_table[rhs_index].width;
   symbol_table[lhs_index].expt = symbol_table[rhs_index].expt;
   symbol_table[lhs_index].signif = symbol_table[rhs_index].signif;
   symbol_table[lhs_index].num_dim = symbol_table[rhs_index].num_dim;*/

   if (paratype == T_TO)
     genmove(lhs_index,lhs_offset,lhsdx_dx,lhsdx_os,
				 rhs_index,rhs_offset,flag,type,paratype);
   else /* if T_FROM save on list_stack */
     {if(rhs_offset != SUCCESS)
		  {print_opcode("stack_",T_STACK,T_NONE,symbol_table[rhs_offset].mode2);
	 print_args(symbol_table[rhs_offset].tokpts,NULL,NULL,NULL,NULL);}
		if(save_cnt+8 < LIST_STACK_SIZE)
	{list_stack[save_cnt++] = rhs_index; /* reverse rhs and lhs */
	 list_stack[save_cnt++] = rhs_offset; /* because is T_FROM */
	 list_stack[save_cnt++] = SUCCESS;
	 list_stack[save_cnt++] = SUCCESS;
	 list_stack[save_cnt++] = lhs_index;
	 list_stack[save_cnt++] = lhs_offset;
	 list_stack[save_cnt++] = flag;
	 list_stack[save_cnt++] = type;}
     };

 if (debug == 2)
      printf ("Parameters() returning\n");

return(SUCCESS);

}



/* generate move quadprules */
genmove(lhs_index,lhs_offset,lhsdx_dx,lhsdx_os,
	rhs_index,rhs_offset,flag,type,paratype)
int lhs_index,lhs_offset,lhsdx_dx,lhsdx_os,
    rhs_index,rhs_offset,flag,type,paratype;
{
   char msgbuf[16];     /* for debug print  buffers */
   int temp_dx;
   char c_size[10];            /* char buf for size1 */
   int c_size_dx;

/* for paratype == T_FROM only
   if lhs_offset is non  null, restore from stack after call
	use lhs_offset here, because rhs-lhs have been reversed above */

	if(paratype == T_FROM && lhs_offset != SUCCESS)
     {print_opcode("pop_",T_POP, T_NONE, symbol_table[lhs_offset].mode2);
	   print_args(symbol_table[lhs_offset].tokpts,NULL,NULL,NULL,NULL);}
   switch(type) /* lhs type2 */
     {case T_PARALLEL: /* T_MVSC - asns_idxpa - MREG masked move */
		strcpy(msgbuf," parallel ");
		if(!flag){print_msg(ASGNCON_ERR,msgbuf);return(FAIL);}
		if (symbol_table[rhs_index].mode2 == T_SCALAR ||
			      symbol_table[rhs_index].mode2 == T_CNST )
	     {/* pa[$,offset]=sc[os]; */
	      pc_itoa(symbol_table[rhs_index].width,c_size);
	      c_size_dx = get_const(c_size,T_DEC_CONST);
			     print_opcode("mvsc_",T_MVSC,
					   symbol_table[rhs_index].mode1,T_PARALLEL);
	      print_args (off_exp1(rhs_index,rhs_offset),
			  off_exp2(lhs_index,lhs_offset),
			  off_exp3(c_size_dx,SUCCESS),NULL,NULL); }
	   else /* rhs_index.mode2 == T_PARALLEL */
	     {/* T_MVPA - asnp_idxpa  */
			     /* pa[$,offset]=pa[os]; */
	      print_opcode("mvpa_",T_MVPA,
					  symbol_table[rhs_index].mode1,T_PARALLEL);
	      print_args (off_exp1(rhs_index,rhs_offset),
			  off_exp2(lhs_index,lhs_offset),NULL,NULL,NULL);};
	    break;
       case T_MASKEDMOVE: /* mvsc - asns_idxpa - explicitly masked */
					 /* pa[xx,offset] = sc[os];  */
			   strcpy(msgbuf," maskedmove");
		 if(!flag){print_msg(ASGNCON_ERR,msgbuf);return(FAIL);}
			   /* move "xx" to a temp so that it can be resvfst-ed  */
		      temp_dx = get_symtab_temp(T_LOGICAL,T_PARALLEL,W_LOGICAL);
	    print_opcode("resf_",T_RESFST, T_NONE, T_PARALLEL);
	    print_args (off_exp1(lhsdx_dx,lhsdx_os),
					    off_exp2(temp_dx,SUCCESS),NULL, NULL, NULL);
	    print_opcode("mvsc_",T_MVSC,symbol_table[rhs_index].mode1,T_MASKEDMOVE);
	    print_args(symbol_table[temp_dx].tokpts,
					   off_exp2(rhs_index,rhs_offset),
		       off_exp3(lhs_index,lhs_offset),NULL,NULL);
	    break;
     case T_SCALAR:  /* asns_idxsc */
     case T_CNST:    /* sc[offset] = sc;  */
		      print_opcode("mvsc_",T_MVSC,symbol_table[rhs_index].mode1, T_SCALAR);
	    print_args (off_exp1(rhs_index,rhs_offset),
			off_exp2(lhs_index,lhs_offset),NULL, NULL, NULL);
	    break;
     default: printf("In genmove, illegal subsubscript_type %d detected\n",type);
		 return (FAIL);
     };
return(SUCCESS);
 }
