/* 910415  P1_PARSER.C */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"

/* ===========================================================================

   function name : parse_start
   input	        : ---
   output	     : ---
   return value  : ret_value	- SUCCESS, if no errors detected
		  		                  - FAIL, otherwise

   global vars   : token_type		- type of token
		             token_buf		- token buffer
		             debug			- debugging statement flag
		             symbol_table		- symbol table data structure
		             module_flag		- type of module : MAIN, SUBROUTINE

   description   : This function is the main routine of a parser/intermediate
		             code generator. It handles the following production from
		             the start symbol 'parse_start' :

		       PARSE_START <--  main id
										DEFINE DEFLOG DECLARE ASSOCIATE
										BODY
					               end ; |

				                  subroutine id
										DEFINE DEFLOG DECLARE ASSOCIATE
										BODY
					               end ; |

					               DEFINE DECLARE DEFLOG ASSOCIATE
										BODY

        No code is generated at this time for 'procedure' calls,
		  'parameters', or 'include' statements. A 'main' program
		  declaration causes code to be generated to define the
		  run-time stack, to identify the program, and to clear the
		  parallel memory. The function returns FAIL if a syntax error
		  is detected and returns SUCCESS, otherwise.

=========================================================================== */
char arg_str_ff[100];

int parse_start(ex_vl)
int ex_vl;
{
   int ret_value;	    /* value returned by function */
   int prog_index;	 /* symbol table index of program identifier */
   int sub_index;	    /* symbol table index of subroutine identifier */
   int dim_size[MAX_NUM_DIM];
	int i;
	int dummy;

   ret_value = SUCCESS;
   sub_index = FAIL;
   if (debug == 2)
      printf("In parse_start(), exit value = %d, token=%s..\n",ex_vl,token_buf);

   if(ex_vl != 2)
     {print_opcode ("include",T_INCLUDE,T_NONE,T_NONE);
      print_args (NULL, NULL, NULL, NULL, NULL);}
   if (token_type == T_MAIN)
     {module_flag = T_MAIN;
      token_type = get_token();
      if (token_type == T_ID)
	{dummy = 1;
	 prog_index = symtab_insert(T_MAIN,T_NONE,T_NONE,token_size,token_buf,
						       &dummy, 0, 0, 0 ,0);
	 symbol_table[prog_index].temp = T_MAIN;
	 printf ("Compiling %s\n",symbol_table[prog_index].tokpts);
	 printf ("-- Pass 1 (950101) --\n");/* print "compiling... message */
/*	 token_type = get_token();  to32 - this line must be removed */
	      if (ret_value != FAIL)
		{init_shape();
		 print_opcode ("main", T_MAIN, T_NONE, T_NONE);
		 print_args (symbol_table[prog_index].tokpts, NULL,NULL,NULL,NULL);
		 print_opcode ("ident", T_IDENT, T_NONE, T_NONE);
		 print_args (NULL, NULL, NULL, NULL, NULL);
		 init_symbol_table(ex_vl);
		 for(i=0; i<MAX_NUM_DIM; i++) dim_size[i]=0;
				sysin_dx = symtab_insert(T_ID,T_INT,T_SCALAR,6,"SYSIN",dim_size,32,0,32,1);
				sysot_dx = symtab_insert(T_ID,T_INT,T_SCALAR,6,"SYSOT",dim_size,32,0,32,1);
		 while(ret_value == SUCCESS && token_type != T_EOF)
		   {switch(token_type)
			   {case T_DEFINE: ret_value = define(ex_vl); break;
			    case T_DEFLOG: ret_value = deflog(); break;
			    case T_DEFVAR: ret_value = defvar(); break;
		  case T_LONGINT:
			    case T_INT:
			    case T_BIN:
			    case T_OCT:
			    case T_HEX:
			    case T_REAL:
			    case T_CARD:
			    case T_CHAR:
			    case T_INDEX:
			    case T_LOGICAL:
			    case T_DECLARE: ret_value = declare(); break;
			    case T_ASSEMBLE: ret_value = assemble_(); break;
			    case T_EOS:
			    case T_END:
				  printf("%d ERROR - illegal main parse. Token_type = %d.\n",
							  linenum,token_type);
				  exit(0);
				  break;
			    case T_ASSOCIATE:
							  ret_value = associate();  break;
			    default:
		       print_opcode ("clr_array", T_CLRARRAY, T_NONE, T_NONE);
		       print_args (NULL, NULL, NULL, NULL, NULL);
		       /* talloc must be before init_stack */
		       print_opcode("alloc_",T_ALLOCATE,T_NONE,T_THE);
		       print_args(symbol_table[symtab_fetch(3,"THE")].tokpts,
							   NULL,NULL,NULL,NULL);
		       print_opcode ("init_stack", T_INITSTACK, T_NONE, T_NONE);
		       print_args (NULL,NULL,NULL,NULL, NULL);
		       if ((ret_value = body()) != FAIL)
		                   {if (token_type == T_END)
		                      {token_type = get_token();
			                    if (token_type == T_EOS)
                               {print_opcode("label_",T_LABEL,T_NONE,T_NONE);
                                if(wt == WT)  print_args("_EXIT_", NULL, NULL, NULL, NULL);
 			                       else print_args("$EXIT$", NULL, NULL, NULL, NULL);
                                print_opcode("stop_",T_STOP,T_NONE,T_NONE);
 			                       print_args(NULL, NULL, NULL, NULL, NULL);
                                dump_scalar();
                                print_opcode("endmain",T_ENDMAIN,T_NONE,T_NONE);
                                print_args(symbol_table[prog_index].tokpts,
				                               NULL, NULL, NULL, NULL);
			                       token_type = get_token();}
			                    else
			                      {print_msg(EOS_ERR, token_buf);
			                       ret_value = FAIL;}
		                       }
   		                 else
		                      {print_msg(END_ERR, token_buf);
                             printf("In p1_parse at 1\n");
                             ret_value = FAIL;}
								  if(WT == wt && token_type == T_SUBROUTINE) ret_value = 2;
	                        }
                        break;
		           }
              } /* end of while */
           }
        }
      else
        {print_msg(ID_ERR,token_buf);
         printf("In p1_parse, main identifier missing, linenum=%d\n",linenum);
         ret_value = FAIL; }
     }
   else if (token_type == T_SUBROUTINE)
     {module_flag = T_SUBROUTINE;
      token_type = get_token();
      if (token_type == T_ID)
        {dummy = 1;
			sub_index = symtab_insert(T_SUBROUTINE, T_NONE, T_NONE, token_size,
				                       token_buf, &dummy, 0, 0, 0, 0);
         printf ("Compiling %s\n",symbol_table[sub_index].tokpts);
         printf ("-- Pass 1 (931024) --\n");/* print "compiling... message */
         symbol_table[sub_index].temp = T_SUBROUTINE;
         par_fun();
         print_opcode ("subroutine", T_SUBROUTINE, T_NONE, T_NONE);
         if(arg_str_ff != NULL)
	         print_args(symbol_table[sub_index].tokpts, arg_str_ff, NULL, NULL, NULL);
         else
	         print_args(symbol_table[sub_index].tokpts, NULL, NULL, NULL, NULL);
         if (ret_value != FAIL)
           {print_opcode ("ident", T_IDENT, T_NONE, T_NONE);
	         print_args (NULL, NULL, NULL, NULL, NULL);
            if(ex_vl == 0) wt_sub = WT; else wt_sub = FALSE;
  	         init_symbol_table(ex_vl);
            if(ex_vl != 2)
              {for(i=0; i<MAX_NUM_DIM; i++) dim_size[i]=0;
				   sysin_dx = symtab_insert(T_ID,T_INT,T_SCALAR,6,"SYSIN",dim_size,32,0,32,1);
				   sysot_dx = symtab_insert(T_ID,T_INT,T_SCALAR,6,"SYSOT",dim_size,32,0,32,1);}
				while(ret_value == SUCCESS && token_type != T_EOF)
	           {switch(token_type)
		           {case T_DEFINE: ret_value = define(ex_vl); break;
		            case T_DEFLOG: ret_value = deflog(); break;
		            case T_DEFVAR: ret_value = defvar(); break;
		            case T_INT:
		            case T_BIN:
		            case T_OCT:
		            case T_HEX:
		            case T_REAL:
		            case T_CARD:
		            case T_CHAR:
		            case T_INDEX:
		            case T_LOGICAL:
		            case T_DECLARE: ret_value = declare(); break;
		            case T_ASSOCIATE: ret_value = associate(); break;
                  case T_EOS:
						case T_END:
									 printf("%d ERROR - illegal subroutine parse. token_type=%d\n",
											  linenum,token_type);
                            break;
		            default:
                      in_subody = TRUE;
                      if ((ret_value = body()) != FAIL)
		                  {if (token_type == T_END)
		                     {token_type = get_token();
			                   if (token_type == T_EOS)
                              {if(ret_value != TRUE &&
											 code_type != EMULATE)
                                 {print_opcode("return",T_RETURN,T_NONE,T_NONE);
 			                         print_args(NULL, NULL, NULL, NULL, NULL);}
                               print_opcode("endsub",T_ENDSUB,T_NONE,T_NONE);
                               print_args("0", symbol_table[sub_index].tokpts,
				                              NULL, NULL, NULL);
				                   token_type = get_token();}
			                   else
			                     {print_msg(EOS_ERR, token_buf);
			                      ret_value = FAIL;}
		                      }
   		                 else
		                      {print_msg(END_ERR, token_buf);
                             printf("In p1_parse at 2\n");
		                       ret_value = FAIL;}
								  if(WT == wt && token_type == T_SUBROUTINE) ret_value = 2;
	                      }
							 in_subody = FALSE;
                      break;
		               }
                 } /* end of while */
              }
        }
      else
        {print_msg(ID_ERR,token_buf);
         printf("In p1_parse subroutine\n");
         ret_value = FAIL;}
     }
   else
     {
     init_symbol_table(ex_vl);
	   while(ret_value == SUCCESS && token_type != T_EOF)
	     {switch(token_type)
		     {case T_DEFINE: ret_value = define(ex_vl); break;
		      case T_DEFLOG: ret_value = deflog(); break;
 		      case T_DEFVAR: ret_value = defvar(); break;
		      case T_INT:
		      case T_BIN:
		      case T_OCT:
		      case T_HEX:
		      case T_REAL:
		      case T_CARD:
		      case T_CHAR:
		      case T_INDEX:
		      case T_LOGICAL:
		      case T_DECLARE: ret_value = declare(); break;
		      case T_ASSOCIATE: ret_value = associate(); break;
		      default:
          	     print_opcode ("init_stack", T_INITSTACK, T_NONE, T_NONE);
	              print_args (NULL,NULL,NULL,NULL, NULL);
                 if ((ret_value = body()) != FAIL)
		             {if (token_type == T_END)
		                {token_type = get_token();
			              if (token_type == T_EOS)
                         {print_opcode("return",T_RETURN,T_NONE,T_NONE);
 			                 print_args(NULL, NULL, NULL, NULL, NULL);
                          dump_scalar();
                          print_opcode("endsub",T_ENDSUB,T_NONE,T_NONE);
                          print_args(symbol_table[sub_index].tokpts,
				                         NULL, NULL, NULL, NULL);
			                 token_type = get_token();}
							  else if(WT == wt) ret_value = 2;
			              else
			                {print_msg(EOS_ERR, token_buf);
			                 ret_value = FAIL;}
		                }
   		           else
		                {print_msg(END_ERR, token_buf);
									  printf("In p1_parse at 3\n");
		                 ret_value = FAIL;}
	                }
                break;
		         }
            } /* end of while */
         dump_scalar();
        }

   if (debug == 2)
      printf("Parse_start is returning %d\n", ret_value);
   return(ret_value);

} /* end of parse_start() */

par_fun()
{
  arg_str_ff[0] = (char)  0;
  if((token_type = get_token()) != T_LPAR)
	  return 0;
  else
	 {strcpy(arg_str_ff,"long ");
	  while ((token_type = get_token()) != T_RPAR && token_type != T_EOF)
		 strcat(arg_str_ff,token_buf);}
 token_type = get_token();
  return 0;
 }
