#ifndef P1_PARSER
#define P1_PARSER

/* ==========================================================================

   This file contains preprocessor definitions and global data structure
   declarations for use with 'genicode' intermediate code generator.

============================================================================ */

/*---index and grouping operators---*/
#define T_LSQU          1000
#define T_RSQU          1100
#define T_LPAR          1200
#define T_RPAR          1300

/*---identifier and constant token types---*/
#define T_HEX_CONST     2000
#define T_OCT_CONST     2100
#define T_BIN_CONST     2200
#define T_DEC_CONST     2300
#define T_ID               2400
#define T_PAR_SUBSCRIPT 2500
#define T_INDEX      2600
#define T_STRING     2700
#define T_SID        2800   /* system id, ie. IPTMP$ */
#define T_FLT_CONST     2900
#define T_ACE        3000
/*---miscellaneous token types---*/
#define T_EOS              3100
#define T_INITVAL       3100
#define T_UNKNOWN       3200
#define T_COMMA         3300
#define T_COLON         3400
#define T_SQUOTE     3500

/* DEFINITIONS FOR SYMBOL TABLE CONSTANTS */
/* These values are used in p1_init.c */
#define SYMTAB1_SIZE             732       /* symbol table size */
#define NON_TEMP_ID_END          562               /* room for 512 identifiers*/
#define INT_SCA_TEMP_START       563          /* 48 scalar temps */
#define INT_SCA_TEMP_END         610
#define REAL_SCA_TEMP_START      611            /* 31 real scalar temps */
#define REAL_SCA_TEMP_END        671
#define INT_PAR_TEMP_START       672            /* 16 int par temps */
#define INT_PAR_TEMP_END         687
#define REAL_PAR_TEMP_START      688            /* 5 real par temps */
#define REAL_PAR_TEMP_END        702
#define LOG_PAR_TEMP_START       703            /* 16 log par temps */
#define LOG_PAR_TEMP_END         719
#define G_OFFSET_START             0
#define LINT_PAR_TEMP_START      720           /* 5 long int par temps */
#define LINT_PAR_TEMP_END        724
#define SCALAR_STACK             725       /* the scalar stack entry */
#define LP_TRUE                  726       /* parallel logical with a value ones */
#define LP_FALSE                 727       /* parallel logical with a value zeros */
#define SC_IO_BUFFER             728       /* the io buffer in sc memory */
//#define IOSTACK not used in pass 1  729       /* the array io stack */
#define PARALLEL_STACK           730
/*                               731?       $ IS IN LAST POSITION IN TABLE */

#define BUSY            1
#define IDLE            0

#define IOSTACK_SIZE    96            /* width of IO stack */
#define STACK_SIZE    1024            /* size of scalar stack */

/* DEFINITIONS FOR PARSER CONSTANTS */

#define STACKSIZE             "32"      /* size of scope stack */
#define MAX_SCALARS     400   /* maximum number of scalar variables */
#define MAX_NUM_DIM       3     /* maximum number of dimensions */
#define W_REAL                 32       /* bit field width for real */
#define W_INT                  16       /* bit field width for integer */
#define W_LOGICAL         1     /* bit field width for logical */
#define W_LONGINT              32       /* bit field width for long integer */
#define W_OCT            16   /* bit field width for octal */
#define W_CHAR            8   /* bit field width for character */
#define W_EXPT            8   /* default length for real exponent */
#define W_SIGNIF         24   /* default length for real mantisa */
#define W_BIN            16
#define W_HEX            16
#define W_CARD           16
#define SC_TEMP_SIZE 128   /* the size of SC_TEMP - the maximun field width in the emulator */

#define M_REAL          16               /* minimum bit field width for real */
#define M_INT           2                /* minimum bit field width for integer */
#define M_LOGICAL       1                /* minimum bit field width for logical */
#define M_INDEX   1              /* minimum bit field width for index */
#define M_OCT     2           /* minimum bit field width for octal */
#define M_CHAR    8           /* minimum bit field width for character */
#define M_BIN     2
#define M_HEX     2
#define M_CARD    1
#define M_EXPT    8           /* minimum length for real exponent */
#define M_SIGNIF  16          /* minimum length for real mantisa */

/* AL VARIABLE DECLARATIONS */

 int      sca_temp,           /* symtab index to int scalar temp identifier */
	    sca_temp_start, /* variable SCA_TEMP_START for "global" temp
					      variable allocation - i.e. longer than a
					      statement - say, control statement size.     */
	    real_sca_temp,  /* symtab index to real scalar temp identifier */
	    log_par_temp_start, /* variable LOG_PAR_TEMP_START */
		 g_offset_start, /* variable similar to log_par_temp_start */
	    ipar_temp,      /* symtab index to integer parallel temp identifier */
		 rpar_temp,         /* symtab index to real parallel temp identifier */
		 lpar_temp,         /* symtab index to logical parallel temp identifier */
		 lipar_temp,        /* symtab index to long int parallel temp identifier */
		 for_count,         /* used to make unique labels for 'for' statement */
	    loop_count,     /* used to make unique labels for 'loop' statement */
	      while_count,       /* used to make unique label for 'while' statement */
	    get_count,      /* used to make unique label for 'get' statement */
		 if_count,          /* used to make unique label for 'if' statement */
	    any_count,      /* used to make unique label for 'any' statement */
		 ifnany_count,   /* used to make unique label for 'ifany' statement */
	    next_count,     /* used to make unique label for 'next' statement */
	    io_count,       /* used to make unique label for io statement */
	    defvar_flag,         /* used to flag not to generate intermediate codes */
	    total_scalars,       /* used to make the table entry for the scalar */
		 g_offset;       /* used to allocate temp space in pass one */



/* AL ARRAY DECLARATIONS */

struct symtab1 {/* symbol table format */
	       int      toktype;        /* type of token */
		    int         mode1;  /* T_INT, T_REAL or T_LOGICAL */
		    int         mode2;  /* T_SCALAR or T_PARALLEL */
		    char  *tokpts;      /* pointer to token string */
	       int      status; /* whether slot is BUSY or IDLE */
		    int         width;  /* bit field width */
		    int   expt;    /* exponent width in bits - real only*/
		    int   signif;  /* mantisa width in bits - real only */
		    int         num_dim;        /* number of dimensions */
		    int         dim_size[MAX_NUM_DIM];  /* size of dimensions */
/*                  char  *ascpts;    pointer to association name */
		    int   bi;      /* index of association name */
	       int   temp;    /* 1 if entry is system temp var */
	       int   offset;  /* offset of temp from base */
	       int   iospec;  /* index of io spec */
	      };

 struct symtab1  symbol_table[SYMTAB1_SIZE];

struct scalar {                                 /* scalar table format */
		    char *arg1;         /* pointer to scalar identifier name */
		    char *arg2;         /* pointer to repeat count for id */
			 char *arg3;            /* pointer to initial value for id */
			 char *arg4;            /* pointer to mode1 value for id */
			 char *arg5;            /* pointer to width of id */
			 char *arg6;            /* pointer to expt of id */
			 char *arg7;            /* pointer to signif of id */
			 int   arg8;    /* TRUE is temp, FALSE otherwise */
	       int   arg9;    /* index of io shape */
	      };

 struct scalar  scalar_table[MAX_SCALARS];
 int table_parse; /* TRUE in in table_parse */

/* DEFINITIONS FOR ERROR MESSAGES */
/*   (must match positions in 'messages' array -- see error.c) */

#define DUP_ERR         5       /* multiply defined symbol */
#define SYMTAB_ERR      6       /* symbol table overflow */
#define MEM_ERR         7       /* out of memory */
#define MODE_ERR           8    /* missing mode keyword */
#define ID_ERR          9       /* identifier expected */
#define EOS_ERR         10      /* end of statement expected */
#define END_ERR         11      /* missing end keyword */
#define RPAR_ERR           12   /* missing right parenthesis */
#define LPAR_ERR           13   /* missing left parenthesis */
#define UNDEF_ERR          14   /* undefined symbol */
#define TEMP_ERR           15   /*using too many compiler temporary variables*/
#define WIDTH_ERR          16   /* invalid width */
#define DIM_ERR         17      /* invalid dimension */
#define LSQU_ERR           18   /* left square bracket expected */
#define RSQU_ERR           19   /* right square bracket expected */
#define ASGNCON_ERR     20      /* illegal type conversion in assignment */
#define ASGNOP_ERR      21      /* missing assignment operator */
#define ASGNP2S_ERR     22      /* can't assign parallel to scalar */
#define ASGNID_ERR      23      /* invalid symbol on left side of assignment */
#define INVID_ERR          24   /* invalid identifier */
#define CONST_ERR          25   /* constant expected */
#define COMMA_ERR          26   /* comma expected */
#define UNRSYM_ERR      27      /* unrecognized symbol */
#define ELEM_ERR           28   /* +, -, (, id, or constant expected */
#define EXPCON_ERR      29      /* illegal type conversion in expression */
#define SUBSCR_ERR      30      /* invalid subscript */
#define IN_ERR             31   /* keyword 'in' expected */
#define ENDFOR_ERR      32      /* statement or keyword 'endfor' expected */
#define PAREXP_ERR      33      /* parallel expression expected */
#define LOGEXP_ERR      34      /* logical expression expected */
#define INXMIS_ERR      35      /* index variable expected */
#define INXPAR_ERR      36      /* index must be parallel */
#define INXUNM_ERR      37      /* unmatched loop index variable */
#define PARVAR_ERR      38      /* parallel variable expected */
#define LOGVAR_ERR      39      /* logical variable expected */
#define ENDNEXT_ERR     40      /* statement or keyword 'endnext' expected */
#define ENDGET_ERR      41      /* statement or keyword 'endget' expected */
#define ENDIF_ERR          42   /* statement or keyword 'endif' expected */
#define ENDIFNANY_ERR 43/* statement or keyword 'endifnany' expected */
#define ENDANY_ERR      44      /* statement or keyword 'endany' expected */
#define ENDWHILE_ERR    45      /* statement or keyword 'endwhile' expected */
#define ENDLOOP_ERR     46      /* statement or keyword 'endloop' expected */
#define WITH_ERR           47   /* keyword 'with' expected */
#define THEN_ERR           48   /* keyword 'then' expected */
#define INCLUDE_ERR     49      /* invalid include statement : quit compile */
#define LOOP_ERR           50   /* statement or keyword 'loop' expected */
#define UNTIL_ERR          51   /* statement or keyword 'until' expected */
#define SCAEXP_ERR      52      /* scalar expression expected */
#define SCASYS_ERR   53 /* too many compiler constant variables */
#define ASSOC_ERR          54   /* invalid associate statement */
#define ENDALLOCATE_ERR 55 /* statement/keyword 'endallocate' expected*/
#define ALLOCATE_ERR 56 /* invalid allocate statement */
#define DEFVAR_ERR      57      /* warning for defvar variables */
#define DEFLOG_ERR      58      /* invalid define scalar logical statement */
#define FILE_ERR           59   /* missing file name in 'include' statement */
#define OPEN_ERR           60   /* file can't be opened */
#define NEST_INCL_ERR 61/* invalid nested 'include' statement */
#define SUBNAME_ERR     62      /* invalid subroutine name */
#define MISSUB_ERR      63      /* missing subroutine name */
#define RETURN_ERR      64      /* invalid return statement */
#define TYPE_ERR           65   /* invalid type of operands */
#define MISINX_ERR      66      /* missing index variable */
#define END_SCOPE_ERR 67/* keyword 'endsetscope' expected */
#define MINMAX_ERR      68 /* parallel integer or real argument expected */
#define ANYLOGPAR_ERR 69/* logical parallel expression expected in any */
#define ASMEOS_ERR      70 /* missing end of statement ';' after 'endasm' */
#define ADDR_SCA_ERR    71      /* scalar var for indirect address expected */
#define ADDR_COL_ERR    72      /* colon expected for indirect address */
#define ADDR_FIELD_ERR 73       /* field length expected for indirect address*/
#define ADDR_TYPE_ERR  74       /* data type expected for indirect address */
#define ADDR_WIDTH_ERR 75       /* invalid width for indirect address */
#define ADDR_LHS_ERR    76      /* invalid left-hand-side address */
#define LOGPAR_ERR      77      /* logical parallel variable expected */
#define ATHEN_ERR    78 /* keyword ANDTHEN expected */
#define ENDANDIF_ERR 79 /* keyword ENDANDIF expected */
#define ENDANDFOR_ERR 80/* keyword ENDANDFOR expected */
#define ANDBODY_ERR  81 /* incorrect AND construction */
#define CONST_WRN    82 /* undeclaired constant type */
#define ZERO_OFF     83 /* offset of zero found */
#define RECURSE_ERR  84 /* keyword RECURSE expected */
#define ENDRECURSE_ERR 85/* keyword ENDRECURSE expected */
#define ENDSTACK_ERR 86 /* keyword  ENDSTACKFOR or
					       ENDSTACKWHILE expected */
#define CUBE_ERR     87 /* illegal network notation on left hand side*/
#define NET_ERR      88 /* illegal network expression */
#define NANY_ERR     89 /* nany requires parallel variable */
#define DO_ERR       90
#define COLON_ERR    91
#define OPEN2_ERR    92 /* error on open file */
#define RWBUF_ERR    93 /* error in read or write statement */
#define FUN_ERR      94 /* error in system subroutine call */
#define ALLELSE_ERR  95 /* error in body of allocat/elsenany */

/* Function DECLARATIONS */

int     parse_start(),             /* top-level of parser */
	   declare(),                 /* parse declarations */
	   var(),                       /* parse declarations */
	   width(),                     /* parse field width portion of declarations */
	   dimension(),            /* parse dimension portion of declarations */
	   varlist(),                 /* parse declarations */
	   symtab_insert(),        /* insert a token into symbol table */
	   get_hash(),                /* compute hashed symbol table index of token*/
	   symtab_fetch(),         /* fetch symbol table index of token */
	   get_symtab_temp(),   /* get symbol table index of temporary var */
	   get_op_mode1(),              /* get operation mode: int, real, logical */
	   get_op_mode2(),              /* get operation mode: scalar, para., common */
	   get_result_mode2(),  /* get result mode: scalar or parallel */
      get_index(),                 /* get symbol table index of index variable */
	   more_identifier(),   /* parse subscripted identifier */
	   subscript(),            /* parse subscript */
	   sum_dif_exp(),                  /* parse add/subt arithmetic expression */
	   more_sum_dif_exp(),  /* parse add/subt arithmetic expression */
	   prod_quot_exp(),        /* parse mult/div arithmetic expression */
	   more_prod_quot_exp(),/* parse mult/div arithmetic expression */
	   element(),                 /* parse plus/minus/paren/constants in exp. */
      check_width(),               /* parse and check address width validation */
	   log_exp(),                 /* parse 'and', 'or', & 'xor'  expressions */
	   more_log_exp(),              /* parse 'and', 'or', & 'xor'  expressions */
	   log_not_exp(),               /* parse logical not expressions */
	   more_log_not_exp(),  /* parse relational expressions */
	   body(),                            /* parse statements */
	   define(),                  /* parse define statements */
	   parameter(),            /* parse parameter list */
	   assign(),                  /* parse assignment statement */
	   for_(),                            /* parse for statement */
      get_(),                         /* parse get statement */
      any_(),                         /* parse any statement */
      ifnany_(),                      /* parse ifnany statement */
      while_(),               /* parse while statement */
      loop_(),                   /* parse loop statement */
	   more_loop(),            /* parse loop statement */
	   first_(),                  /* parse loop statement */
	   first_body(),                   /* parse loop statement */
      if_(),                          /* parse if statement */
	   more_if_(),                /* parse if statement */
	   next_(),                      /* parse next statement */
      more_next(),                 /* parse next statement */
	   assemble_(),            /* parse assembly language statement */
      process_subscr(),    /* parse and calculate the subscript offset */
	   process_opt_subscr(),/* parse and calculate the subscript offset */
	   get_offset(),                   /* parse and calculate the subscript offset */
	   get_dimension(),        /* parse and calculate the subscript offset */
	   calculate_offset(),  /* parse and calculate the subscript offset */
	   add_index(),            /* parse and calculate the subscript offset */
	   allocate_(),            /* parse allocate statement */
      associate_(),                /* parse associate statement */
	   pvar_list(),            /* parse associate statement */
	   check_parallel(),       /* parse associate statement */
      get_plist(),                 /* parse associate statement */
      deflog(),               /* parse define logical statement */
	scalar_fetch(),         /* get the symbol table index for constants */
	   define_fetch(),              /* get scalar table index for defined const */
      call_(),                   /* parse 'call' statement */
      return_(),                      /* parse return statement */
      stop_(),                   /* parse stop statement */
	   setscope_();            /* parse setscope statement */


void    init_symbol_table(),    /* initialize symbol table */
	print_opcode(),            /* print opcode to listing and object files */
	print_args(),                 /* print arguments to listing & object files */
	fix_(),                          /* convert real to integer */
	float_(),                        /* convert integer to real */
   short_(),                     /* convert long integer to short integer */
   long_(),                         /* convert short integer to long integer */
	symtab_dump(),                /* dump symbol table */
   install_scalar(),          /* store the scalar variables in a table */
   dump_scalar(),                     /* dump table containing scalar variables */
   include();                    /* parse the 'include' statement */



char    *make_filename(),          /* make a new file name */
	   *make_constname(),   /* make a name for a constant */
	   *make_label(),                  /* make a unique statement label */
	   *first_word(),                  /* find first word in a buffer */
      *get_file_name();    /* get a file name after 'include' keyword */


/* these are global variables for the list stack used in p1_stackfor.c,
   p1_stackwhile.c  and  the field list in p1_print.c
*/
#define LIST_STACK_SIZE 15*2 /* max number of fields in an assoc list */

 int list_stack[LIST_STACK_SIZE];
 int list_pt,save_cnt;

 int count_index,stack_index,stack_offset,mreg_save;
 char *stack_end,*stack_repeat,*stack_recurse;
 int cube_flag,cube_op,cube_index,cube_offset;
 int g_lhs_dx;   /* see p1_assign.c and p1_subscript.c */
 int g_assign;   /* TRUE if inside an assign statement */

 int iofp;      /* true when iospec redirction starts */
 int iospec_on; /* true when iospec redirction ends   */

#endif

