/* 910415  P1_RECURSE.C */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"

/* ===========================================================================

   function name : recurse_
   input	        :
   output	     :
   return value  :


   global vars   :

   description	  : recurse_ must be a separate statement so that a call to
                   body() will parse correctly. i.e.

		  stackwhile
		     :
		     while
		     :
		  recurse           need   ... endrecurse;
		     :
		     endwhile;
		     :
        endstackwhile;

=========================================================================== */

int recurse_()
{
   int return_value;		/* return value of function */

	if (token_type != T_RECURSE)
	  {print_msg (RECURSE_ERR, token_buf);
	   return_value = FAIL;}
	else if(stack_list() != SUCCESS)
	  {print_msg (RECURSE_ERR, token_buf);
		return_value = FAIL;}
   else
	  {list_pt = 0;
	   while((stack_index=next_stack()) != FAIL)
		  {stack_offset=next_stack();
         print_opcode ("stack_", T_STACK, T_NONE,
			   	        symbol_table[stack_index].mode2);
         print_args (off_exp1(stack_index,stack_offset),NULL,NULL,NULL, NULL);}
      if((return_value = body()) != SUCCESS)
		  {print_msg (RECURSE_ERR, token_buf);
		   return_value = FAIL;}
      else
		  {print_opcode ("recurse", T_RECURSE, T_NONE, T_NONE);
         print_args (symbol_table[count_index].tokpts,
                     symbol_table[mreg_save].tokpts,
			            stack_recurse, stack_repeat, NULL);
		  list_pt--;
        while((stack_offset=stack_next()) != FAIL)
		    {stack_index=stack_next();
           print_opcode ("poprc_", T_POPRC, T_NONE,
		   	             symbol_table[stack_index].mode2);
           print_args (symbol_table[count_index].tokpts,
							  off_exp1(stack_index,stack_offset),
							  NULL,NULL, NULL);}
        if (token_type != T_ENDRECURSE)
	       {print_msg (ENDRECURSE_ERR, token_buf);
	        return_value = FAIL;}
		  else if ((token_type = get_token()) != T_EOS)
		    {print_msg (EOS_ERR, token_buf);
		     return_value = FAIL;}
		  else
		    {token_type = get_token();
		     return_value = SUCCESS;}
	    }
     }
   return(return_value);
}



int stack_list()
/* This routine parses lists of the form
   field1[index,exp] field2[index,exp] ... fieldn[index,exp] then assign;
   It saves all of the fields and their offsets on a stack and returns a
   pointer to the association name.   The THEN is parsed, and
   the nextoken is scanned.
   Next_elem() reads the items off the list.
   If no offset, SUCCESS is returned by subscript and it is inserted.
   The list may be reread by setting list_pt to zero.

WARNING - this routine uses list_pt, save_cnt, list_stack[]
          variables declared in p1_print.c
*/
{
 int field_dx,log_dx,log_off;

   save_cnt=0;
   list_pt=0;
   token_type = get_token();
   while(token_type == T_ID && save_cnt<LIST_STACK_SIZE)
     {if((field_dx =
             list_stack[save_cnt++]=symtab_fetch(token_size,token_buf))== FAIL)
        {print_msg(UNDEF_ERR,token_buf); return(FAIL);}
	   token_type=get_token();
	   subscript(field_dx,&log_dx,&log_off);
	   list_stack[save_cnt++] = log_off; }
/* subscript calls get_token */
   if(token_type != T_THEN) { print_msg(THEN_ERR,token_buf); return(FAIL);};
   token_type = get_token();
   return(SUCCESS);
  }


next_stack()
{
   int result;

   if(list_pt < 0 )
     {printf("ERROR in next_stack - list_pt has negative value %d\n",list_pt);
      result = FAIL;}
   else if(list_pt < save_cnt)
	  {result =list_stack[list_pt]; }
   else result = FAIL;
   list_pt++;

   return(result);
}


stack_next()
/* stack_next retruns the stack list in reverse order.  Start
   with list_pt at max value(save_cnt) and go until zero.
   Note, offset is returned first and then index   */
{
   int result;

   list_pt--;
   if(list_pt < 0 ) result = FAIL;
   else if(list_pt < save_cnt)
	  {result =list_stack[list_pt]; }
   else result = FAIL;

   return(result);
}
