/* 910415 P1_RELEASE.C */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"

/* ==================================================
   function name : release_
	input         : ---
	output        : ---
	return value  : return_value - SUCCESS, if no error
										  - FAIL, otherwise
   global vars   : symbol_table - array containing symbol table
						 token_type   - type of current token
						 token_buf    - character string for current token
						 debug        - flag for debugging statements
   description   : This routine is part of a recursive descent parser and
						 handles the following production from the grammar:

                      release <-- LOG_EXP from ASSOCIATION_NAME;
						 The expression must evaluate to a logical parallel result.
						 The responding items will be cleared in association name.
	=================================================== */

int release_()
{
  int return_value;
  int expr_index;
  int expr_offset;
  int bi_index,bi_offset;

  if(debug == 2)
	 printf("In release_().............................\n");

  return_value = SUCCESS;
  log_exp(&expr_index,&expr_offset);
  if(expr_index == FAIL)
	  return_value = FAIL;
  else if (symbol_table[expr_index].mode2 == T_PARALLEL &&
	        symbol_table[expr_index].mode1 == T_LOGICAL)
	 {if(token_type != T_FROM)
		 {printf("ERROR - keyword FROM expected. %s found on line %d.\n",
					 token_buf,linenum);
		  return_value = FAIL;}
	  else if ((bi_index = getbi(&bi_index,&bi_offset)) == FAIL)
		 {print_msg(PARVAR_ERR, token_buf);
		  return_value = FAIL;}
	  else if (token_type != T_EOS)
		 {print_msg(EOS_ERR,token_buf);
		  return_value = FAIL;}
	  else
		 token_type = get_token();
	  print_opcode("release_",T_RELEASE, T_NONE, T_PARALLEL);
	  print_args(off_exp1(expr_index,expr_offset),
					 off_exp2(bi_index,bi_offset), NULL, NULL, NULL);
	 }
  else
    {printf("ERROR - logical parallel expressions expected\n");
	  return_value = FAIL;}

  if(debug == 2)
	 printf("release_() returning %d\n", return_value);

  return(return_value);
}

getbi(bi_index, bi_offset)
int *bi_index, *bi_offset;
{
  int index;

  token_type = get_token();
  index = symtab_fetch(token_size, token_buf);
  token_type = get_token();
  subscript(index,bi_index,bi_offset);

  return (*bi_index);
}
