/* 910415  P1_SCANNER.C */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_scanner.h"
#include "p1_parser.h"

#define C_DASH 45 /* ascii - */
#define C_UNDER 95 /* ascii _ */
#define T_BAR 124 /* ascii | */
#define T_TIMES 88 /* ascii X */
#define T_EQUAL T_ASSIGN /* ascii = */

/* ===========================================================================

   function name : get_token
   input                : ---
   output            : ---
   return value  : returns type of token

   global vars   : token_type   - type of current token
			     token_buf  - current token string
			     symbol_table       - symbol table data structure
			     debug              - debugging statement flag

   description    : this routine is checking for the "#include" statement in
			     the source file. If the current token is "#include", this
			     routine will invoke 'include()' routine to switch the
			     file pointer to read in data from the new included file until
			     EOF is detected. After consuming all data in the included
			     file, this routine will switch back the file pointer to
			     the original source file.

=========================================================================== */

int get_token()
{
   int current_toktype;
   char ch;

   if(strlen(line_buf) + strlen(token_buf) > 255)
	{printf("%d ERROR - line buffer overflow \nline_buf=%s.\ntoken_buf=%s.\n",
	    linenum,line_buf,token_buf);
	 if(token_type == T_EOF)
	    printf("%d ERROR - unexpected EOF\n",linenum);
	    exit(1);}
   strcat(line_buf,token_buf); /* save old tokens in line_buf */
   strcat(line_buf," ");
   current_toktype = get_current_token();
   if (current_toktype == T_INCLUDE ||
       current_toktype == T_IOSPEC )
     {if (include_flag != TRUE)
	{save_ln = linenum;
	 linenum = 0;
	 include();
	 if (current_toktype == T_IOSPEC )
	     iofp = TRUE; /* turned off in fill_buf */
	 current_toktype = get_current_token();
	 }
      else         
	{print_msg(NEST_INCL_ERR, NULL);
	 print_msg(INCLUDE_ERR, NULL);
	 printf("\n");
	 exit(0);}
     }

  if(current_toktype == T_ID)
    {ch = getnextchar();
      if(ch == '$' && code_type == EMULATE && check_dol() != FAIL)
		   {*token_pnt++ = ch;
	  current_toktype = T_SID;}
	   else putbackchar();
    }

 if(current_toktype == T_ID &&    /* for ACE */
	  strcmp(token_buf,"X") == 0  &&
	  symtab_fetch(token_size,token_buf) == FAIL)
		 {if(table_parse != TRUE) printf("%d WARNING - undeclared X interpreted as multiply\n",linenum);
			current_toktype = T_MULT;}
/*
printf("In get_token, token_type=%d, token_buf=%s\n",current_toktype,token_buf);
*/
   return(current_toktype);

} /* end of get_token() */



/* ==========================================================================

   function name : get_current_token
   input                : ---
   output            : ---
   return value  : returns type of token

   global vars   : token_size   - size of current token
			     token_buf  - char. array containing current token
			     token_pnt  - pointer to 'token_buf' array
			     debug         - variable for debugging statement
			     previous_type - type of previous token

   description   : This function is the main routine of a scanner. It
			     recognizes the following tokens in a context-free
			     manner:

		  --KEYWORDS--
		  any, allocate, associate, define, during, end, endany,
		  endfor, endget, endif, endifnany, endloop, endnext,
		  endwhile, else, elseany, for, first, get, in, #include,
		  index, int, if, ifnany, logical, loop, main, next,
		  parallel, procedure, real, scalar, then, thenany, until,
		  while, with

		  --CONSTANTS/IDENTIFIERS--
		  H'....', O'....', B'....', decimal constants,
		  identifiers

		  --OPERATORS--
		  !, &&, ||, ^^, >, <, >=, <=, ==, !=, +, -, *, /, [, ]
		  **
		  --SPECIAL SYMBOLS--
		  ), (, $, ;, :, ,, CTL-D

		  The tokens '+' and '-' are recognized in a context-
		  sensitive manner. If the token is preceeded by a ')',
		  ']', identifier, or number constant, it is a binary
		  ADD or SUBTRACT. Otherwise, it is a unary PLUS or
		  MINUS operator. The token 'identifier' is defined
		  as a letter followed by any combination of letters,
		  numbers, or underscores. No distinction is made
		  between upper and lowercase. Only integer numeric
		  constants are recognized at this time. The function
		  returns the token type. The token size and token
		  spelling are made available in the global variables
		  'token_size' and 'token_buf'. The token is terminated
		  by EOB (end of buffer).

=========================================================================== */

int get_current_token()
{
   char ch;                     /* current character from input buffer */
   int type;            /* return value -- type of token found */
   int num_chars;       /* number of characters in token so far */
   unsigned long int temp, mask, accum;
   int i, scnt;
   char *savencpnt;

   token_pnt = token_buf;
/* Remove any whitespace characters preceeding the token. */
   savencpnt = ncpnt;
   while (((ch = getnextchar()) != EOF) && char_type(ch) == WHITESPACE);
   delim_size = ncpnt - savencpnt -1;
/*
 * Place the first non-whitespace character in the token buffer and
 * proceed through the state transition network to find the rest of
 * that token.
 */
   *token_pnt++ = ch;
   num_chars = 1;
   switch (ch)
     {case 'X' :
		 type = state_2(num_chars);     /* HEX CONSTANT, ID */
		 if(type == T_HEX_CONST)
					    {*token_pnt = 0;
						  sscanf(token_buf,"%lx",&temp);
					     pc_ultoa(temp,token_buf);
						  num_chars = strlen(token_buf);
		    token_pnt = token_buf + num_chars;}
		 break;
      case 'B' : type = state_5(num_chars);     /* BINARY CONSTANT, ID */
		 if(type == T_BIN_CONST)
					    {*token_pnt = 0;
						  sscanf(token_buf,"%lo",&temp);
		    mask = 1;
		    accum = 0;
		    for(i=0; i<10; i++)
		      {if((mask & temp) != 0) accum |= 1<<i;
		       mask <<= 3;}
					     pc_ultoa(accum,token_buf);
						  num_chars = strlen(token_buf);
		    token_pnt = token_buf + num_chars;}
		 break;
      case 'O' : type = state_8(num_chars);     /* OCTAL CONSTANT, ID */
		 if(type == T_OCT_CONST)
					    {*token_pnt = 0;
						  sscanf(token_buf,"%lo",&temp);
					     pc_ultoa(temp,token_buf);
						  num_chars = strlen(token_buf);
		    token_pnt = token_buf + num_chars;}
		 break;
      case '^' : type = state_12();     /* LOGICAL XOR */
		 break;
      case '!' : type = state_13();     /* LOGICAL NOT, NOT EQUAL */
		 break;
      case '&' : type = state_14();     /* LOGICAL AND */
		 break;
      case '<' : type = state_15();     /* LESS THAN, LESS OR EQUAL */
		 break;
      case '>' : type = state_16();     /* GREATER, GREATER OR EQ. */
		 break;
      case '=' : type = state_17();     /* ASSIGNMENT, EQUAL */
		 break;
      case '|' : type = state_18();     /* LOGICAL OR */
		 break;
      case '[' : type = T_LSQU;         /* LEFT SUBSCRIPT */
		 break;
      case ']' : type = T_RSQU;         /* RIGHT SUBSCRIPT */
		 break;
      case '(' : type = T_LPAR;         /* LEFT PARENTHESIS */
		 break;
      case ')' : type = T_RPAR;         /* RIGHT PARENTHESIS */
		 break;
      case '*' : type = state_41();     /* MULTIPLY */
		 break;
      case '/' : type = T_DIV;          /* DIVIDE */
		 break;
      case '%' : type = T_MOD;          /* MODULO DIVIDE */
		 break;
      case '+' : switch (previous_type) /* ADD, UNARY PLUS */
		   {case T_RPAR      :
		    case T_RSQU      :
		    case T_ID        :
		    case T_HEX_CONST :
		    case T_BIN_CONST :
		    case T_OCT_CONST :
		    case T_FLT_CONST :
		    case T_DEC_CONST : type = T_ADD;
				   break;
		    case T_LT        :
		    case T_LE        :
		    case T_EQ        :
		    case T_NE        :
		    case T_GT        :
		    case T_GE        :
		    case T_COMMA     :
		    case T_COLON     :
		    case T_LSQU      :
		    case T_ASSIGN    :
		    case T_SUB       :
		    case T_PAR_SUBSCRIPT:
		    case T_MINUS     :
		    case T_IF        :
		    case T_ANY       :
		    case T_IN        :
		    case T_UNTIL     :
		    case T_NANY      :
		    case T_RELEASE   :
		    case T_SETSCOPE  :
		    case T_BAR       :   /* for |+5| in ace */
		    case T_LPAR      : type = T_PLUS;
				break;
		    default          : type = T_PLUS;
				break;
/*                            default          : printf("%d ERROR - illegal type preceding a %s, %x\n",
													linenum,token_buf,previous_type);
													break;
*/
		   }
		 break;
      case '-' :
		switch (previous_type) /* SUBTRACT, UNARY MINUS */
		   {case T_RPAR      :
		    case T_RSQU      :
		    case T_ID        :
		    case T_BIN_CONST :
		    case T_HEX_CONST :
		    case T_OCT_CONST :
		    case T_FLT_CONST :
		    case T_DEC_CONST : type = T_SUB;
				       break;
		    case T_LT        :
		    case T_LE        :
		    case T_EQ        :
		    case T_NE        :
		    case T_GT        :
		    case T_GE        :
		    case T_COMMA     :
		    case T_COLON     :
		    case T_LSQU      :
		    case T_ASSIGN    :
		    case T_SUB       :
		    case T_PAR_SUBSCRIPT:
		    case T_MINUS     :
		    case T_IF        :
		    case T_ANY       :
		    case T_IN        :
		    case T_UNTIL     :
		    case T_NANY      :
		    case T_RELEASE   :
		    case T_SETSCOPE  :
		    case T_BAR       :   /* for |-5| in ace */
		    case T_LPAR      : type = T_MINUS;
				       break;
		    default          : type = C_DASH;
				       break;
/*                            default          : printf("%d ERROR - illegal type preceding a -, %x\n",
													linenum,previous_type);
													break;
*/
		   }
		 break;
      case '@' : type = T_ADDRESS;      /* INDIRECT ADDRESS UNARY */
printf("At @ in p1_scanner.c\n");
			   break;
      case ';' : type = T_EOS;          /* END OF STATEMENT */
		 break;
      case ':' : type = T_COLON;        /* COLON */
			   break;
      case EOF : type = T_EOF;          /* END OF FILE */
		 break;
      case '$' : type = T_PAR_SUBSCRIPT;/* PARALLEL SUBSCRIPT */
		 break;
      case ',' : type = T_COMMA;        /* COMMA */
		 break;
      case '.' : type = state_19(num_chars);      /* RELATIONAL OPERATORS */
		 break;
      case  39 : type = T_SQUOTE;
		  break;
      case '"' : token_pnt--; /* erase " */
		 upper = FALSE;
		 scnt=0;
		 while (((*token_pnt = getnextchar()) != T_EOF) &&
			 (*token_pnt != '"') &&
			 (*token_pnt++ != '\n')) scnt++;
		  if(scnt > TOKEN_SIZE)
printf("%d ERROR - string exceeds buffer size\n   %s\n",linenum,token_buf);
		  upper = TRUE;
		 *token_pnt = 0;
					  type = T_STRING;
					  break;
      default  : if (char_type(ch) == NUMERIC)    /* DECIMAL NUMBER */
			     type = state_11(num_chars);
		 else if (char_type(ch) == ALPHA) /* IDENTIFIER */
		       {if(ace_type == ACE && strncmp(token_buf,"_",1) == 0)
			   type = C_UNDER;
			else
			   type = state_1(num_chars);
		       }
		 else
			     type = T_UNKNOWN;            /* UNKNOWN */
     }

   if(type == T_UNKNOWN)
	  printf("%d ERROR - unknown char type in scanner %d, %c\n",linenum,ch,ch);
/*   else printf("In scanner, type = %d\n",type); */
   *token_pnt = EOB;

/*
 * If token type is T_ID, check to see if it is a compiler keyword. (If so,
 * its type will be changed to that of the correct keyword.) If the token
 * type is T_ID and it is not a compiler keyword, check to see if it is an
 * assembler keyword. (If so, its spelling will be modified to have a '$'
 * preceding it.) If the new length exceeds MAX_ID_SIZE it is truncated and a
 * message is printed.)
*/
   if (type == T_ID)
     {if (!compiler_keyword(token_buf, &type))
	{assembler_keyword(token_buf);
	 token_size = token_pnt - token_buf;
	 if (token_size > MAX_ID_SIZE &&
		 (code_type != EMULATE && code_type != CM) )
	   {token_pnt = &token_buf[MAX_ID_SIZE];
	   *token_pnt = EOB;
	    if(table_parse != TRUE)
	      printf("%d WARNING - identifier truncated to 6 char\n",linenum);
	   }
	}
     }


   token_size = token_pnt - token_buf;
   previous_type = type;
   return(type);

} /* end of get_current_token() */

/* ===========================================================================

   function name: getnextchar
   input        : ---
   output       : ---
   return value : ret_value = ch        ; if EOF has not been detected.
		  ret_value = EOF       ; if EOF has been detected.

   global vars  : eofflg        - flag for EOF detected or not
			    ncpnt               - pointer to next char in input buffer
			    buffer      - array of pointers to 2 input buffers
			    current_buf - index for 'buffer' array
			    fill_buf_flg        - flag for whether to fill buffer or not
		  debug         - variable for debugging statement

   description  : This routine is supplying a character from the current
		  input buffer array. If the end of buffer has been reached
			    then it will call 'fill_buf' to fill another input buffer
			    from the input stream. EOF will be returned if there are
			    no more characters to be returned. Otherwise a character
			    will be returned.

=========================================================================== */

int getnextchar()
{
   char ch;                  /* character identifier read from the current input
				      buffer */
   char ret_value;  /* identifier for storing the return value */
   int  done;             /* variable for ending the loop */

   done = FALSE;
   if (!eofflg)
     {ch = *ncpnt++;
      while (!done)
	{switch (ch)
		{case EOF : done = TRUE;
			    eofflg = TRUE;
				 ret_value = ch;
		       break;
	    case EOB : /* save position of last character of current buffer */
			    buffer[current_buf].end = ncpnt-2;
			   if(buffer[current_buf].end < buffer[current_buf].start)
			  buffer[current_buf].end = buffer[current_buf].start;
			    /* switch to other buffer */
				 if (current_buf == 0)
					current_buf = 1;
				 else
					current_buf = 0;
				 ncpnt = buffer[current_buf].start;
			      /* decide if it needs to be filled with new char */
				 if (fill_buf_flg)
				   {fill_buf(ncpnt);
					 ncpnt = buffer[current_buf].start;}
				 else
					 fill_buf_flg = TRUE;
		       ch = *ncpnt++;
				 break;
		 default  : done = TRUE;
				 ret_value = ch;
	   }
	}
     }
   else
      ret_value = EOF;

   if (isalpha(ret_value) && islower(ret_value))
     if(code_type == EMULATE && ace_type != ACE)
      if(upper == TRUE) ret_value = toupper(ret_value);
   return (ret_value);

} /* end of getnextchar() */


/* ===========================================================================

   function name: putbackchar
   input               : ---
   output           : ---
   return value : ---

   global vars  : ncpnt         - pointer to next char in input buffer
			    buffer      - array of pointers to 2 input buffers
			    current_buf - index for 'buffer' array
			    fill_buf_flg        - flag for whether to fill buffer or not
		      debug             - variable for debugging statement

   Description  : This routine is called by the 'state_???' functions in
			 the process of getting a token. This routine is putting
			    the last character read from the current input buffer
			    back to its location in the current input buffer. This
			    is done by decrementing the pointer to the buffer. If the
			    pointer has reached the starting location of the buffer,
			    a switch is made to the other buffer. The pointer is set to
			    the last character of the other buffer. The 'fill_buf_flg'
			    variable will be set to FALSE to prevent the first buffer
			    from being filled again.

=========================================================================== */

void putbackchar()
{
   if (ncpnt == buffer[current_buf].start)
     {if (current_buf)
	     current_buf = 0;
      else
	     current_buf = 1;
      ncpnt = buffer[current_buf].end;
      fill_buf_flg = FALSE;}
   else
      --ncpnt;

} /* end of putbackchar() */


/* ==========================================================================

   function name: fill_buf
   input               : inbuf          - pointer to buffer to be filled
   output       : ---
   return value : ---

   global vars  : linenum       - current line number
			    in_comment  - flag for in comment or not
			    current_char        - current character from input stream
			    next_char   - next character form input stream
			    list_fp     - file pointer for listing file
			    active_fp   - file pointer for active file
			    count               - position of character on line in listing
			    debug         - variable for debugging statement

   description  : This routine provides double buffering for the input
			    stream. The buffer pointed to by 'inbuf' will be
			 filled with at most BUF_SIZE characters. Comments will
			    be stripped out but all characters (including comments)
			    are printed to a listing file (list_fp) with line
			    numbers. The buffer is returned when BUF_SIZE characters
			    are read or when newline is read. Each buffer ends with
			    EOB (end of buffer) character.

=========================================================================== */

void fill_buf(inbuf)
char *inbuf;            /* pointer to buffer to be filled */
{
   int i, j;            /* loop counter */
   int done;            /* flag for ending filling buffer */

   i = 0;
   done = FALSE;
   if (incl_eof_flag == TRUE) {include_flag = FALSE; linenum = save_ln;}
   while((i < BUF_SIZE) && !done)
     {if (current_char == '\n')
	{linenum++;
	 fprintf(list_fp, "%4d ", linenum);
	      strcpy(line_buf,"");
	 count = 0;}
      incl_eof_flag = FALSE;
      if (use_next_char_save == TRUE)
	{current_char = next_char_save;
	 use_next_char_save = FALSE;}
      else
	{if (include_flag == TRUE && use_once_flag == TRUE)
	   {current_char = getc(active_fp);
	    use_once_flag = FALSE;}
	 else
	    current_char = next_char;
	}
      next_char = getc(active_fp);
      if (include_flag == FALSE)
	 next_char_save = next_char;
      if (next_char == EOF && include_flag == TRUE)
	{fclose(include_fp);
	 incl_eof_flag = TRUE;
	 active_fp = source_fp;
	 use_next_char_save = TRUE;
	 next_char = '\n';
	 if(iofp == TRUE)
	   {iospec_on = TRUE; /* turned off in a_parse */
	    iofp = FALSE;}
	}

      putc(current_char, list_fp);
      count++;

      switch (current_char)
	{case EOF : if(in_comment)
				print_msg(EOF_ERR, NULL);
			      done = TRUE;
			      *inbuf++ = current_char;
			      break;
	      case '\n': done = TRUE;
			      *inbuf++ = current_char;
			      break;
	      case '/' : if (next_char == '*')
				{current_char = next_char;
				 next_char = getc(active_fp);
				 in_comment++;
				 putc(current_char, list_fp);
				 count++;}
			      else if (!in_comment)
				{*inbuf++ = current_char;
			    i++;}
			      break;
	      case '*' : if (next_char == '/')
				{current_char = next_char;
				 next_char = getc(active_fp);
				 putc(current_char, list_fp);
				 count++;
				 in_comment--;
		       if (in_comment < 0)
			 {in_comment = 0;
					 print_msg(EOC_ERR, NULL);}
			   }
			      else if (!in_comment)
				{*inbuf++ = current_char;
				 i++;}
			      break;
	  default : if (!in_comment)
			   {if (char_type(current_char) != ILLEGAL)
				   {*inbuf++ = current_char;
			       i++;}
				 else
				   {print_msg(CHAR_ERR, NULL);
				  printf("line_buf=%s, char=%x, %c\n",
				  line_buf,current_char,current_char);
					 for (j = 0; j < count+4; j++)
					    fprintf(list_fp, "$");}
				}
		     break;
	   }
      }
   *inbuf = EOB;

}  /* end of fill_buf() */


/* ===========================================================================

   function name: char_type
   inputs           : ch         - an ASCII character
   outputs          : ---
   return value : ALPHA      - if the character is a letter
			    NUMERIC    - if the character is a digit
			    WHITESPACE - if the character is blank, tab, or newline
			    SPECIAL    - if the char. is legal, but not one of the above
			    ILLEGAL    - if the char. is illegal in the language

   global vars   : ---

   description  : This routine examines the character passed as an argument
			    and returns one of the categories listed above.

=========================================================================== */


int char_type(ch)
int ch;
{
   int type;            /* value to be returned by function */

   if ((ch >= 'a' && ch <= 'z') ||
		 (ch >= 'A' && ch <= 'Z') ||
		  ch == '#'                   ||
		  ch == '_')
      type = ALPHA;
   else if (ch >= '0' && ch <= '9')
      type = NUMERIC;
   else if (ch == ' ' || ch == '\t' || ch == '\n')
      {type = WHITESPACE;
		 if(ch == '\n') nl_on++;} /* count nls */
   else
     {switch(ch)
	{case EOF :
	 case '!' :
	      case '"' :
	      case '$' :
	      case '%' :
	      case '&' :
	      case '\'':
	      case '(' :
	      case ')' :
	      case '*' :
	      case '+' :
	      case ',' :
	      case '-' :
	      case '.' :
	      case '/' :
	      case ':' :
	      case ';' :
	      case '<' :
	      case '=' :
	      case '>' :
	      case '?' :
	      case '@' :
	      case '[' :
	      case '\\':
	      case ']' :
	      case '^' :
	      case '`' :
	      case '{' :
	      case '|' :
	      case '}' :
	      case '~' : type = SPECIAL;
			      break;
	      default  : type = ILLEGAL;
	}
     }
   return(type);
}
 /* end of char_type() */


/* ===========================================================================

   function name: hex
   input               : ch             - ASCII character
   output           : ---
   return value : returns non-zero if character is hex letter, otherwise 0

   global vars. : ---

   description  : This routine determines whether a character is in the range
			    a - f or A - F.

=========================================================================== */

int hex(ch)
char ch;
{

  return ((ch >= 'a' && ch <= 'f') || (ch >= 'A' && ch <= 'F'));

} /* end of hex() */


/* ===========================================================================

   function name: octal
   input               : ch             - ASCII character
   output           : ---
   return value : returns non-zero if character is octal, otherwise 0

   global vars. : ---

   description  : This routine determines whether the character is in the
			    range 0 - 7.

=========================================================================== */

int octal(ch)
char ch;
{

   return (ch >= '0' && ch <= '7');

} /* end of octal() */

