/* 910415  P1_SETSCOPE.C */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"

/* ===========================================================================

   function name : setscope
   input	        : ---
   output	     : ---
   return value  : ret_value	- SUCCESS, if no error is detected
				                  - FAIL, otherwise, if there is an error

   global vars   : token_type 	- type of current token
		             token_buf	- current token string
		             symbol_table	- symbol table data structure
		             debug		- debugging statement flag

   description	  : This routine parses the setscope statement.

			SETSCOPE   <- logical_parallel BODY endsetscope ;

		  logical_parallel should be a logical parallel variable or
		  a system value TRUE or FALSE.

=========================================================================== */

int setscope_()
{
   int ret_value;		/* variable for value being return */
   int log_index;		/* index variable for logical variable */
   int log_offset;

   if (debug == 2)
      printf ("In setscope_()..................................\n");


   ret_value = SUCCESS;

   if (token_type == T_TRUE)
     {print_opcode("stack_scope",T_STACK_SCOPE,T_NONE,T_NONE);
	   print_args (symbol_table[LP_TRUE].tokpts, NULL, NULL, NULL, NULL);
      token_type = get_token();}
   else if (token_type == T_FALSE)
     {print_opcode("stack_scope",T_STACK_SCOPE,T_NONE,T_NONE);/*added 880316 jlp */
      print_args (symbol_table[LP_FALSE].tokpts, NULL, NULL, NULL, NULL);
      token_type = get_token();}
   else
     {if(log_exp(&log_index,&log_offset) == FAIL)
         ret_value = FAIL;
      else if (symbol_table[log_index].mode2 != T_PARALLEL)
        {print_msg(PAREXP_ERR, NULL);
         ret_value = FAIL;}
      else if (symbol_table[log_index].mode1 != T_LOGICAL)
        {print_msg(LOGEXP_ERR, NULL);
         ret_value = FAIL;}
      else
        {save_em(log_index,log_offset);
			print_opcode("stack_scope",T_STACK_SCOPE,T_NONE,T_NONE);
         print_args (off_exp1(log_index,log_offset), NULL, NULL, NULL, NULL);}
   }

   if (ret_value != FAIL)
     {if ((ret_value = body()) != FAIL)
        {if (token_type != T_ENDSETSCOPE)
           {print_msg (END_SCOPE_ERR, token_buf);
            ret_value = FAIL;}
         else
           {token_type = get_token();
            if (token_type != T_EOS)
              {print_msg(T_EOS, token_buf);
               ret_value = FAIL;}
            else
              {print_opcode ("pop_scope", T_POP_SCOPE, T_NONE, T_NONE);
               print_args (NULL, NULL, NULL, NULL, NULL);
               token_type = get_token();}
         }
      }
   }

   if (debug == 2)
      printf ("setscope returning ret_value : %d\n", ret_value);

   return(ret_value);

} /* end of setscope_() */
