/* p1_shape.c */
#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"

/* ===========================================================================

   function name: shape
   input	: ---
   output	: ---
   return value : return_value	- SUCCESS, if no errors detected
		  		- FAIL, otherwise

   global vars  : token_type	- type of current token
		  debug		- flag for debugging statements
		  token_size	- size of current token
		  token_buf	- character string for current token
		  symbol_table	- array containing symbol table

   description	: This routine is part of a recursive descent parser and
		  handles the following production from the grammar:

			define <-- shape [ sizex, sizey, sizez ] |
		  '@' represents the empty production. The shape sizes
		  are used to determine the dimensionally of the parallel
	          array.  If no shape function is called, the sizes are
                  set to default values in p1_main.

=========================================================================== */

int init_shape(ex_vl)
int ex_vl;
{
   int return_value;		/* value returned by function */
   int i,mask;
   char buffer[32], tbuffer[10];

   if (debug == 2)
      printf ("In shape(ex_vl=%d)...........................\n",ex_vl);

   return_value = SUCCESS;
   /* ex_vl == 2 multiprogram file, do not reshape */
   if(ex_vl == 2 ) return(return_value);

 if(wt == WT)
   {xdim = 64;
    ydim = 64;
    zdim = 1;}
 else
   {xdim = 4096;
    ydim = 1;
    zdim = 1;}

  if ((token_type = get_token()) == T_SHAPE)
    if ((token_type = get_token()) != T_LSQU)
      {
         print_msg (LSQU_ERR, token_buf);
         return_value = FAIL;
      }
      else
        {token_type = get_token();
	      if (token_type == T_BIN_CONST || token_type == T_OCT_CONST ||
	          token_type == T_DEC_CONST || token_type == T_HEX_CONST)
	          xdim = atoi(token_buf);
         else printf("%d ERROR - unrecognized x dimension constant in shape statement\n",linenum);
         if((token_type = get_token()) == T_COMMA) token_type = get_token();
         if (token_type == T_BIN_CONST || token_type == T_OCT_CONST ||
	          token_type == T_DEC_CONST || token_type == T_HEX_CONST)
	        ydim = atoi(token_buf);
         else printf("%d ERROR - unrecognized y dimension constant in shape statement\n",linenum);
         if((token_type = get_token()) == T_COMMA) token_type = get_token();
	      if (token_type == T_BIN_CONST || token_type == T_OCT_CONST ||
	          token_type == T_DEC_CONST || token_type == T_HEX_CONST)
	        zdim = atoi(token_buf);
         else printf("%d ERROR - unrecognized z dimension constant in shape statement\n",linenum);

         if ((token_type = get_token()) != T_RSQU)
	        {print_msg (RSQU_ERR, token_buf);
	         return_value = FAIL;}
         else if ((token_type = get_token()) != T_EOS)
	        {print_msg (EOS_ERR, token_buf);
	         return_value = FAIL;}
	      else
           {token_type = get_token();
	         return_value = SUCCESS;}
         }

   mask = 1;
   xid_siz = 1; yid_siz = 1; zid_siz = 1;
   for(i=0; i<12; i++)
      {if(mask & xdim) xid_siz = i;
       if(mask & ydim) yid_siz = i;
       if(mask & zdim) zid_siz = i;
       mask = mask<<1;}

      pc_itoa(xdim,buffer);
      strcat(buffer,",");
      pc_itoa(ydim,tbuffer);
      strcat(buffer,tbuffer);
      strcat(buffer,",");
      pc_itoa(zdim,tbuffer);
      strcat(buffer,tbuffer);
      strcat(buffer," ");
      pc_itoa(xid_siz,tbuffer);
      strcat(buffer,tbuffer);
      strcat(buffer,",");
      pc_itoa(yid_siz,tbuffer);
      strcat(buffer,tbuffer);
      strcat(buffer,",");
      pc_itoa(zid_siz,tbuffer);
      strcat(buffer,tbuffer);
      print_opcode("shape_", T_SHAPE, NULL, NULL);
      print_args(buffer, NULL, NULL, NULL, NULL);

  if (debug == 2)
      printf("shape() returning x,y,zdim=%d %d %d, x,y,zid_siz=%d %d %d.\n",
              xdim,ydim,zdim,xid_siz,yid_siz,zid_siz);

   return(return_value);
}
