/* 910415  P1_STATE.C */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_scanner.h"
#include "p1_parser.h"

#define C_DASH 45 /* ascii - */
#define T_BAR 124 /* ascii | */
#define T_TIMES 88 /* ascii X */
#define T_EQUAL 61 /* ascii = */
/* ===========================================================================

   function name : state_???
   input                : ---
   output            : ---
   return value  : returns the token type if it is a final state, otherwise
			     FAIL
   global vars    : token_pnt - pointer to buffer containing token

   description   : The following routines form the state transition network
			     for the legal tokens recognized by the scanner. The
			     last final state successfully reached determines the
			     type of token found.

=========================================================================== */

state_1(num_chars)      /* recognizes T_ID */
int num_chars;                  /* number of characters seen so far */
{
   int type;
   char ch;

   ch = getnextchar();
   if((type = ace_mult(ch)) != FAIL) return(type);
   if (char_type(ch) == ALPHA || char_type(ch) == NUMERIC)
     {if ((num_chars + 1) < TOKEN_SIZE)
	      *token_pnt++ = ch;
      type = state_1(++num_chars);}
   else
     {if (num_chars > (TOKEN_SIZE - 1))
	{*token_pnt = EOB;
	      print_msg(TOKSIZ_ERR, token_buf);}
      putbackchar();
      type = T_ID;
     }
   return (type);

} /* end of state_1() */


/*-------------------------------------------------------------------------*/
state_2(num_chars)              /* recognizes T_ID or T_HEX_CONST or X for ACE */
int num_chars;                  /* number of characters seen so far */
{
  int type;
  char ch;

  if(code_type == EMULATE && ace_type == ACE)
    {type = T_MULT;
     return(type);}

  ch = getnextchar();
  switch (ch)
    {case '\'' : /* Check for single quote */
		 token_pnt--; /* erase X */
		 if ((type = state_3(++num_chars)) == FAIL)
			     {token_pnt--;
		    printf("%d ERROR - illegal characters in hexadecimal string\n",linenum);
			      type = T_ID;}
			   break;
     default   : if((type = ace_mult(ch)) != FAIL) return(type);
                 if (char_type(ch) == ALPHA || char_type(ch) == NUMERIC)
			     {*token_pnt++ = ch;
			      type = state_1(++num_chars);}
			   else
			     {putbackchar();
			      type = T_ID;}
    }
  return (type);

}  /* end of state_2() */


/*-------------------------------------------------------------------------*/
state_3(num_chars)              /* part of T_HEX_CONST */
int num_chars;                  /* number of characters seen so far */
{
  int type;
  char ch;

  ch = getnextchar();
  if (hex(ch) || char_type(ch) == NUMERIC)
    {*token_pnt++ = ch;
     if ((type = state_4(++num_chars)) == FAIL)
       {putbackchar();
	token_pnt--;}
    }
  else
    {putbackchar();
     type = FAIL;}
  return (type);

} /* end of state_3() */


/*-------------------------------------------------------------------------*/
state_4(num_chars)              /* final state for T_HEX_CONST */
int num_chars;                  /* number of characters seen so far */
{
  int type;
  char ch;

  ch = getnextchar();
  switch (ch)
   {case '\'': if ((num_chars + 1) < TOKEN_SIZE)
			  /*  *token_pnt++ = ch */ ;
		    else
		      {--token_pnt;
			    *token_pnt++ = ch;
			    *token_pnt = EOB;
			    print_msg(TOKSIZ_ERR, token_buf);}
		    type = T_HEX_CONST;
		    break;
    default  : if (hex(ch) || char_type(ch) == NUMERIC)
		 {if ((num_chars + 1) < TOKEN_SIZE)
		    *token_pnt++  = ch;
		  if ((type = state_4(++num_chars)) == FAIL)
		    {putbackchar();
			       if (num_chars < TOKEN_SIZE)
		       token_pnt--;
		    }
		 }
	       else
		 {putbackchar();
		  type = FAIL;}
    }
  return (type);

} /* end of state_4() */


/*-------------------------------------------------------------------------*/
state_5(num_chars)              /* recognizes T_ID or T_BIN_CONST */
int num_chars;                  /* number of characters seen so far */
{
  int type;
  char ch;

  ch = getnextchar();
  switch (ch)
    {case '\'' : token_pnt--; /* erase B */
		 if ((type = state_6(++num_chars)) == FAIL)
			     {token_pnt--;
		    printf("%d ERROR - illegal characters in binary string\n",linenum);
			      type = T_ID;}
			   break;
     default   : if((type = ace_mult(ch)) != FAIL) return(type);
                 if (char_type(ch) == ALPHA || char_type(ch) == NUMERIC)
			     {*token_pnt++ = ch;
			      type = state_1(++num_chars);}
			   else
			     {putbackchar();
			      type = T_ID;}
    }
  return (type);

}  /* end of state_5() */


/*------------------------------------------------------------------------*/
state_6(num_chars)      /* part of T_BIN_CONST */
int num_chars;                  /* number of characters seen so far */
{
  int type;
  char ch;

  ch = getnextchar();
  switch (ch)
    {case '0' :
     case '1' : *token_pnt++ = ch;
		if ((type = state_7(++num_chars)) == FAIL)
		  {putbackchar();
		   token_pnt--;}
			  break;
     default  : putbackchar();
		type = FAIL;
    }
  return (type);

} /* end of state_6() */


/*-------------------------------------------------------------------------*/
state_7(num_chars)      /* final state for T_BIN_CONST */
int num_chars;                  /* number of characters seen so far */
{
  int type;
  char ch;

  ch = getnextchar();
  switch (ch)
   {case '\'': if ((num_chars + 1) < TOKEN_SIZE)
		 ;
		    else
		      {--token_pnt;
			    *token_pnt++ = ch;
			    *token_pnt = EOB;
			    print_msg(TOKSIZ_ERR,token_buf);}
		    type = T_BIN_CONST;
		    break;
    case '0' :
    case '1' : if ((num_chars + 1) < TOKEN_SIZE)
			   *token_pnt++ = ch;
		    if ((type = state_7(++num_chars)) == FAIL)
		 {putbackchar();
						if (num_chars < TOKEN_SIZE)
						  token_pnt--;}
		    break;
    default  : putbackchar();
		    type = FAIL;
  }
  return (type);

} /* end of state_7() */


/*-------------------------------------------------------------------------*/
state_8(num_chars)      /* recognizes T_ID or T_OCT_CONST */
int num_chars;                  /* number of characters seen so far */
{
  int type;
  char ch;

  ch = getnextchar();
  switch (ch)
    {case '\'' : token_pnt--; /* erase O */
		 if ((type = state_9(++num_chars)) == FAIL)
			     {token_pnt--;
		    printf("%d ERROR - illegal characters in octal string\n",linenum);
			      type = T_ID;}
			   break;
     default   : if((type = ace_mult(ch)) != FAIL) return(type);
                 if (char_type(ch) == ALPHA || char_type(ch) == NUMERIC)
			     {*token_pnt++ = ch;
			      type = state_1(++num_chars);}
			   else
			     {putbackchar();
			      type = T_ID;}
     }
   return (type);

}  /* end of state_8() */


/*--------------------------------------------------------------------------*/
state_9(num_chars)      /* part of T_OCT_CONST */
int num_chars;                  /* number of characters seen so far */
{
  int type;
  char ch;

  ch = getnextchar();
  if (char_type(ch) == NUMERIC && octal(ch))
    {*token_pnt++ = ch;
     if ((type = state_10(++num_chars)) == FAIL)
       {putbackchar();
	token_pnt--;}
    }
  else
    {putbackchar();
     type = FAIL;}
  return (type);

} /* end of state_9() */


/*-------------------------------------------------------------------------*/
state_10(num_chars)     /* final state for T_OCT_CONST */
int num_chars;                  /* number of characters seen so far */
{
  int type;
  char ch;

  ch = getnextchar();
  switch (ch)
   {case '\'': if ((num_chars + 1) < TOKEN_SIZE)
			   ;
		    else
		      {--token_pnt;
			    *token_pnt++ = ch;
			    *token_pnt = EOB;
			    print_msg(TOKSIZ_ERR, token_buf);}
		    type = T_OCT_CONST;
		    break;
    default  : if (char_type(ch) == NUMERIC && octal(ch))
		 {if ((num_chars + 1) < TOKEN_SIZE)
			      *token_pnt++ = ch;
		       if ((type = state_10(++num_chars)) == FAIL)
			      {putbackchar();
			       if (num_chars < TOKEN_SIZE)
				 token_pnt--;}
		      }
		    else
		      {putbackchar();
			    type = FAIL;}
    }
 return (type);

} /* end of state_10() */


/*-------------------------------------------------------------------------*/
state_11(num_chars)             /* recognizes T_DEC_CONST */
int num_chars;                  /* number of characters seen so far */
{
   int type;
   char ch;

   ch = getnextchar();
   if (char_type(ch) == NUMERIC)
     {if ((num_chars + 1) < TOKEN_SIZE)
	*token_pnt++ = ch;
      type = state_11(++num_chars);}
   else if (ch == '.')
     {if ((num_chars + 1) < TOKEN_SIZE)
	*token_pnt++ = ch;
      type = state_40(++num_chars);}
   else
     {if (num_chars > (TOKEN_SIZE - 1))
	{*token_pnt = EOB;
	      print_msg(TOKSIZ_ERR, token_buf);}
      putbackchar();
      type = T_DEC_CONST;}
   return (type);

} /* end of state_11() */


/*-------------------------------------------------------------------------*/
state_12()                      /* recognizes T_XOR */
{
   int type;
   char ch;

   ch = getnextchar();
   if (ch == '^')
     {*token_pnt++ = ch;
      type = T_XOR;}
   else
     {putbackchar();
      type = T_UNKNOWN;}
   return(type);

} /* end of state_12() */


/*-------------------------------------------------------------------------*/
state_13()                      /* recognizes T_NOT or T_NE */
{
   int type;
   char ch;

   ch = getnextchar();
   if (ch == '=')
     {*token_pnt++ = ch;
      type = T_NE;}
   else
     {putbackchar();
      type = T_NOT;}
   return (type);

} /* end of state_13() */


/*-------------------------------------------------------------------------*/
state_14()                      /* recognizes T_AND */
{
   int type;
   char ch;

   ch = getnextchar();
   if (ch == '&')
     {*token_pnt++ = ch;
      type = T_AND;}
   else
     {putbackchar();
      type = T_UNKNOWN;}
   return (type);

} /* end of state_14() */


/*-------------------------------------------------------------------------*/
state_15()                      /* recognizes T_LT or T_LE */
{
   int type;
   char ch;

   ch = getnextchar();
   if (ch == '=')
     {*token_pnt++ = ch;
      type = T_LE;}
   else
     {putbackchar();
      type = T_LT;}
   return (type);

} /* end of state_15() */


/*--------------------------------------------------------------------------*/
state_16()                      /* recognizes T_GT or T_GE */
{
   int type;
   char ch;

   ch = getnextchar();
   if (ch == '=')
     {*token_pnt++ = ch;
      type = T_GE;}
   else
     {putbackchar();
      type = T_GT;}
   return (type);

} /* end of state_16() */


/*--------------------------------------------------------------------------*/
state_17()                      /* recognizes T_ASSIGN or T_EQ */
{
   int type;
   char ch;

   ch = getnextchar();
   if (ch == '=')
     {*token_pnt++ = ch;
      type = T_EQ;}
   else
     {putbackchar();
      type = T_ASSIGN;}
   return (type);

} /* end of state_17() */


/*--------------------------------------------------------------------------*/
state_18()                      /* recognizes T_OR || */
{
   int type;
   char ch;

   ch = getnextchar();
   if (ch == '|')
     {*token_pnt++ = ch;
      type = T_OR;}
   else
		{type = T_BAR;
		 putbackchar();}
   return (type);

} /* end of state_18() */


/*--------------------------------------------------------------------------*/
state_19()                      /* recognizes alternate relational keywords */
{
   int type;
   char ch;

   ch = getnextchar();
   switch (ch)
     {case 'A' : *token_pnt++ = ch;
		 type = state_20();
		 break;
      case 'E' : *token_pnt++ = ch;
		 type = state_23();
		 break;
      case 'G' : *token_pnt++ = ch;
		 type = state_25();
		 break;
      case 'L' : *token_pnt++ = ch;
		 type = state_28();
		 break;
      case 'N' : *token_pnt++ = ch;
		 type = state_31();
		 break;
      case 'O' : *token_pnt++ = ch;
		 type = state_35();
		 break;
      case 'X' : *token_pnt++ = ch;
		 type = state_37();
		 break;
      default  : if (char_type(ch) == NUMERIC)    /* DECIMAL NUMBER */
		    {*token_pnt++ = ch;
		     type = state_40(1);}
		 else
		   {putbackchar();
		    type = FAIL;}
     }
   return (type);

} /* end of state_19() */

/*--------------------------------------------------------------------------*/
state_20()              /* recognizes T_AND */
{
   int type;
   char ch;

   ch = getnextchar();
   switch (ch)
     {case 'N' : *token_pnt++ = ch;
		 if ((type = state_21()) == FAIL)
		   {putbackchar();
		    token_pnt--;}
			break;
      default  : putbackchar();
		 type = FAIL;
     }
   return (type);

} /* end of state_20() */


/*--------------------------------------------------------------------------*/
state_21()                      /* part of T_AND */
{
   int type;
   char ch;

   ch = getnextchar();
   switch (ch)
     {case 'D' : *token_pnt++ = ch;
		 if ((type = state_22()) == FAIL)
		   {putbackchar();
		    token_pnt--;}
			break;
      default  : putbackchar();
		 type = FAIL;
     }
   return (type);

} /* end of state_21() */


/*--------------------------------------------------------------------------*/
state_22()                      /* final state for T_AND */
{
   int type;
   char ch;

   ch = getnextchar();
   if (ch == '.')
      type = T_AND;
   else
     {putbackchar();
      type = FAIL;}
   return (type);

} /* end of state_22() */


/*--------------------------------------------------------------------------*/
state_23()                      /* recognizes T_EQ */
{
   int type;
   char ch;

   ch = getnextchar();
   switch (ch)
     {case 'Q' : *token_pnt++ = ch;
		 if ((type = state_24()) == FAIL)
		   {putbackchar();
		    token_pnt--;}
			   break;
      default  : putbackchar();
		 type = FAIL;
     }
   return (type);

} /* end of state_23() */


/*--------------------------------------------------------------------------*/
state_24()                      /* final state for T_EQ */
{
   int type;
   char ch;

   ch = getnextchar();
   if (ch == '.')
      type = T_EQ;
   else
     {putbackchar();
      type = FAIL;}
   return (type);

} /* end of state_24() */


/*--------------------------------------------------------------------------*/
state_25()                      /* recognizes T_GE or T_GT */
{
   int type;
   char ch;

   ch = getnextchar();
   switch (ch)
     {case 'E' : *token_pnt++ = ch;
		 if ((type = state_26()) == FAIL)
		   {putbackchar();
		    token_pnt--;}
			   break;
      case 'T' : *token_pnt++ = ch;
		 if ((type = state_27()) == FAIL)
		   {putbackchar();
		    token_pnt--;}
			break;
      default  : putbackchar();
		 type = FAIL;
     }
   return (type);

} /* end of state_25() */


/*--------------------------------------------------------------------------*/
state_26()                      /* final state for T_GE */
{
   int type;
   char ch;

   ch = getnextchar();
   if (ch == '.')
      type = T_GE;
   else
     {putbackchar();
      type = FAIL;}
   return (type);

} /* end of state_26() */


/*--------------------------------------------------------------------------*/
state_27()              /* final state for T_GT */
{
   int type;
   char ch;

   ch = getnextchar();
   if (ch == '.')
      type = T_GT;
   else
     {putbackchar();
      type = FAIL;}
   return (type);

} /* end of state_27() */


/*---------------------------------------------------------------------------*/
state_28()                      /* recognizes T_LE or T_LT */
{
   int type;
   char ch;

   ch = getnextchar();
   switch (ch)
     {case 'E' : *token_pnt++ = ch;
		 if ((type = state_29()) == FAIL)
		   {putbackchar();
		    token_pnt--;}
			   break;
      case 'T' : *token_pnt++ = ch;
		 if ((type = state_30()) == FAIL)
		   {putbackchar();
		    token_pnt--;}
			   break;
      default  : putbackchar();
		 type = FAIL;
     }
   return (type);

} /* end of state_28() */


/*--------------------------------------------------------------------------*/
state_29()                      /* final state for T_LE */
{
   int type;
   char ch;

   ch = getnextchar();
   if (ch == '.')
      type = T_LE;
   else
     {putbackchar();
      type = FAIL;}
   return (type);

} /* end of state_29() */


/*---------------------------------------------------------------------------*/

state_30()                      /* final state for T_LT */
{
   int type;
   char ch;

   ch = getnextchar();
   if (ch == '.')
      type = T_LT;
   else
     {putbackchar();
      type = FAIL;}
   return (type);

} /* end of state_30() */


/*---------------------------------------------------------------------------*/
state_31()                      /* recognizes T_NE or T_NOT */
{
   int type;
   char ch;

   ch = getnextchar();
   switch (ch)
     {case 'E' : *token_pnt++ = ch;
		 if ((type = state_32()) == FAIL)
		   {putbackchar();
		    token_pnt--;}
			   break;
      case 'O' : *token_pnt++ = ch;
		 if ((type = state_33()) == FAIL)
		   {putbackchar();
		    token_pnt--;}
		 break;
      default  : putbackchar();
		 type = FAIL;
     }
   return (type);

} /* end of state_31() */


/*---------------------------------------------------------------------------*/
state_32()                      /* final state for T_NE */
{
   int type;
   char ch;

   ch = getnextchar();
   if (ch == '.')
      type = T_NE;
   else
     {putbackchar();
      type = FAIL;}
   return (type);

} /* end of state_32() */


/*--------------------------------------------------------------------------*/
state_33()                      /* part of T_NOT */
{
   int type;
   char ch;

   ch = getnextchar();
   switch (ch)
     {case 'T' : *token_pnt++ = ch;
		 if ((type = state_34()) == FAIL)
		   {putbackchar();
		    token_pnt--;}
			   break;
      default  : putbackchar();
		 type = FAIL;
     }
   return (type);

} /* end of state_33() */


/*---------------------------------------------------------------------------*/
state_34()                      /* final state for T_NOT */
{
   int type;
   char ch;

   ch = getnextchar();
   if (ch == '.')
      type = T_NOT;
   else
     {putbackchar();
      type = FAIL;}
   return (type);

} /* end of state_34() */


/*---------------------------------------------------------------------------*/
state_35()                      /* recognizes T_OR */
{
   int type;
   char ch;

   ch = getnextchar();
   switch (ch)
     {case 'R' : *token_pnt++ = ch;
		 if ((type = state_36()) == FAIL)
		   {putbackchar();
		    token_pnt--;}
			   break;
      default  : putbackchar();
		 type = FAIL;
     }
   return (type);

} /* end of state_35() */


/*---------------------------------------------------------------------------*/
state_36()                      /* final state for T_OR */
{
   int type;
   char ch;

   ch = getnextchar();
   if (ch == '.')
      type = T_OR;
   else
     {putbackchar();
      type = FAIL;}
   return (type);

} /* end of state_36() */


/*---------------------------------------------------------------------------*/
state_37()                      /* recognizes T_XOR */
{
   int type;
   char ch;

   ch = getnextchar();
   switch (ch)
     {case 'O' : *token_pnt++ = ch;
		 if ((type = state_38()) == FAIL)
		   {putbackchar();
		    token_pnt--;}
			   break;
      default  : putbackchar();
		 type = FAIL;
     }
   return (type);

} /* end of state_37() */


/*--------------------------------------------------------------------------*/
state_38()                      /* part of T_XOR */
{
   int type;
   char ch;

   ch = getnextchar();
   switch (ch)
     {case 'R' : *token_pnt++ = ch;
		 if ((type = state_39()) == FAIL)
		   {putbackchar();
		    token_pnt--;}
			   break;
      default  : putbackchar();
		 type = FAIL;
     }
   return (type);

} /* end of state_38() */


/*---------------------------------------------------------------------------*/
state_39()                      /* final state for T_XOR */
{
   int type;
   char ch;

   ch = getnextchar();
   if (ch == '.')
      type = T_XOR;
   else
     {putbackchar();
      type = FAIL;}
   return (type);

} /* end of state_39() */

/*-----------------------------------------------------------------------*/
state_40(num_chars)  /* floating dec constant */
int num_chars;                  /* number of characters seen so far */
{
   int type;
   char ch;

   ch = getnextchar();
   if (char_type(ch) == NUMERIC)
     {if ((num_chars + 1) < TOKEN_SIZE)
	*token_pnt++ = ch;
      type = state_40(++num_chars);}
   else
     {if (num_chars > (TOKEN_SIZE - 1))
	{*token_pnt = EOB;
	      print_msg(TOKSIZ_ERR, token_buf);}
      putbackchar();
      type = T_FLT_CONST;}
   return (type);

} /* end of state_11() */


/*---------------------------------------------------------------------------*/
state_41()                      /* recognizes * and ** */
{
   int type;
   char ch;

   ch = getnextchar();
   switch (ch)
     {case '*' : *token_pnt++ = ch;
		 type = T_POWER;
		   break;
      default  : putbackchar();
		 type = T_MULT;
     }
   return (type);

} /* end of state_41() */

/* processes X for ACE as in rateXtime */
ace_mult(ch)
char ch;
{  int type;

   if(ch == 'X')
     if(code_type == EMULATE && ace_type == ACE)
        {*token_pnt = EOB;
         putbackchar();
         type = T_ID;
         return(type);
        }
   return FAIL;
}
