/* 910415  P1_SUBSCRIPT.C */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"

/* NOTE: Subscript() is for the right hand side only, it automatically
	 reduces multiple indicies to one set by generating indexed gets.
	      Assign() and parameters() call subsubroutine directly */

int cube_moddex, cube_modoff, cube_rotate;
int array_size;
char dim_width[20];

subscript(in_dx,out_dx,out_os)
int in_dx,*out_dx,*out_os;
{
  int flag,    /* FALSE      if in_dx        */
	       /* TRUE       if in_dx[ ...]  */
      type,    /* T_PARALLEL if in_dx[$,...] or in_dx$*/
	       /* if in_dx[$+ex,...] cube_flag = TRUE *//* added 880221 jlp */
	       /* T_SCALAR   if in_dx[ ...]  */
	       /* T_MASKEDMOVE  if in_dx[log_exp(),...] */
      log_dx,  /* in_dx[log_dx[log_os],os_dx[os_os]] */
      log_os,
      os_dx,
      os_os,
      scalar_dx; /*temp to hold value of indexed get, i.e. log_dx and log_os */
  int mode_one, size_one; /* temp variable for mode1 */
  int outbits,outbits_dx;
  char offset[32];
/*  NOTE a SUCCESS offset will be interpreted as a nil offset */

  if (debug == 2)
    printf("Entering subscript, id = %s\n",symbol_table[in_dx].tokpts);

  *out_dx = in_dx; /* set outputs to default case */
  *out_os = SUCCESS;
  /* This call casues cube_index to be set if $+cube_index is present */
  if(subsubscript(in_dx,&type,&flag,&log_dx,&log_os,&os_dx,&os_os) == FAIL )
     return (FAIL);
  /* reduce subsubscript results to a single dx,os pair */
  if(flag)
/*a*/{if(os_dx != SUCCESS)
/*b*/  {if(os_os != SUCCESS) /* SUCCESS means nil */
/*c*/     {mode_one = symbol_table[os_dx].mode1;
		if((scalar_dx = get_symtab_temp(mode_one,T_SCALAR,T_NONE))== FAIL)
				 {print_msg(TEMP_ERR, "In subsc 1"); return(FAIL); }
	   else  /* put indexed offset into a scalar */
		  {print_opcode ("mvsc_",T_MVSC,mode_one,T_SCALAR);
		   print_args (off_exp1(os_dx,os_os),
			  off_exp2(scalar_dx,SUCCESS),NULL,NULL,NULL);
		   *out_dx = scalar_dx;
		   *out_os = SUCCESS;}
/*c*/     }
/*b*/   else
	  *out_os = os_dx;} /* simple reduction because no os_os */
     if (type == T_MASKEDMOVE)
/*d*/ {mode_one = symbol_table[in_dx].mode1;
       size_one = symbol_table[in_dx].width;
       if ((scalar_dx = get_symtab_temp(mode_one,T_REG_SCALAR,size_one))== FAIL)
	 {print_msg(TEMP_ERR, "In subsc 2");
	       return(FAIL); }
       else    /*indexed get, ie. scalar_dx=os_dx[log_dx[log_os],os_os] */
	  /* g_lhs_dx is set in p1_assign.h.  If it is T_SCALAR, p[xx] will be
	     left as a scalar, otherwise it is promoted to a parallel
	     g_assign is set to TRUE when inside p1_assign.c            */
/*e*/    {outbits = symbol_table[in_dx].width;
	       pc_itoa(outbits,offset);
	       outbits_dx = get_const(offset,T_DEC_CONST);
	       print_opcode ("mvpa_",T_MVPA,mode_one,T_REG_SCALAR);
	  print_args (off_exp1(*out_dx,*out_os),
		      off_exp2(scalar_dx,SUCCESS),
				off_exp3(log_dx,log_os),
		      off_exp4(outbits_dx,SUCCESS), NULL);
	  save_the(scalar_dx,SUCCESS,*out_dx,*out_os);
	  *out_dx = scalar_dx;
/*e*/       *out_os = SUCCESS;}
/*d*/ }
  /* cube_flag set in subsubscript */
  if(type == T_PARALLEL && cube_flag == TRUE &&
	  (code_type == CM || code_type == EMULATE))
	  genmvcb(out_dx,out_os);
/*a*/}
 return(SUCCESS);
}

/* ===========================================================================

   function name: subsubscript
   input               : index  - symbol table index for subscripted variable
   output           : type  - T_SCALAR or T_PARALLEL (i.e. [&,...]) or
			  T_MASKEDMOVE (i.e. [log_exp,...]) - type2.
		: flag  - FALSE if no subscript, TRUE otherwise
		: log_index - the index of the logical expression found
   return value : the symbol table index of the subscript offset if flag=TRUE
		: SUCCESS if flag = FALSE

   global vars  : token_type    - type of current token
			    token_buf   - current token string
			    symbol_table        - symbol table data structure
			    debug               - debugging statement flag

   description   : This routine is the driver for processing the multiple
			    dimension variable. It will control other routines to
			    get the offset of the index subscript.

  if [$,offset_index[offset_offset]  - type = T_PARALLEL
  if [log_index[log_offset],offset_index[offset_offset]] - type = T_MASKEDMOVE
=========================================================================== */
subsubscript(index,type,flag,log_index,log_offset,offset_index,offset_offset)
int index,*type,*flag,*log_index,*log_offset,*offset_index,*offset_offset;
{
   int count,size_index,temp_index,c_flag;
   char awidth[20];

/* c_flag is saved on a stack */
if(debug == 2) printf("In subsubscript()...................\n");
*type = T_SCALAR; /* parameters to default case */
*flag = FALSE;
*log_index = FAIL;
*log_offset = SUCCESS;
*offset_index = SUCCESS;
*offset_offset = SUCCESS;
count = 0;
c_flag = FALSE;
if(symbol_table[index].num_dim > 0)/* dimensioned scalars, all parallel */
/*a*/{if(token_type != T_LSQU)
	{if(symbol_table[index].mode2 != T_PARALLEL)
			  {print_msg(LSQU_ERR, token_buf);
printf("token=%s, mode2=%d, num_dim=%d\n",
symbol_table[index].tokpts,
symbol_table[index].mode2,
symbol_table[index].num_dim);
				printf("In p1_subsc\n");
symtab_dump();
				return(FAIL); }
	 else if(strcmp(symbol_table[index].tokpts,"THEM") == 0 ||
				strcmp(symbol_table[index].tokpts,"ITS") == 0 ||
				strcmp(symbol_table[index].tokpts,"THEIR") == 0   )
	   {*type = T_PARALLEL;
	    cube_flag = FALSE;
	    if(debug == 2) printf("Exit subsubscript() - pronoun found.\n");
	     return(SUCCESS);}
	 else if( token_type == T_PAR_SUBSCRIPT)
	   {token_type = get_token();
		 *type = T_PARALLEL;
	    cube_flag = FALSE;
			   if (token_type == T_RSQU)
	      {print_msg(RSQU_ERR, token_buf);
				  return(FAIL); };
	    if(debug == 2) printf("Exit subsubscript() - $ subscript found.\n");
	    return(SUCCESS);}
	 else
	   {print_msg(RSQU_ERR, token_buf);
			  return(FAIL); };
	}
  *flag = TRUE;
  token_type = get_token();
  if (symbol_table[index].mode2 == T_PARALLEL)
/*b*/{count = 1;
      if(token_type == T_PAR_SUBSCRIPT)
/*c*/    {token_type = get_token();
	     *type = T_PARALLEL;
	if(code_type == CM || code_type == EMULATE)  /* look for $+expression or $-expression  */
/*d*/       {if(token_type == T_MINUS) token_type = T_ADD;/* switched to agree with book*/
		if(token_type == T_PLUS) token_type = T_SUB;
	   if(token_type == T_ADD || token_type == T_SUB ||
		   token_type == T_DIV || token_type == T_MULT ||
               token_type == T_POWER || token_type == T_MOD)
/*e*/        {cube_op = token_type;
	      pc_itoa(NOOFP,awidth);
			     cube_moddex = get_const(awidth,T_DEC_CONST);
			     cube_modoff = SUCCESS;
		   token_type = get_token();
		   if(sum_dif_exp(&cube_index,&cube_offset) == FAIL)
		     {print_msg(CUBE_ERR,token_buf); return(FAIL);}
		   else
				    {if(token_type == T_COLON)
					    {token_type = get_token();
					     if(token_type != T_COLON)
						    if(sum_dif_exp(&cube_moddex,&cube_modoff) == FAIL)
						      {print_msg(CUBE_ERR,token_buf);
							    return(FAIL);}
						    if(token_type == T_COLON)
						      {token_type = get_token();
							    if(token_buf[0] == 'R')
							      cube_rotate = get_const("1",T_DEC_CONST);
							    token_type = get_token();}
				       }
		      c_flag = TRUE;
				     cube_rotate = get_const("0",T_DEC_CONST);
			       }
/*e*/          }
/*d*/       }
/*c*/  }
     else   /* look for parallel[log_exp,..] */
/*f*/  {if (log_exp(log_index,log_offset) == FAIL) return(FAIL);
	if (symbol_table[*log_index].mode2 != T_PARALLEL &&
	    symbol_table[*log_index].mode1 != T_MASKEDMOVE)
	   {print_msg(SUBSCR_ERR, token_buf);
	    return(FAIL); };
/*f*/   *type = T_MASKEDMOVE;};
/*b*/if(token_type == T_COMMA) token_type = get_token(); }
/* end of first dimension for mode2==T_PARALLEL */

/* process remainder of subscript */
array_size = symbol_table[index].dim_size[1]; /* init for wt */
if(p_index_add(index,count,offset_index,offset_offset) == FAIL)return(FAIL);
if(*offset_index == SUCCESS)
   if (token_type != T_RSQU)
     {print_msg(RSQU_ERR, token_buf);
	  return(FAIL); }
   else
	  {cube_flag = c_flag;
		token_type = get_token();
		return(SUCCESS);}

/* parallel variable with two or more dimensions */
if(symbol_table[index].mode2 == T_PARALLEL &&
   symbol_table[index].num_dim > 1)
/*g*/{if( wt == WT)
/*k*/  {if((temp_index=get_symtab_temp(T_INT,T_SCALAR,T_NONE)) == FAIL)
	 {print_msg(TEMP_ERR,"In subsc 3"); return(FAIL); };
	array_size--;
	pc_itoa(array_size,dim_width);
	size_index = get_const(dim_width,T_DEC_CONST);
	print_opcode("sub_",T_SUB,T_INT,T_SCALAR);
	print_args(off_exp1(size_index,SUCCESS),
			off_exp2(*offset_index,*offset_offset),
			off_exp3(temp_index,SUCCESS),NULL,NULL);
	*offset_index = temp_index;
/*k*/   *offset_offset = SUCCESS; }
      if((temp_index=get_symtab_temp(T_INT,T_SCALAR,T_NONE)) == FAIL)
       {print_msg(TEMP_ERR,"In subsc 4"); return(FAIL); };
      pc_itoa(symbol_table[index].width,awidth);
      size_index = get_const(awidth,T_DEC_CONST);
      print_opcode("mul_",T_MULT,T_INT,T_SCALAR);
      print_args(off_exp1(size_index,SUCCESS),
		      off_exp2(*offset_index,*offset_offset),
		      off_exp3(temp_index,SUCCESS),NULL,NULL);
      *offset_index = temp_index;
/*g*/ *offset_offset = SUCCESS; }

if (token_type != T_RSQU)
     {print_msg(RSQU_ERR, token_buf);
     return(FAIL); };
token_type = get_token();
cube_flag = c_flag;

if(debug == 2) printf("Exit subsubscript() - subscript found.\n");

return(SUCCESS);
/*a*/ };
/* else {};  scalar - no dimensions */

if(debug == 2) printf("In subsubscript() no subscript found.\n");

cube_flag = c_flag;
return(SUCCESS);
}



/*  p_index_add and p_index_mult are the routines that parse an id
    subscript and generate the code that calculates the offset at run time.
    i.e. arg3_index[arg3_offset]

    If the id is parallel, the count parameter should start at one,
    otherwise it starts at zero.

*/

p_index_add(idex,count,arg3_index,arg3_offset)
int idex,count,*arg3_index,*arg3_offset;
{ int log_index, log_offset, arg2_index,arg2_offset, size;

  if(debug == 2) printf("In p_index_add. idex=%s, count=%d\n",
			symbol_table[idex].tokpts,count);

  *arg3_index = SUCCESS;
  *arg3_offset = SUCCESS;

  if(token_type == T_RSQU)
    {if(debug == 2) printf("Exit p_index_add at T_RSQU with SUCCESS\n");
     return(SUCCESS);}

  if(sum_dif_exp(&log_index,&log_offset)==FAIL)
    {print_msg(SUBSCR_ERR,token_buf); return(FAIL);};

  *arg3_index = log_index;
  *arg3_offset = log_offset;
  if(token_type == T_RSQU) return(SUCCESS);

  if(token_type == T_COMMA)
    {if(count > symbol_table[idex].num_dim)
	    {print_msg(DIM_ERR,symbol_table[idex].tokpts); return(FAIL); };
     size = symbol_table[idex].dim_size[count++];
     array_size *= size;
     if(p_index_mult(idex,size,count,&arg2_index,&arg2_offset) == FAIL)
	    {print_msg(SUBSCR_ERR,"arg2 fail in p_index_add");return(FAIL);};
     if((*arg3_index=get_symtab_temp(T_INT,T_SCALAR,T_NONE)) == FAIL)
	    {print_msg(TEMP_ERR,"In subsc 5");return(FAIL);};
     *arg3_offset = SUCCESS;
     print_opcode("add_",T_ADD,T_INT,T_SCALAR);
     print_args(off_exp1(log_index,log_offset),
			    off_exp2(arg2_index,arg2_offset),
			  off_exp3(*arg3_index,SUCCESS),NULL,NULL);
     return(SUCCESS);
    }
  else {print_msg(SUBSCR_ERR,token_buf); return(FAIL);};
}


p_index_mult(idex,size,count,arg2_index,arg2_offset)
/* generates code for:
      [....,sizedx*arg1_index[arg1_offset],....]     */
int idex,size,count,*arg2_index,*arg2_offset;
{ int log_index, log_offset,sizedx;
  char asize[20];

  if(debug == 2) printf("In p_index_mul.");

  *arg2_index=SUCCESS;
  *arg2_offset=SUCCESS;
  token_type = get_token();
  if(p_index_add(idex,count,&log_index,&log_offset) == FAIL)
    {print_msg(SUBSCR_ERR,"log failure in p_index_mul"); return(FAIL); };
  if(log_index == SUCCESS) return(SUCCESS);
  *arg2_offset = SUCCESS;
  if((*arg2_index=get_symtab_temp(T_INT,T_SCALAR,T_NONE)) == FAIL)
    {print_msg(TEMP_ERR,"In subsc 6"); return(FAIL); };
  pc_itoa(size,asize);
  sizedx = get_const(asize,T_DEC_CONST);
  print_opcode("mul_",T_MULT,T_INT,T_SCALAR);
  print_args(off_exp1(sizedx,SUCCESS),
	     off_exp2(log_index,log_offset),
		  off_exp3(*arg2_index,SUCCESS),NULL,NULL);
  return(SUCCESS);
}


/* genmvcb generates the mvcb(rotate/shift) quadruple in
   responce to a[$+3].  */

genmvcb(out_dx,out_os)
int *out_dx,*out_os;
{
   int cube_add_dx, cube_data_dx, cube_mode2, wid;

   wid=symbol_table[cube_index].width;
   if(code_type == CM)
     if((cube_add_dx = get_symtab_temp(T_INT,T_PARALLEL,wid)) == FAIL)
       {print_msg(TEMP_ERR,"In subsc 7"); return(FAIL);}
     if(code_type == EMULATE)
       if((cube_add_dx = get_symtab_temp(T_INT,T_SCALAR,wid)) == FAIL)
	 {print_msg(TEMP_ERR,"In subsc 8"); return(FAIL);}
     if(symbol_table[cube_index].mode2 == T_PARALLEL)
	cube_mode2 = T_PARALLEL;
	  else cube_mode2 = T_COMMON_RIGHT;
     if(cube_op == T_ADD)
       {if(code_type == CM)
	   print_opcode("add_",T_ADD,T_INT,cube_mode2);
	if(code_type == EMULATE)
	 if(wt == WT)
	   print_opcode("sub_",T_SUB,T_INT,T_SCALAR);
	 else
	   print_opcode("add_",T_ADD,T_INT,T_SCALAR);}
     if(cube_op == T_SUB)
       {if(code_type == CM)
	   print_opcode("sub_",T_SUB,T_INT,cube_mode2);
	if(code_type == EMULATE)
	  if(wt == WT)
				  print_opcode("add_",T_ADD,T_INT,T_SCALAR);
			 else
	     print_opcode("sub_",T_SUB,T_INT,T_SCALAR);}
     if(cube_op == T_DIV)
	if(code_type == CM)
	   print_opcode("div_",T_DIV,T_INT,cube_mode2);
     /* T_MOD implemented on emulator only */
     if(cube_op ==T_MULT)
	if(code_type == CM)
	   print_opcode("mult_",T_MULT,T_INT,cube_mode2);
     if(code_type == CM)
	print_args("PEID",off_exp1(cube_index,cube_offset),
			  off_exp2(cube_add_dx,SUCCESS),NULL,NULL);
     if(code_type == EMULATE)
       if(wt == WT)
	print_args("C0D$",off_exp1(cube_index,cube_offset),
			  off_exp2(cube_add_dx,SUCCESS),NULL,NULL);
      else
       print_args("C0D$",off_exp1(cube_index,cube_offset),
			  off_exp2(cube_add_dx,SUCCESS),NULL,NULL);
     if((cube_data_dx = get_symtab_temp(symbol_table[*out_dx].mode1,
					symbol_table[*out_dx].mode2,
					     symbol_table[*out_dx].width))==FAIL)
       {print_msg(TEMP_ERR,"In subsc 9"); return(FAIL);}
     print_opcode("mvcb_",T_MVCB,symbol_table[*out_dx].mode1,
				 symbol_table[*out_dx].mode2);
     if(code_type == CM)
	print_args(off_exp1(*out_dx,*out_os),
		   off_exp2(cube_data_dx,SUCCESS),
		   off_exp3(cube_add_dx,SUCCESS),NULL,NULL);
     else if(code_type == EMULATE)
	print_args(off_exp1(*out_dx,*out_os),
		   off_exp2(cube_data_dx,SUCCESS),
		   off_exp3(cube_add_dx,SUCCESS),
		   off_exp4(cube_moddex,cube_modoff),
		   symbol_table[cube_rotate].tokpts);
     *out_dx = cube_data_dx;
     *out_os = SUCCESS;
     cube_flag = FALSE;
    return 0;
}
