/* 910415  P1_SUMDIFFERENCE.C */

#include "stdarg.h"
#include "stdio.h"
#include "ctype.h"
#include "string.h"
#include "alloc.h"
#include "p1_common.h"
#include "p1_parser.h"
#include "p1_scanner.h"

char *off_exp1();
char *off_exp2();
char *off_exp3();
char *off_exp4();

/* ===========================================================================

   function name: sum_dif_exp
   input               : ---
   output           : result_index, result_offset;
		: result_index  - symbol table index of result, if no errors
						     - FAIL, otherwise

   global vars  : debug         - flag for debugging statements

   description   : This routine is part of a recursive descent parser and
			    handles the following production from the grammar:

			SUM_DIF_EXP <-- PROD_QUOT_EXP MORE_SUM_DIF_EXP

		  If an argument (resulting from a possible product/quotient
		  expression) is obtained, it is passed to 'more_sum_dif_exp'
		  to perform a possible addition or subtraction. If the first
		  argument cannot be obtained, FAIL is returned.

=========================================================================== */

int sum_dif_exp(result_index,result_offset)
int *result_index,*result_offset;
{

 if (debug == 2)
      {printf ("In sum_dif_exp()...............................\n");
       printf("In sum_dif_exp, token_buf=%s, token_type=%d\n",token_buf,token_type);}

   if (prod_quot_exp(result_index,result_offset) == FAIL) return(FAIL);
   if (more_sum_dif_exp(result_index,result_offset) == FAIL) return(FAIL);

   if (debug == 2)
      {printf ("line %d End sum_dif_exp()...............................\n",
		linenum);
       printf ("result_index=%s, result_offset=",
	       symbol_table[*result_index].tokpts);
       if(*result_offset == SUCCESS) printf("SUCCESS\n");
       else printf("%s\n\n", symbol_table[*result_offset].tokpts);}

   return(SUCCESS);

}

/* ===========================================================================

   function name: more_sum_dif_exp
   input               : arg1_index     - symbol table index of argument 1
   output           : result_index, result_offset
		: result_index  - symbol table index of result, if no errors
						     - FAIL, otherwise

   global vars  : token_type    - type of current token
			    symbol_table        - array containing symbol table
			    debug               - flag for debugging statements

   description   : This routine is part of a recursive descent parser and
			    handles the following production from the gammar:

			MORE_SUM_DIF_EXP <-- + PROD_QUOT_EXP MORE_SUM_DIF_EXP |
						       - PROD_QUOT_EXP MORE_SUM_DIF_EXP |
						       @

		  If an add or subtract sign is found, the symbol table index
		  of the second argument is obtained. Intermediate code is
		  generated and the symbol table index of the result is passed
		  in a recursive call to this function to handle any other
		  additons or subtractions. The symbol table index of the
		  final result is returned. If no add or subtract sign is
		  found, the function returns the first argument as it was
		  received. NOTE: Type-checking is done to ensure that the
		  operands for addition and subtraction are type integer or
		  real--conversions are done between these two types if needed.

=========================================================================== */

int more_sum_dif_exp(arg1_index,arg1_offset)
int *arg1_index,*arg1_offset;
{
   int op_mode1;                /* operation mode: int or real */
   int op_mode2;                /* operation mode: scalar, parallel, common */
   int op_type;         /* operation type: add or subtract */
   int result_mode1;    /* result mode: int or real */
   int result_mode2;    /* result mode: scalar or parallel */
   int result_size;  /* size of output field */
   int arg2_index;      /* symbol table index of second argument */
   int arg2_offset;
   int arg3_index;
   int size1, size2, temp_dx, temp2_dx, num_dx;
   char offset[10];
   int foo_flag; /* secondary flag to allow 2 1/8 to be parsed when table_flag is true */

   if (debug == 2)
    {printf ("In more_sum_dif_exp() at line %d...................\n",linenum);
     printf ("arg1_index=%s, arg1_offset=",symbol_table[*arg1_index].tokpts);
     if(*arg1_offset == SUCCESS) printf("SUCCESS\n");
     else printf("%s\n", symbol_table[*arg1_offset].tokpts);
     printf ("token_type=%x, token_buf=%s\n",token_type,token_buf);  }

    foo_flag = FALSE;
    if(token_type == T_FLT_CONST && table_parse == TRUE)
      {token_type = T_ADD;
       foo_flag = TRUE;}

   if (token_type == T_ADD || token_type == T_SUB)
     {if (symbol_table[*arg1_index].mode1 != T_REAL &&
	       symbol_table[*arg1_index].mode1 != T_OCT  &&
	       symbol_table[*arg1_index].mode1 != T_HEX  &&
	       symbol_table[*arg1_index].mode1 != T_BIN  &&
	       symbol_table[*arg1_index].mode1 != T_CHAR &&
	       symbol_table[*arg1_index].mode1 != T_CARD &&
	       symbol_table[*arg1_index].mode1 != T_LONGINT &&
	       symbol_table[*arg1_index].mode1 != T_INT)
	     {print_msg(EXPCON_ERR, token_buf);
	 printf("In sumdi, token=%s, mode1=%d\n",symbol_table[*arg1_index].tokpts,
		 symbol_table[*arg1_index].mode1);
			return(FAIL);}
      op_type = token_type;
      /* if foo_lag is TRUE, in TABLE_PARSE, and 5 1/8 in countered */
      /* supply implied T_ADD, do not get_token */
      /* otherwise do get_token */
      if(foo_flag == TRUE) token_type = T_FLT_CONST;
      else token_type = get_token();
      if (prod_quot_exp(&arg2_index,&arg2_offset) == FAIL) return(FAIL);
      if (symbol_table[arg2_index].mode1 != T_REAL &&
	       symbol_table[arg2_index].mode1 != T_OCT  &&
	       symbol_table[arg2_index].mode1 != T_HEX  &&
	       symbol_table[arg2_index].mode1 != T_BIN  &&
	       symbol_table[arg2_index].mode1 != T_CARD &&
	       symbol_table[arg2_index].mode1 != T_CHAR &&
	       symbol_table[arg2_index].mode1 != T_LONGINT &&
	       symbol_table[arg2_index].mode1 != T_INT)
	     {print_msg (EXPCON_ERR, token_buf);
	      printf("In sumdi 2\n");
			return(FAIL); }
      op_mode1 = get_op_mode1(arg1_index,arg1_offset,&arg2_index,&arg2_offset);
      op_mode2 = get_op_mode2 (*arg1_index, arg2_index);
      if(op_mode2 == T_COMMON_LEFT)
	{size1 = symbol_table[*arg1_index].width;
	size2 = symbol_table[arg2_index].width;
	if(size1 < size2)
	  {temp_dx = get_symtab_temp(symbol_table[*arg1_index].mode1,
										  T_PARALLEL,size2);
	    pc_itoa(size2,offset);
				num_dx = get_const(offset,T_DEC_CONST);
	    print_opcode("mvsc_",T_MVSC,symbol_table[*arg1_index].mode1,T_PARALLEL);
		print_args(off_exp1(*arg1_index,*arg1_offset), off_exp2(temp_dx,SUCCESS),
		       off_exp3(num_dx,SUCCESS),NULL,NULL);
		op_mode2 = T_PARALLEL;
		*arg1_index = temp_dx;
		*arg1_offset = SUCCESS;}}
      else if(op_mode2 == T_COMMON_RIGHT)
	{size1 = symbol_table[*arg1_index].width;
	size2 = symbol_table[arg2_index].width;
	if(size1 > size2)
	  {temp_dx = get_symtab_temp(symbol_table[arg2_index].mode1,
										  T_PARALLEL,size1);
	    pc_itoa(size1,offset);
				num_dx = get_const(offset,T_DEC_CONST);
				print_opcode("mvsc_",T_MVSC,symbol_table[arg2_index].mode1,T_PARALLEL);
		print_args(off_exp1(arg2_index,arg2_offset), off_exp2(temp_dx,SUCCESS),
		       off_exp3(num_dx,SUCCESS),NULL,NULL);
		op_mode2 = T_PARALLEL;
		arg2_index = temp_dx;
		arg2_offset = SUCCESS;}}
      else if(op_mode2 == T_SCALAR)
/* this code converts scalar ints greater than HOST SIZE to parallel
   temps and converts the mode of the operation to parallel */
	{size1 = symbol_table[*arg1_index].width;
	 size2 = symbol_table[arg2_index].width;
			if(size2 > size1) size1 = size2;
	if( size1 > SIZE_HOST && op_mode1 == T_INT)
	  {pc_itoa(size1,offset);
				num_dx = get_const(offset,T_DEC_CONST);
				temp_dx = get_symtab_temp(symbol_table[*arg1_index].mode1,
										  T_PARALLEL,size1);
	    print_opcode("mvsc_",T_MVSC,symbol_table[*arg1_index].mode1,T_PARALLEL);
		print_args(off_exp1(*arg1_index,*arg1_offset),
							  off_exp2(temp_dx,SUCCESS),
		       off_exp3(num_dx,SUCCESS),NULL,NULL);
	    temp2_dx = get_symtab_temp(symbol_table[arg2_index].mode1,
										  T_PARALLEL,size1);
	    print_opcode("mvsc_",T_MVSC,symbol_table[arg2_index].mode1,T_PARALLEL);
		print_args(off_exp1(arg2_index,arg2_offset),
							  off_exp2(temp2_dx,SUCCESS),
		       off_exp3(num_dx,SUCCESS),NULL,NULL);
		op_mode2 = T_PARALLEL;
		*arg1_index = temp_dx;
		*arg1_offset = SUCCESS;
		arg2_index = temp2_dx;
		arg2_offset = SUCCESS;}}
/* end of coversion to parallel */
      result_mode1 = op_mode1;
      result_mode2 = get_result_mode2 (op_mode2);
      result_size = biggest_size(*arg1_index,arg2_index);
      if((arg3_index =
			 get_symtab_temp (result_mode1,result_mode2,result_size)) == FAIL)
		  {print_msg(TEMP_ERR, "In sumdi 1"); return(FAIL);}
      symbol_table[arg3_index].width = result_size;
/*      if((result_mode2 == T_SCALAR || result_mode2 == T_REG_SCALAR)
			  && result_size > 32)
		  printf("%d WARNING - scalar results greater than 32 bits not suported.\n",
					 linenum);    */
      if(op_mode2 == T_CNST) op_mode2 = T_SCALAR;
      if (op_type == T_ADD)
	     print_opcode ("add_1", T_ADD, op_mode1, op_mode2);
      else
	     print_opcode ("sub_", T_SUB, op_mode1, op_mode2);
      print_args(off_exp1(*arg1_index,*arg1_offset),
		      off_exp2(arg2_index,arg2_offset),
		     off_exp3(arg3_index,SUCCESS), NULL, NULL);
      *arg1_index = arg3_index;
      *arg1_offset= SUCCESS;
      more_sum_dif_exp(arg1_index,arg1_offset);
     }

  if (debug == 2)
     {printf ("More_sum_dif_exp() returning *arg1_index =");
      if(*arg1_index == FAIL) printf(" FAIL\n");
	 else printf("%s\n", symbol_table[*arg1_index].tokpts);}

  return(SUCCESS);

}


/* ===========================================================================

   function name: prod_quot_exp
   input               : ---
   output           : result_index, result_offset
		: result_index  - symbol table index of result, if no errors
						     - FAIL, otherwise

   global vars  : debug         - flag for debugging statements

   description   : This routine is part of a recursive descent parser and
			    handles the following production from the grammar:

			PROD_QUOT_EXP <-- ELEMENT MORE_PROD_QUOT_EXP

		  If an argument (resulting from a possible unary or paren-
		  thesized expression or consisting of an identifier, constant,
		  or defined constant) is obtained, it is passed to 'more_
		  prod_quot_exp' to perform a possible multiplication or
		  division. If the first argument cannot be obtained, FAIL is
		  returned.

=========================================================================== */

int prod_quot_exp(result_index,result_offset)
int *result_index, *result_offset;
{
   int foo;

   if (debug == 2)
      {printf ("In prod_quot_exp()...............................\n");
       printf("In prod_quote, token_buf=%s, token_type=%d\n",token_buf,token_type);}

   foo = element(result_index,result_offset);

if(debug == 2)
 if(*result_index != FAIL)
   printf("Element returning result_index=%s\n",symbol_table[*result_index].tokpts);

   if (foo == FAIL)
		 return(FAIL);
   if (more_prod_quot_exp(result_index,result_offset) == FAIL) return(FAIL);

   if (debug == 2)
      printf ("Prod_quot_exp() returning *result_index %d\n", *result_index);

   return(SUCCESS);
}


/* ===========================================================================

   function name: more_prod_quot_exp
   input               : arg1_index     - symbol table index of argument 1
   output           : result_index,result_offset
   return value : result_index  - symbol table index of result, if no errors
						     - FAIL, otherwise

   global vars  : debug         - flag for debugging statements
			    token_type  - type of current token
			    symbol_table        - array containing symbol table

   description   : This routine is part of a recursive descent parser and
			    handles the following production from the gammar:

			MORE_PROD_QUOT_EXP <-- * ELEMENT MORE_PROD_QUOT_EXP |
							 / ELEMENT MORE_PROD_QUOT_EXP |
					       ** ELEMENT MORE_PROB_QUOT_EXP |
							 @

		  If a multiply or divide sign is found, the symbol table index
		  of the second argument is obtained. Intermediate code is
		  generated and the symbol table index of the result is passed
		  in a recursive call to this function to handle any other
		  multiplications or divisions. The symbol table index of the
		  final result is returned. If no multiply or divide sign is
		  found, the function returns the first argument as it was
		  received. NOTE: Type-checking is done to ensure that the
		  operands for multiplication and division are type integer or
		  real--conversions are done between these two types if needed.

=========================================================================== */

int more_prod_quot_exp(arg1_index,arg1_offset)
int *arg1_index,*arg1_offset;
{
   int op_mode1;                /* operation mode: int or real */
   int op_mode2;                /* operation mode: scalar, parallel, common */
   int op_type;         /* operation type: multiply or divide */
   int result_mode1;    /* result mode: int or real */
   int result_mode2;    /* result mode: scalar or parallel */
   int result_size;  /* size of output field */
   int arg2_index;      /* symbol table index of second argument */
   int arg2_offset;
   int arg3_index;
   int size1, size2, temp_dx, temp2_dx, num_dx;
   char offset[10];

   if (debug == 2)
    {printf ("In more_prod_quot_exp()................................\n");
     printf ("token_type=%x, token=%s\n",token_type,token_buf); }

   if (token_type == T_MULT || token_type == T_DIV ||
       token_type == T_POWER || token_type == T_MOD)
     {if (symbol_table[*arg1_index].mode1 != T_REAL &&
	       symbol_table[*arg1_index].mode1 != T_OCT  &&
	       symbol_table[*arg1_index].mode1 != T_HEX  &&
	       symbol_table[*arg1_index].mode1 != T_BIN  &&
	       symbol_table[*arg1_index].mode1 != T_CARD &&
	       symbol_table[*arg1_index].mode1 != T_CHAR &&
	       symbol_table[*arg1_index].mode1 != T_LONGINT &&
	       symbol_table[*arg1_index].mode1 != T_INT)
	       {print_msg(EXPCON_ERR, token_buf);
		printf("In more_prod_quot 1; ");
		printf("arg1=%s, mode1=%d\n",symbol_table[*arg1_index].tokpts,
			symbol_table[*arg1_index].mode1);
	     return(FAIL); }
      op_type = token_type;
      token_type = get_token();
      if(element(&arg2_index,&arg2_offset) == FAIL) return(FAIL);
      if(symbol_table[arg2_index].mode1 != T_REAL &&
	      symbol_table[arg2_index].mode1 != T_OCT  &&
	      symbol_table[arg2_index].mode1 != T_HEX  &&
	      symbol_table[arg2_index].mode1 != T_BIN  &&
	      symbol_table[arg2_index].mode1 != T_CARD &&
	      symbol_table[arg2_index].mode1 != T_CHAR &&
	      symbol_table[arg2_index].mode1 != T_LONGINT &&
	      symbol_table[arg2_index].mode1 != T_INT)
	{print_msg (EXPCON_ERR, token_buf);
	      printf("In more_prod_quot 2; ");
printf("arg2=%s, mode1=%d\n",symbol_table[arg2_index].tokpts,
			     symbol_table[arg2_index].mode1);
	return(FAIL); }
	
	op_mode1 = get_op_mode1(arg1_index,arg1_offset,&arg2_index,&arg2_offset);
	op_mode2 = get_op_mode2 (*arg1_index, arg2_index);
	/* pow is T_REAL only - convert before mode adjustment */
	if(op_type == T_POWER)
	 if(op_mode1 == T_INT || op_mode1 == T_LONGINT)
	     {if(symbol_table[*arg1_index].mode1 != T_REAL)
		float_(arg1_index,arg1_offset);
	      if(symbol_table[arg2_index].mode1 != T_REAL)
		float_(&arg2_index,&arg2_offset);
	      op_mode1 = T_REAL;}
	      
	if(op_mode2 == T_COMMON_LEFT ||
/*         op_mode2 == T_SCALAR_LEFT || */
	 op_mode2 == T_REG_LEFT)
	{size1 = symbol_table[*arg1_index].width;
	size2 = symbol_table[arg2_index].width;
	if(size1 < size2)
	  {temp_dx = get_symtab_temp(symbol_table[*arg1_index].mode1,T_PARALLEL,size2);
	    pc_itoa(size2,offset);
				num_dx = get_const(offset,T_DEC_CONST);
				print_opcode("mvsc_",T_MVSC,symbol_table[*arg1_index].mode1,T_PARALLEL);
		print_args(off_exp1(*arg1_index,*arg1_offset), off_exp2(temp_dx,SUCCESS),
		       off_exp3(num_dx,SUCCESS),NULL,NULL);
		op_mode2 = T_PARALLEL;
		*arg1_index = temp_dx;
		*arg1_offset = SUCCESS;}}
      else if(op_mode2 == T_COMMON_RIGHT ||
/*              op_mode2 == T_SCALAR_RIGHT ||   */
	      op_mode2 == T_REG_RIGHT)
	{size1 = symbol_table[*arg1_index].width;
	size2 = symbol_table[arg2_index].width;
	if(size1 > size2)
	  {temp_dx = get_symtab_temp(symbol_table[arg2_index].mode1,
										  T_PARALLEL,size1);
	    pc_itoa(size1,offset);
				num_dx = get_const(offset,T_DEC_CONST);
				print_opcode("mvsc_",T_MVSC,symbol_table[arg2_index].mode1,T_PARALLEL);
		print_args(off_exp1(arg2_index,arg2_offset), off_exp2(temp_dx,SUCCESS),
		       off_exp3(num_dx,SUCCESS),
					  NULL,NULL);
		op_mode2 = T_PARALLEL;
		arg2_index = temp_dx;
		arg2_offset = SUCCESS;}}
      else if(op_mode2 == T_SCALAR)
/* this code converts scalar ints greater than HOST SIZE to parallel
   temps and converts the mode of the operation to parallel */
	{size1 = symbol_table[*arg1_index].width;
	 size2 = symbol_table[arg2_index].width;
			if(size2 > size1) size1 = size2;
	if( size1 > SIZE_HOST && op_mode1 == T_INT)
	  {pc_itoa(size1,offset);
				num_dx = get_const(offset,T_DEC_CONST);
				temp_dx = get_symtab_temp(symbol_table[*arg1_index].mode1,
										  T_PARALLEL,size1);
	    print_opcode("mvsc_",T_MVSC,symbol_table[*arg1_index].mode1,T_PARALLEL);
		print_args(off_exp1(*arg1_index,*arg1_offset),
							  off_exp2(temp_dx,SUCCESS),
		       off_exp3(num_dx,SUCCESS),NULL,NULL);
	    temp2_dx = get_symtab_temp(symbol_table[arg2_index].mode1,
										  T_PARALLEL,size1);
	    print_opcode("mvsc_",T_MVSC,symbol_table[arg2_index].mode1,T_PARALLEL);
		print_args(off_exp1(arg2_index,arg2_offset),
							  off_exp2(temp2_dx,SUCCESS),
		       off_exp3(num_dx,SUCCESS),NULL,NULL);
		op_mode2 = T_PARALLEL;
		*arg1_index = temp_dx;
		*arg1_offset = SUCCESS;
		arg2_index = temp2_dx;
		arg2_offset = SUCCESS;}}
/* end of coversion to parallel */
      result_mode1 = op_mode1; /* set default mode1 */
      result_mode2 = get_result_mode2 (op_mode2);
      /* if both scalar - same size as either                  */
      /* if both parallel - if T_MULT size is sum of sizes     */
      /*                       otherwise size is longest       */
      /* if one parallel -  if T_MULT size is parallel times two*/
      /*                       otherwise is parallel          */
      result_size = biggest_size(*arg1_index,arg2_index);
      /* Generate long size for T_PARALLEL, T_MULT & fixed    */
      if(op_mode1 == T_INT  || op_mode1 == T_LONGINT || op_mode1 == T_OCT ||
	      op_mode1 == T_CARD || op_mode1 == T_BIN     || op_mode1 == T_HEX   )
	     if (result_mode2 == T_PARALLEL && op_type == T_MULT)
	       if(symbol_table[*arg1_index].mode2==symbol_table[arg2_index].mode2)
	      result_size=symbol_table[*arg1_index].width+
			     symbol_table[arg2_index].width;
	  else if(symbol_table[*arg1_index].mode2==T_PARALLEL)
		 result_size=symbol_table[*arg1_index].width*2;
	  else
		 result_size=symbol_table[arg2_index].width*2;
	if((arg3_index =
				  get_symtab_temp (result_mode1,result_mode2,result_size)) == FAIL)
			  return(FAIL);
	symbol_table[arg3_index].width = result_size;
	if(result_mode1 == T_MULT)
	       {symbol_table[arg3_index].expt = symbol_table[arg2_index].expt;
	   symbol_table[arg3_index].signif = symbol_table[arg2_index].signif;}
	else
	       {symbol_table[arg3_index].expt = 0;
	   symbol_table[arg3_index].signif = result_size;}
	if((result_mode2 == T_SCALAR || result_mode2 == T_REG_SCALAR)
				&& result_mode1 != T_REAL && result_mode1 != T_LONGINT &&
				result_size > 2*SIZE_HOST)
			  printf("%d WARNING - mode1 %d results greater than 2*%d bits not suported. Variable=%s\n",
		  linenum,result_mode1,SIZE_HOST,symbol_table[arg3_index].tokpts);

	if(op_mode2 == T_CNST) op_mode2 = T_SCALAR;
	if (op_type == T_MULT)
	  print_opcode ("mul_", T_MULT, op_mode1, op_mode2);
	else if (op_type == T_DIV)
	  print_opcode ("div_", T_DIV, op_mode1, op_mode2);
        else if (op_type == T_MOD)
          print_opcode ("mod_", T_MOD, op_mode1, op_mode2);
	else if (op_type == T_POWER)
	  print_opcode ("pow_", T_POWER, op_mode1, op_mode2);
	else
	  printf("%d ERROR - unrecognized operator type = %x\n",linenum,op_type);
	print_args(off_exp1(*arg1_index,*arg1_offset),
		     off_exp2(arg2_index,arg2_offset),
			     off_exp3(arg3_index,SUCCESS),NULL, NULL);
	*arg1_index = arg3_index;
	*arg1_offset = SUCCESS;
      if(more_prod_quot_exp(arg1_index,arg1_offset) == FAIL) return(FAIL);
     }

   if (debug == 2)
     {printf ("More_prod_quot_exp() returning *arg1_index =");
      if(*arg1_index == FAIL) printf(" FAIL\n");
      else printf("%s\n", symbol_table[*arg1_index].tokpts);}

   return(SUCCESS);

}
